import click
from flask.cli import with_appcontext
from .config.flask_app_config import app
from .seed import gauge_type_seeder, add_units
from .appservices import database as db
from .reportseed import reports,expert_data_groups
from mdhwebserver import appconfigs
from mdhwebserver.models.models import SmtpConfig
from flask_migrate import stamp

from .models.models import (
    ExpertReport,
    ExpertDataItem,
    ExpertReportItem,
    Assembly,
    ExpertDataGroup
)

@app.cli.command("test")
def command_test():
    print("Testing")

def get_expert_data_item(name):
    edi = ExpertDataItem.query.filter(ExpertDataItem.name==name).first()
    return edi

def seed_expert_data_groups():
    for group in expert_data_groups:
        assemblyName = group['assembly']
        print("Assembly name: ", assemblyName)
        assembly = Assembly.query.filter(Assembly.name==assemblyName).first()
        edg = ExpertDataGroup.firstOrCreate(**{
            'name':group['name'],
            'display_name':group['displayName'],
            'assembly_id':assembly.id
        });
        print("Group: ",group);
        for dataItem in group['expertDataItems']:
            edg.expert_data_items.append(ExpertDataItem(**{
                'name':dataItem['name'],
                'display_name':dataItem['displayName']
            }))

def seed_expert_reports():
    for report in reports:
        print(report)
        r=ExpertReport.create(**{
            'name':report['name'],
            'display_name':report['display_name']
        })
        for item in report['items']:
            edi = get_expert_data_item(item['expert_field'])
            if not edi:
                print("No expert field: ",item['expert_field'])
                continue
            report_item = ExpertReportItem(**{
                'name':item['name'],
                'color':item['color'],
                'expression':item['expression']
            })
            for param in item['parameters']:
                t = get_expert_data_item(param)
                report_item.parameters.append(t)
            r.items.append(report_item)
        
@app.cli.command("report_seed")
def report_seed():
    print("Report Seeding")
    seed_expert_data_groups()
    seed_expert_reports()
    '''
    for report in reports:
        print(report)
        r=ExpertReport.create(**{
            'name':report['name'],
            'display_name':report['display_name']
        })
        for item in report['items']:
            edi = get_expert_data_item(item['expert_field'])
            if not edi:
                print("No expert field: ",item['expert_field'])
                continue
            report_item = ExpertReportItem(**{
                'name':item['name'],
                'color':item['color'],
                'expression':item['expression']
            })
            for param in item['parameters']:
                t = get_expert_data_item(param)
                report_item.parameters.append(t)
            r.items.append(report_item)
    '''
    db.session.commit()

@app.cli.command("db_seed")
def command_db_seed():
    print("Seeding")
    gauge_type_seeder()
    # seed_subassembly() 
    # seed_expert_data_groups()
    # seed_expert_reports()
    db.session.commit()


from .seed import seed_mro_issues

@app.cli.command("seed_mroissues")
def command_mroissue_seed():
    seed_mro_issues()

from .seeder import load_seeds

@app.cli.command("db_seed_files")
def command_db_seed_files():
    print("db_seed_files")
    load_seeds()

@app.cli.command("init_db",short_help="Initialize the database")
#@click.help_option("Help message")
def command_db_init():
    print("DB Init")
    db.create_all()


@app.cli.command("db_drop")
def command_db_drop():
    print("DB drop")
    db.drop_all()

from .database.db_config import (
    create_admin_user
)

from sqlalchemy import text
from mdhwebserver.models.SiUnitModels import UnitTranslate
from mdhwebserver.models.models import PmFunction



import os
import simplejson as json

def seed_unit_translations():
    ut = UnitTranslate().query.limit(1).first()
    if ut:
        return
    module_dir = os.path.join(os.path.dirname(__file__),"seedfiles/unit_system")
    print("module_dir: ",module_dir)

    for module in os.listdir(module_dir):
        modpath=module_dir + "/" + module
        if os.path.isdir(modpath):
            continue
        modname=os.path.basename(module).split('.')[0]
        extension = os.path.basename(module).split('.')[1]
        if extension != 'json':
            continue
        print("Loading seed file: ",modname)
        print("----------- Extension: ", extension)
        with open(modpath) as f:
            data = json.load(f)

        add_units(data)

    db.session.commit()
    

@app.cli.command("db_rebuild")
@click.pass_context
def command_db_rebuild(ctx):
    print("db_rebuild")
    ctx.invoke(command_db_drop)
    ctx.invoke(command_db_init)
    ctx.invoke(command_db_seed)
    ctx.invoke(command_db_seed_files)
    ctx.invoke(command_create_admin)
    ctx.invoke(command_smtp_init)
    seed_expert_data_groups()
    seed_expert_reports()

    stamp()
    seed_unit_translations()

@app.cli.command("create_admin")
def command_create_admin():
   create_admin_user(app) 

@app.cli.command("smtp_init")
def command_smtp_init():
    smtp = appconfigs.smtp
    print("Init smtp",smtp)
    if SmtpConfig.query.first():
        print("SMTP already created")
    else:
        x = SmtpConfig.create(**smtp.toDict())
        db.session.commit()

@app.cli.command("models")
def command_moddels():
    for mapper in db.Model.registry.mappers:
        print(f'{mapper.class_.__name__}')
        for relationship in mapper.relationships:
            print(f'\t{relationship} -- {relationship.mapper.class_.__name__}')
        print("")


'''
from .flask_commands.test_commands import bp


#app.register_blueprint(bp,cli_group='db')
app.register_blueprint(bp,cli_group=None)
'''
