

from datetime import datetime, timedelta,date
from mdhwebserver.modules.pumpsense.processor import add_kpis
from mdhwebserver.models.models import PmFunction, Gauge, DeviceDataItem, ETLTracker, DeviceDataItemUnitDataPointAssociation
from mdhwebserver.appservices import database as db
from sqlalchemy.sql import text
from sqlalchemy.orm import selectinload, joinedload

sum_etls = ["etl_hourly","etl_daily","etl_weekly","etl_yearly"]
avg_etls = ["etl_avg_hourly","etl_avg_daily","etl_avg_daily","etl_avg_weekly","etl_avg_monthly","etl_avg_yearly"]

gauge_styles = {
    "etl_hourly": "Hourly",
    "etl_daily" : "Daily",
    "etl_weekly" : "Weekly",
    "etl_monthly" : "Monthly",
    "etl_yearly" : "Yearly",
    "etl_avg_hourly": "Avg Hourly",
    "etl_avg_daily" : "Avg Daily",
    "etl_avg_weekly" : "Avg Weekly",
    "etl_avg_monthly" : "Avg Monthly",
    "etl_avg_yearly" : "Avg Yearly"
}

def get_etl_type(etl_name):
    if not etl_name:
        return None
    if etl_name in sum_etls:
        return "sum"
    if etl_name in avg_etls:
        return "avg"
    return None

def get_etl_append_string(etl_name):
    return gauge_styles.get(etl_name,"")

def create_etl_gauge(pmf,name,gauge_style):
    category_name = gauge_styles.get(gauge_style)
    if category_name == None:
        return None
    new_pm_function = PmFunction.firstOrCreate2({'name': f"{name} {category_name}",'gauge_style': gauge_style,'etl_pm_function_id' : pmf.id},
        {
        'display_name': f"{name} {category_name}",
        'subassembly_instance_id': pmf.subassembly_instance_id,
        'gauge_type' : 'value',
        'units': pmf.units,
        'unit_system': pmf.unit_system,
        'x_unit': pmf.x_unit,
        'y_unit': pmf.y_unit,
        'y_trend_min' : pmf.y_trend_min,
        'y_trend_max': pmf.y_trend_max,
        'gauge_group_id' : pmf.gauge_group_id,
        'trend_existence': True
    })
    
    return new_pm_function

def create_sum_etl_gauges(pmf,name):
    end_time=datetime.now()
    start_time = end_time - timedelta(days=100)

    hourly_gauge = create_etl_gauge(pmf,name,'etl_hourly')
    daily_gauge = create_etl_gauge(pmf,name,'etl_daily')
    weekly_gauge = create_etl_gauge(hourly_gauge,name,'etl_weekly')
    monthly_gauge = create_etl_gauge(daily_gauge,name,'etl_monthly')
    yearly_gauge = create_etl_gauge(daily_gauge,name,'etl_yearly')

    # Need to actuall have tasks for these.

    sum_etl_on_pm_functions([hourly_gauge],start_time,end_time,"hour","hourly")
    sum_etl_on_pm_functions([daily_gauge],start_time,end_time,"day","daily")
    sum_etl_on_pm_functions([weekly_gauge],start_time,end_time,"week","weekly")
    sum_etl_on_pm_functions([monthly_gauge],start_time,end_time,"month","monthly")
    sum_etl_on_pm_functions([yearly_gauge],start_time,end_time,"year","yearly")


def create_avg_etl_gauges(pmf,name):
    end_time=datetime.now()
    start_time = end_time - timedelta(days=100)

    hourly_gauge = create_etl_gauge(pmf,name,'etl_avg_hourly')
    daily_gauge = create_etl_gauge(pmf,name,'etl_avg_daily')
    weekly_gauge = create_etl_gauge(hourly_gauge,name,'etl_avg_weekly')
    monthly_gauge = create_etl_gauge(daily_gauge,name,'etl_avg_monthly')
    yearly_gauge = create_etl_gauge(daily_gauge,name,'etl_avg_yearly')

    # Need to actuall have tasks for these.

    avg_etl_on_pm_functions([hourly_gauge],start_time,end_time,"hour","hourly")
    avg_etl_on_pm_functions([daily_gauge],start_time,end_time,"day","daily")
    avg_etl_on_pm_functions([weekly_gauge],start_time,end_time,"week","weekly")
    avg_etl_on_pm_functions([monthly_gauge],start_time,end_time,"month","monthly")
    avg_etl_on_pm_functions([yearly_gauge],start_time,end_time,"year","yearly")

def update_etl_trends(base_time,trend_meta_data_ids,pmf_etl_dict,result,prefix_name=""):
    record = result.fetchone()
    data_list=[]
    gauge_ids=set()
    date_groups=set()

    while (record):
        kpis = {f'{prefix_name}_{record[4]}': record[2]}
        d = {
            'kpis': kpis,
            'gauge_id': record[0],
            'pm_function_id': record[1],
            'trend_meta_data_id' : record[3],
            'pm_function_name': record[5],
            'date_group': record[6]
        }
        gauge_ids.add(record[0])
        data_list.append(d)
        date_groups.add(record[6])
        record = result.fetchone()
    if gauge_ids and trend_meta_data_ids:
        gauge_ids_csv = ",".join(str(s) for s in gauge_ids)
        trend_meta_data_ids_csv = ",".join(str(s) for s in trend_meta_data_ids)
        date_groups_csv = ",".join(f"'{str(s)}'" for s in date_groups)
        q=f"delete from device_unit_data_points where gauge_id in ({gauge_ids_csv}) and trend_meta_data_id in ({trend_meta_data_ids_csv}) and timestamp in ({date_groups_csv})"
        print("Delete Query: ",q)
        result = db.session.execute(text(q))
    for data_item in data_list:
        PmFunc = pmf_etl_dict[str(data_item['pm_function_id'])]
        device_data = DeviceDataItem.where(gauge_id=data_item['gauge_id'],pm_func_id=PmFunc.id).first()
        G = Gauge.where(id=data_item['gauge_id']).first()
        # GaugeGroup = get_gauge_group_function(PmFunc)
        GaugeGroup = None
        
        if GaugeGroup is None:
            ggid = None
            ggname = None
        else:
            ggid = GaugeGroup.id
            ggname = GaugeGroup.name
        if device_data is None:
            device_data = DeviceDataItem(**{
                # 'data': {},
                'gauge_id': G.id,
                # 'pm_func_id': PmFunc.id,
                'pm_function': PmFunc,
                'pm_function_name': PmFunc.name,
                'subassembly_instance_id': PmFunc.subassembly_instance_id,
                'subassembly_instance_name': PmFunc.subassembly_instance_name,
                'gauge_group_id': ggid,
                'gauge_group_name': ggname,
                'colors': ['green'],
                'values': None,
                'reason': None,
                'activate_hourglass': False,
                'feedback': False,
                'user_input': False,
                'hide_from_group': False,
                'message': None,
                'yellow_threshold': None,
                'red_threshold': None,
                'kpis':data_item['kpis'],
                'color': None
            })
            G.device_data_items.append(device_data)
        else:
            device_data.fill(**{
                'colors': ['green'],
                'values': None,
                'reason': None,
                'activate_hourglass': False,
                'feedback': None,
                'user_input': False,
                'hide_from_group': False,
                'message': None,
                'yellow_threshold': None,
                'red_threshold': None,
                'kpis':data_item['kpis'],
                # 'color': max_gauge_color
            })
        add_kpis(G,PmFunc,data_item['kpis'],data_item['date_group'],PmFunc.units)

def sum_etl_on_pm_functions(PmFuncs,start_time, end_time, d_trunc, prefix_name=""):
    if not PmFuncs:
        return
    trend_meta_data_ids=set()
    pm_fun_etl_ids = set()
    pmf_etl_dict = {}
    for pmf in PmFuncs:
        pm_fun_etl_ids.add(pmf.etl_pm_function_id)
        pmf_etl_dict[str(pmf.etl_pm_function_id)] = pmf
        for tmd in pmf.trend_meta_data:
            trend_meta_data_ids.add(tmd.id)
    # pm_function_id=PmFunc.etl_pm_function_id 
    pmf_etl_ids_csv = ",".join(str(s) for s in pm_fun_etl_ids)
    q=f"select dp.gauge_id as gauge_id,pmf.id as pm_function_id, unit_at_double(sum(value),tmd.units) as etl_value, \
        tmd.id as trend_meta_data_id, tmd.name, pmf.name,date_trunc('{d_trunc}',dp.timestamp) as date_group \
        from pm_functions pmf \
        join trend_meta_data tmd on pmf.id = tmd.pm_func_id \
        join device_unit_data_points dp on dp.trend_meta_data_id = tmd.id \
        where pmf.id in ({pmf_etl_ids_csv}) and dp.timestamp >= '{start_time}' and dp.timestamp < '{end_time}' \
        group by gauge_id, pmf.id,tmd.id, tmd.name,date_trunc('{d_trunc}',dp.timestamp) order by date_trunc('{d_trunc}',dp.timestamp)"

    print("Query: ", q)
    result = db.session.execute(text(q))
    update_etl_trends(end_time,trend_meta_data_ids,pmf_etl_dict,result,prefix_name=prefix_name)
    sync_device_data_items_trend_metas(pmf)    


def avg_etl_on_pm_functions(PmFuncs,start_time, end_time, d_trunc, prefix_name=""):
    if not PmFuncs:
        return
    trend_meta_data_ids=set()
    pm_fun_etl_ids = set()
    pmf_etl_dict = {}
    for pmf in PmFuncs:
        pm_fun_etl_ids.add(pmf.etl_pm_function_id)
        pmf_etl_dict[str(pmf.etl_pm_function_id)] = pmf
        for tmd in pmf.trend_meta_data:
            trend_meta_data_ids.add(tmd.id)
    # pm_function_id=PmFunc.etl_pm_function_id 
    pmf_etl_ids_csv = ",".join(str(s) for s in pm_fun_etl_ids)
    q=f"select dp.gauge_id as gauge_id,pmf.id as pm_function_id, unit_at_double(avg(value),tmd.units) as etl_value, \
        tmd.id as trend_meta_data_id, tmd.name, pmf.name,date_trunc('{d_trunc}',dp.timestamp) as date_group \
        from pm_functions pmf \
        join trend_meta_data tmd on pmf.id = tmd.pm_func_id \
        join device_unit_data_points dp on dp.trend_meta_data_id = tmd.id \
        where pmf.id in ({pmf_etl_ids_csv}) and dp.timestamp >= '{start_time}' and dp.timestamp < '{end_time}' \
        group by gauge_id, pmf.id,tmd.id, tmd.name,date_trunc('{d_trunc}',dp.timestamp) order by date_trunc('{d_trunc}',dp.timestamp)"

    print("Query: ", q)
    result = db.session.execute(text(q))
    update_etl_trends(end_time,trend_meta_data_ids,pmf_etl_dict,result,prefix_name=prefix_name)
    sync_device_data_items_trend_metas(pmf)

def sync_device_data_items_trend_metas(pmf):
    for ddi in pmf.device_data_items:
        for tm in pmf.trend_meta_data:
            if tm not in ddi.trend_meta_datas:
                ddi.device_unit_data_point_associations.append(DeviceDataItemUnitDataPointAssociation(**{
                    "gauge_id": ddi.gauge_id,
                    "trend_meta_data_id": tm.id
                }))

def sum_etl(start_time, end_time,gauge_style, d_trunc, prefix_name=""):
    PmFuncs = PmFunction.where(gauge_style=gauge_style).options(selectinload(PmFunction.trend_meta_data)).all()
    sum_etl_on_pm_functions(PmFuncs,start_time, end_time, d_trunc, prefix_name=prefix_name)

def avg_etl(start_time, end_time,gauge_style, d_trunc, prefix_name=""):
    PmFuncs = PmFunction.where(gauge_style=gauge_style).options(selectinload(PmFunction.trend_meta_data)).all()
    avg_etl_on_pm_functions(PmFuncs,start_time, end_time, d_trunc, prefix_name=prefix_name)


def get_etl_tracker(etl_category):
    etl_item = ETLTracker.where(etl_category=etl_category).first()
    if etl_item == None:
        etl_item = ETLTracker().create(etl_category=etl_category)
    return etl_item

# def set_etl_tracker_last_run(etl_item,start_time, end_time):
#     etl_item.fill(**{
#         'last_run': datetime.now(),
#         'start_time': start_time,
#         'end_time' : end_time
#     })


def run_etl(base_time, end_time, delta_time, etl_category, d_trunc,prefix_name='',etl_fun=sum_etl):
    print("HOURLY ETL: ")
    etl_item = get_etl_tracker(etl_category)
    print("etl_item: ", etl_item.start_time, etl_item.end_time)
    # end_time = datetime.now().replace(microsecond=0, second=0, minute=0)
    # end_time = datetime.now()
    # base_time = end_time.replace(microsecond=0, second=0, minute=0)
    if etl_item.start_time is None:
        start_time = base_time - timedelta(days=91)
        print("No start time doing 91 days: ", start_time)
    elif etl_item.end_time > base_time:
        start_time = base_time
    else:
        start_time = base_time - delta_time

    print(f"start: {start_time} - {end_time}")
    # sum_etl(start_time,end_time,'etl_hourly',"hour",prefix_name='hourly')
    etl_fun(start_time,datetime.now(),etl_category,d_trunc,prefix_name=prefix_name)
    etl_item.set_last_run(start_time,end_time)
    db.session.commit()

def hourly_etls():
    print("HOURLY ETL: ")
    end_time = datetime.now()
    base_time = end_time.replace(microsecond=0, second=0, minute=0)
    run_etl(base_time,end_time,timedelta(hours=1),"etl_hourly","hour","hourly",etl_fun=sum_etl)
    run_etl(base_time,end_time,timedelta(hours=1),"etl_avg_hourly","hour","hourly",etl_fun=avg_etl)


def weekly_etls():
    print("WEEKLY ETL: ")
    today = datetime.combine(date.today(), datetime.min.time())
    end_time = datetime.now()

    base_time = today - timedelta(days=today.weekday())
    run_etl(base_time,end_time,timedelta(days=6),"etl_weekly","week","weekly",etl_fun=sum_etl)
    run_etl(base_time,end_time,timedelta(days=6),"etl_avg_weekly","week","weekly",etl_fun=avg_etl)


def daily_etls():
    print("Daily ETL: ")
    today = datetime.combine(date.today(), datetime.min.time())
    end_time = datetime.now()
    base_time = today
    run_etl(base_time,end_time,timedelta(days=1),"etl_daily","day","daily",etl_fun=sum_etl)
    run_etl(base_time,end_time,timedelta(days=1),"etl_avg_daily","day","daily",etl_fun=avg_etl)

def monthly_etls():
    print("Monthly ETL: ")
    today = datetime.combine(date.today(), datetime.min.time())
    end_time = datetime.now()
    base_time = today.replace(day=1)
    run_etl(base_time,end_time,relativedelta(months=1),"etl_monthly","month","monthly",etl_fun=sum_etl)
    run_etl(base_time,end_time,relativedelta(months=1),"etl_avg_monthly","month","monthly",etl_fun=avg_etl)

from dateutil.relativedelta import relativedelta

def yearly_etls():
    print("Yearly ETL: ")
    today = datetime.combine(date.today(), datetime.min.time())
    end_time = datetime.now()
    base_time = today.replace(day=1,month=1)
    run_etl(base_time,end_time,relativedelta(years=1),"etl_yearly","year","yearly",etl_fun=sum_etl)
    run_etl(base_time,end_time,relativedelta(years=1),"etl_avg_yearly","year","yearly",etl_fun=avg_etl)
