import logging

from sqlalchemy.exc import (
    SQLAlchemyError,
    NoResultFound
)

#from ..database.db_config import db
from mdhwebserver.appservices import database as db

from ..models.models import User

class UserService:
    logger = logging.getLogger('UserService')

    def get_by_email(self, email, is_active=True):
        try:
            filters = {
                'email': email
            }
            return User.query.filter_by(**filters).one()

        except NoResultFound as e:
            self.logger.error("exception occurred", exc_info=True)
            return None

        except SQLAlchemyError as e:
            self.logger.error("exception occurred", exc_info=True)
            raise e
    
    
    def save(self, new_user: User):
        try:
            db.session.add(new_user)
            db.session.commit()
        
        except SQLAlchemyError as e:
            self.logger.error("exception occurred", exc_info=True)
            raise e
    
    def update_authentication(self, new_user: User):
        try:
            rows_changed = User.query.filter(
                                User.email == new_user.email
                           ).update({
                                User.authenticated: new_user.authenticated
                           })
            
            if rows_changed == 1:
                db.session.commit()
                self.logger.debug(
                    f'user updated successfully - Rows updated: {rows_changed}')

        except SQLAlchemyError as e:
            self.logger.error("exception occurred", exc_info=True)
            raise