import operator

from sqlalchemy.orm.collections import MappedCollection, collection


class KeyedListCollection(MappedCollection):
    """This transforms a collection of attribute values into a dict of lists.
    
    This is useful when representing an EAV table, for instance, because
    any given attribute can have multiple values, but it is typically
    most useful to access them by attribute name.
    """
    def __init__(self, key):
        super(KeyedListCollection, self).__init__(operator.attrgetter(key))
    
    @collection.internally_instrumented
    def __setitem__(self, key, value, _sa_initiator=None):
        print(f"******* KeyedListCollection set item {key} ****")
        if not super(KeyedListCollection, self).get(key):
            super(KeyedListCollection, self).__setitem__(key, [], _sa_initiator)
        super(KeyedListCollection, self).__getitem__(key).append(value)

        
class KeyedDictCollection(MappedCollection):
    """This transforms a collection of attribute values into a dict of lists.
    
    This is useful when representing an EAV table, for instance, because
    any given attribute can have multiple values, but it is typically
    most useful to access them by attribute name.
    """
    def __init__(self, key, key2=None):
        super(KeyedDictCollection, self).__init__(operator.attrgetter(key))
        self.__key2__ = key2
        
    @collection.internally_instrumented
    def __setitem__(self, key, value, _sa_initiator=None):
        print(f"******* KeyedDictCollection set item --- {key} -- {self.__key2__} ****")
        if not super(KeyedDictCollection, self).get(key):
            super(KeyedDictCollection, self).__setitem__(key, [], _sa_initiator)
        #super(KeyedDictCollection, self).__getitem__(key).[self.__key2__]=value
        #super(KeyedDictCollection, self).__getitem__(key).append(value)

#       item = {self.__key2__:value}
#        super(KeyedDictCollection, self).__getitem__(key).append(item)
        super(MappedCollection, self).__setitem__(key,value,_sa_initiator)

