from sqlalchemy.orm import class_mapper
from sqlalchemy import inspect
from collections.abc import Iterable
from stringcase import camelcase

def find_key_in_list(key,l):
    #print("Finding key: ",key)
    if l is None:
        return None
    for item in l:
        if (isinstance(item,dict)):
            r = item.get(key)
            if r is not None:
                return r
    return None


def list_in_filter(source_list,filter_list):
        if filter_list is not None:
            source_list = filter(lambda e: e in filter_list, source_list)
        return source_list

def list_not_in_filter(source_list,filter_list):
        if filter_list is not None:
            source_list = filter(lambda e: e not in filter_list, source_list)
        return source_list

def list_filter(source_list,exclude,only):
    source_list = list_not_in_filter(source_list,exclude)
    source_list = list_in_filter(source_list,only)
    return source_list

def convert_mapped_collection(obj,name,**pkwargs):
    #print("Convert Mapped")
    convert={}
    #print(f'Child ({name}) is string')
    #print("Obj type is: ",type(obj));
    #print("Obj is: ",obj)
    if isinstance(obj,list):
        #print("**************  Child is list don't know what to do  ****************8")
        convert[name] = [o._to_dict(**pkwargs) for o in obj]
    elif isinstance(obj,dict):
        #print("**************  Child is dict don't know what to do  ****************8")
        convert[name] = [convert_mapped_collection(o,key,**pkwargs) for key,o in obj.items()]

    else:
        convert[name] = obj._to_dict(**pkwargs)
    return convert


class SQLAToDictMixin:

    def add_serialized_attributes(self,serialized_data, attr_list,camelKeys=False):
        for c in attr_list:
#            #print("self name: ",self.__class__.__name__)
            colName = c if not camelKeys else camelcase(c)
            serialized_data[colName] = getattr(self, c)

#    def _to_dict(self,visited_children=None, back_relationships=None,exclude=None, only=None, camelKeys=False,includeHybrid=False):
    def _to_dict(self,**kwargs):
        # print("-------------------   To Dict STarting ----------------------- : ")
        # print("self name: ",self.__class__.__name__)
#        #print("Only is: ",only)
        #Unpack args
        visited_children = kwargs.get('visited_children',None)
        back_relationships = kwargs.get('back_relationships',None)
        exclude = kwargs.get('exclude',None)
        only = kwargs.get('only',None)
        camelKeys = kwargs.get('camelKeys',None)
        includeHybrid = kwargs.get('includeHybrid',None)
        hidden = self._hidden if hasattr(self, '_hidden') else None

        if visited_children is None:
            visited_children = set()
        if back_relationships is None:
            back_relationships = set()
        #serialized_data = {c.key: getattr(self, c.key) for c in self.__table__.columns}
        serialized_data = {}
        obj_ins = inspect(self)
#        for c in self.__table__.columns:

        if bool(hidden):
            if not bool(exclude):
                exclude = []
            exclude = exclude + hidden

        #print("Exclude is: ",exclude)
        view_columns = list_filter(self.columns,exclude,only)
#        for c in view_columns:
##            print("self name: ",self.__class__.__name__)
#            colName = c if not camelKeys else camelcase(c)
#            serialized_data[colName] = getattr(self, c)

        self.add_serialized_attributes(serialized_data,view_columns,camelKeys)
        if includeHybrid:
            hkeys = list_filter(self.hybrid_properties,exclude,only)
            self.add_serialized_attributes(serialized_data,hkeys,camelKeys)
#            for key in hkeys:
#                colName = key if not camelKeys else camelcase(key)
#                serialized_data[colName]  = getattr(self, key)

        relationships = class_mapper(self.__class__).relationships
        visitable_relationships = [(name, rel) for name, rel in relationships.items() if name not in back_relationships]
        ins = inspect(self)
        for name, relation in visitable_relationships:
            #print("Relationship: ",name)
            #print("Relation: ",relation)
            if relation.backref:
                # print("In backref")
                if isinstance(relation.backref,tuple):
                    bk_name = relation.backref[0]
                else:
                    bk_name = relation.backref
                #print("Adding backref: ",bk_name)
                back_relationships.add(bk_name)
#            print("After if")
#            print(ins.unloaded)
#            print("self name: ", type(self).__name__)
#            print("With: ",self.__with__)
#            print("Only type is: ",type(only))
#            print("Only is: ",only)
#            r_only = find_key_in_list(name,only)
#            print("r_only is: ",r_only)
            #print("ins.unloaded: ",ins.unloaded)
# Test if relationship is in only...
            if name in ins.unloaded:
                # print("Not loaded: ", name)
                if hasattr(self,"__with__") and name in self.__with__:
                    # print("Force loading relationship: ", name)
                    relationship_children = getattr(self, name)
                else:
                    #print("Setting relationship to none")
                    relationship_children = None
            else:
                # print(f"Already loaded: {name}")
                relationship_children = getattr(self, name)
            #print("After getting children")
            if relationship_children is None:
                #print(f'-------------- No relationship ({name}) need to continue')
                continue
            if relationship_children is not None:
                #print(f'******** Testing relationship({name}) is in only **************')
                if exclude is not None:
                    #print("Have exclusions")
                    if name in exclude:     # if relationship is a key then we want to exclude subfields
                        continue
                if only is not None :
                    #print("Only is not none: ",only)
                    r_only = find_key_in_list(name,only)
                    if name in only or r_only:
                        pass
                    else:
                        #print("Need to exclude")
                        continue
                else:
                    r_only = None
                r_exclude = find_key_in_list(name,exclude)
                pkwargs = kwargs;
                pkwargs['only'] = r_only
                pkwargs['exclude'] = r_exclude
                pkwargs['back_relationships'] = back_relationships.copy() 
                pkwargs['visited_children'] = visited_children.copy() 
                colName = name if not camelKeys else camelcase(name)
                # print("relation.uselist: ",relation.uselist)
                if relation.uselist:
                    children = []
                    # print("Visited_children: ", visited_children)
                    # print("relationship_children: ", relationship_children)

                    # for child in [c for c in relationship_children if c not in visited_children]:
                    for child in relationship_children:

                        # print("Child is: ",child)
                        if isinstance(child,str) or isinstance(child,int):   # Must be a mapped relationship
                            '''
                            convert={}
                            #print(f'Child ({child}) is string')
                            #print("Relations ship type is: ",type(relation))
                            if isinstance(relationship_children[child],list) or isinstance(relationship_children[child],dict):
                                #print("**************  Child is list don't know what to do  ****************8")
                                convert[child] = [o._to_dict(**pkwargs) for o in relationship_children[child]]
                                 
                            else:
                                convert[child] = relationship_children[child]._to_dict(**pkwargs)
                            '''
                            convert = convert_mapped_collection(relationship_children[child],child,**pkwargs)    
                        elif isinstance(child,list):
                            #print("******************  Have a list don't know what to do  **********************")
                            pass
                        else:
                            convert = child._to_dict(**pkwargs)
                        visited_children.add(child)
                        #children.append(self._to_dict(child, visited_children, back_relationships))
#                        children.append(child._to_dict(visited_children=visited_children, back_relationships=back_relationships,only=r_only,camelKeys=camelKeys,includeHybrid=includeHybrid))
                        children.append(convert)
                    #print("Adding childern to key ",colName)
                    serialized_data[colName] = children
                else:
                    serialized_data[colName] = relationship_children._to_dict(**pkwargs)
        return serialized_data

    def _to_dict2(self,obj, visited_children=None, back_relationships=None):
        if visited_children is None:
            visited_children = set()
        if back_relationships is None:
            back_relationships = set()
        #serialized_data = {c.key: getattr(obj, c.key) for c in obj.__table__.columns}
        serialized_data = {}
        obj_ins = inspect(obj)
#        for c in obj.__table__.columns:
        for c in obj.columns:
#            print("obj name: ",obj.__class__.__name__)
            if obj.__class__.__name__ == "Gauge":
                print("-------- Column name: ",c)
            attr = getattr(obj, c)
            serialized_data[c] = attr
        relationships = class_mapper(obj.__class__).relationships
        visitable_relationships = [(name, rel) for name, rel in relationships.items() if name not in back_relationships]
        for name, relation in visitable_relationships:
#            print("Relation ship: ",name)
            if relation.backref:
                back_relationships.add(relation.backref)
#            print("After if")
            ins = inspect(obj)
#            print(ins.unloaded)
#            print("obj name: ", type(obj).__name__)
#            print("With: ",obj.__with__)
            if name in ins.unloaded:
                if hasattr(obj,"__with__") and name in obj.__with__:
                    relationship_children = getattr(obj, name)
                else:
                    relationship_children = None

#           if name == "configs":
#               relationship_children = None
#               ins = inspect(obj)
#               print(ins.unloaded)
            else:
                relationship_children = getattr(obj, name)
#            print("After getting children")
            if relationship_children is not None:
                if relation.uselist:
                    children = []
                    for child in [c for c in relationship_children if c not in visited_children]:
#                        print("Child is: ",child)
                        visited_children.add(child)
                        children.append(self._to_dict(child, visited_children, back_relationships))
                        serialized_data[name] = children
                else:
                    serialized_data[name] = self._to_dict(relationship_children, visited_children, back_relationships)
        return serialized_data

    def _with(self,list):
        self.__with__ = list
        return self

    def toJson(self):
        return self._to_dict()



