import datetime
from ..module import bp
from mdhwebserver.models.models import User
from os import getenv as env

import json

import logging
from cerberus import Validator
from mdhwebserver.appservices import database as db

from http import HTTPStatus
from flask import (
    request,
    redirect,
    abort
)
from uuid import uuid4
import urllib.parse

# from flask_login import (
#     current_user,
#     login_user, 
#     logout_user,
#     login_required,
#     LoginManager
# )
from datetime import datetime, timedelta, timezone
from flask_jwt_extended import create_access_token, get_jwt_identity, JWTManager, current_user, get_jwt, set_access_cookies, unset_jwt_cookies
from mdhwebserver.wrappers.is_authorized import is_authorized

from mdhwebserver.builders.responses import JsonResponse
from mdhwebserver.builders.response_builder import SQLAlchemyResponseBuilder, MDHResponseBuilder
from mdhwebserver.builders import ValidationErrorResponseBuilder
from mdhwebserver.exceptions import ValidationException
from mdhwebserver.utils.listutils import dict_extract
from mdhwebserver import application

from sqlalchemy.exc import IntegrityError

jwt_expire_time = timedelta(minutes=180)

logger = logging.getLogger('auth')
# login_manager = LoginManager(application.app)
jwt = JWTManager(application.app)

# @login_manager.user_loader
# def user_loader(user_id):
#     """Given *user_id*, return the associated User object.

#     :param unicode user_id: user_id (email) user to retrieve

#     """
#     print(f"Auth user loader: {user_id}")
#     user = User.where(email=user_id).first()
#     return user

@jwt.user_identity_loader
def user_identity_lookup(user):
    # print(f"Auth user identity loader: {user}")
    return user.email

@jwt.user_lookup_loader
def user_lookup_callback(_jwt_header, jwt_data):
    # print(f"Auth user lookup callback: {_jwt_header} - {jwt_data}")
    email = jwt_data["sub"]
    return User.where(email=email).first()

@application.app.after_request
def refresh_expiring_jwts(response):
    try:
        exp_timestamp = get_jwt()["exp"]
        now = datetime.now(timezone.utc)
        target_timestamp = datetime.timestamp(now + timedelta(minutes=30))
        if target_timestamp > exp_timestamp:
            email = get_jwt_identity()
            user = User.where(email=email).first()
            if user.authenticated:
                access_token = create_access_token(identity=user, expires_delta=jwt_expire_time)
                set_access_cookies(response, access_token)
        return response
    except (RuntimeError, KeyError):
        # Case where there is not a valid JWT. Just return the original response
        return response
    
@bp.route('/login', methods=['POST'])
def login():
    payload = request.get_json(force=True)
    v = Validator({
        'email':{'required': True,'type': 'string','regex': '^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$'},
        'password':{'required': True,'type': 'string'}
    },allow_unknown=True)
    if not v.validate(payload):
        return JsonResponse(ValidationErrorResponseBuilder(v))

    user = User.where(email=payload.get('email').lower()).first()
    if user and user.check_password(payload['password']):
        if user.active:
            user.authenticated = True
            # login_user(user, remember=True)
            db.session.commit()
            next = request.args.get('next',None)
            access_token = create_access_token(identity=user, expires_delta=jwt_expire_time)
            if next:
                return redirect(next)
            response = JsonResponse(SQLAlchemyResponseBuilder(user).camel(False))
            set_access_cookies(response, access_token)
            return response
    abort(HTTPStatus.FORBIDDEN.value, 'Invalid Login')


@bp.route('/logout', methods=['GET'])
@is_authorized()
def logout():
    """Logout the current user."""
    user = current_user._get_current_object()
    user.authenticated = False
    db.session.commit()
    # logout_user()
    response = JsonResponse(status=HTTPStatus.OK.value)
    unset_jwt_cookies(response)
    return response
    

@bp.route('/user', methods=['GET'])
@is_authorized()
def get_user():
    user = current_user._get_current_object()
    if user:
        return JsonResponse(SQLAlchemyResponseBuilder(user).camel(False))
    return JsonResponse(status=HTTPStatus.FORBIDDEN.value)


@bp.route('/user', methods=['POST'])
@is_authorized()
def add_user():
    v = Validator({
        'name': {'required':True, 'type': 'string'},
        'email': {'required':True, 'type': 'string','regex': '^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$'},
        'password': {'required':True, 'type': 'string','minlength':8},
        'status': {'required':True, 'type': 'string'},
        'is_admin': {'required':True, 'type': 'boolean'}
    }, allow_unknown=True)

    payload = request.get_json(force=True)
    if not v.validate(payload):
        raise ValidationException(description="Validation Exception test.",validator=v)
    values = dict_extract(payload,{'email','name','password'})
    values['password'] = values['password'].encode('utf-8')
    values['is_admin'] = True if payload.get('is_admin') == True else False
    print("Values: ",values)
    user = User(**values)
    try:
        user.save()
        db.session.commit()
        rb={"success": True}
    except IntegrityError as e:
        logger.error("exception occurred", exc_info=True)
        print("Type: ",type(e))
        abort(400,"Error creating user: User already exists")
    except Exception as e:
        logger.error("exception occurred", exc_info=True)
        print("Type: ",type(e))
        abort(400,"Error creating user")

    return JsonResponse(rb)


@bp.route('/user', methods=['PATCH'])
@is_authorized()
def update_user():
    try:
            payload = json.loads(request.data)
            values = dict_extract(payload,{'email'})
            values['name'] = payload.get('name')
            values['is_admin'] = True if payload.get('is_admin') == True else False
            values['active'] = True if payload.get('active') == True else False
            values['unit_system'] = payload.get('unit_system')
            values['heatmap_view_id'] = payload.get('heatmap_view_id')
            values['warn_enabled'] = payload.get('warn_enabled')
            email = values['email']
            print("email is: ",email)
            # del values['firstname']
            # del values['lastname']

            user = User.query.filter(User.email == email).first()
            # print(values)
            user.fill(**values)
            db.session.commit()
    except:
        logger.error("exception occurred", exc_info=True)
        abort(400,"Error modifying user")
    user_details = User.query.filter(User.email == email).first()
    if user_details:
        return JsonResponse(SQLAlchemyResponseBuilder(user_details).camel(False))
    else:
        return JsonResponse(MDHResponseBuilder(statusCode=HTTPStatus.OK.value))

@bp.route('/user/<email>', methods=['DELETE'])
@is_authorized()
def delete_user(email=None):
    try:
            user = User.query.filter(User.email == email).first()
            db.session.delete(user)
            db.session.commit()
            return JsonResponse(MDHResponseBuilder(statusCode=HTTPStatus.OK.value))
    except:
        logger.error("exception occurred", exc_info=True)
        abort(400,"Error deleting user")


@bp.route('/users', methods=['GET'])
@is_authorized()
def users():
    users = User.all()
    rb = SQLAlchemyResponseBuilder(users).camel(False)
    return JsonResponse(rb)

@bp.route('/user', methods=['PUT'])
@is_authorized()
def change_password():
    payload = request.get_json(force=True)
    v = Validator({
        'email':{'required': True,'type': 'string','regex': '^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$'},
        'old_password':{'required': False,'type': 'string'},
        'new_password':{'required': True,'type': 'string','minlength':8}
    },allow_unknown=True)
    if not v.validate(payload):
        return JsonResponse(ValidationErrorResponseBuilder(v))
    try:
        user = User.where(email=payload.get('email').lower()).first()
        login_disabled = env("LOGIN_DISABLED",False)
        if 'old_password' in payload:
            if user and user.check_password(payload['old_password']):
                user.password = payload['new_password'].encode('utf-8')
                db.session.commit()
                return JsonResponse(MDHResponseBuilder(statusCode=HTTPStatus.OK.value))
            
            abort(400,"Old password is incorrect")

        if login_disabled or current_user.is_admin:
                user.password = payload['new_password'].encode('utf-8')
                db.session.commit()
                return JsonResponse(MDHResponseBuilder(statusCode=HTTPStatus.OK.value))
        
        
            
    except Exception as e:
        print("ERROR: ",e)
        abort(400,e)


@bp.route('oauth', methods=['GET'])
@is_authorized()
def oauth():
    redirect_url = request.args.get('redirect_url',None)
    if not redirect_url:
        abort(HTTPStatus.BAD_REQUEST.value,'redirect_url is required')

    user = current_user._get_current_object()
    if not user:
        abort(HTTPStatus.FORBIDDEN.value,'User is not logged in')

    rand_token = uuid4().hex
    user.remote_token = rand_token
    db.session.commit()

    url = redirect_url + "?token=" + rand_token
    print('tokenUrl: ',url)
    return redirect(url)

@bp.route('validate', methods=['GET'])
def validate():
    token = request.args.get('token',None)
    redirect_url = request.args.get('redirect_url',None)
    if not token:
        abort(HTTPStatus.BAD_REQUEST.value,'token is required')

    user = User.where(remote_token=token).first()
    if not user:
        abort(HTTPStatus.FORBIDDEN.value,'Token is invalid')

    user.remote_token = None
    db.session.commit()
    # url = redirect_url+"?user="+user.email
    # print('validateUrl: ',url)
    # return redirect(url)
    return JsonResponse(SQLAlchemyResponseBuilder(user).camel(False))