import datetime
from ..module import bp
from mdhwebserver.models.models import User, ForgotPassword
from os import getenv as env

import json

import logging
from cerberus import Validator
from mdhwebserver.appservices import database as db

from http import HTTPStatus
from flask import (
    request,
    redirect,
    abort
)
from uuid import uuid4
import urllib.parse

from datetime import datetime, timedelta, timezone
from flask_jwt_extended import create_access_token, get_jwt_identity, JWTManager, current_user, get_jwt, set_access_cookies, unset_jwt_cookies
from mdhwebserver.wrappers.is_authorized import is_authorized

from mdhwebserver.builders.responses import JsonResponse
from mdhwebserver.builders.response_builder import SQLAlchemyResponseBuilder, MDHResponseBuilder
from mdhwebserver.builders import ValidationErrorResponseBuilder
from mdhwebserver.exceptions import ValidationException
from mdhwebserver.utils.listutils import dict_extract
from mdhwebserver import application
from ....modules.email.tasks.tasks import submit_task

from sqlalchemy.exc import IntegrityError

logger = logging.getLogger('forgotpassword')

@bp.route('/forgotpassword', methods=['POST'])
def checkuser():
    payload = request.get_json(force=True)
    v = Validator({
        'email':{'required': True,'type': 'string','regex': '^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$'}
    },allow_unknown=True)
    if not v.validate(payload):
        return JsonResponse(ValidationErrorResponseBuilder(v))

    user = User.where(email=payload.get('email').lower()).first()
    if user and user.active:
        values = {}
        randam_token = uuid4().hex
        
        values['user_id'] = user.id
        values['token'] = randam_token
        values['expires'] = datetime.utcnow() + timedelta(hours=24)
        
        fp = ForgotPassword(**values)
        try:
            fp.save()
            db.session.commit()
            rb={"success": True}
            # submit_task.delay(to=[user.email], subject='Reset Password', txt='email.txt', html='forgotpassword.html', token=randam_token, link='http://localhost:8080/#/forgotpassword?email='+payload.get('email').lower()+'&token='+randam_token)
            submit_task.delay(to=[user.email], subject='Reset Password', txt='email.txt', html='forgotpassword.html', token=randam_token, link="https://"+request.host+"/pumpsense#/forgotpassword?email="+payload.get('email').lower()+"&token="+randam_token)
        except IntegrityError as e:
            logger.error("exception occurred", exc_info=True)
            print("Type: ",type(e))
            abort(400,"Error registering to forgot password")
        except Exception as e:
            logger.error("exception occurred", exc_info=True)
            print("Type: ",type(e))
            abort(400,"Error registering fp")

        return JsonResponse(rb)
    abort(HTTPStatus.FORBIDDEN.value, 'Invalid Email')
    
@bp.route('/validateToken', methods=['POST'])
def validateToken():
    payload = request.get_json(force=True)
    v = Validator({
        'email': {'required':True, 'type': 'string','regex': '^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$'},
        'token':{'required':True,'type':'string'},
        'password':{'required': True,'type': 'string','minlength':8}
    },allow_unknown=True)
    if not v.validate(payload):
        return JsonResponse(ValidationErrorResponseBuilder(v))
    try:
        user = User.where(email=payload.get('email').lower()).first()
        token_entry = ForgotPassword.where(token=payload.get('token')).first()
        if user:
            if token_entry and token_entry.user_id == user.id:
                timeDiff = token_entry.expires - datetime.utcnow()
                timeDiff = timeDiff.total_seconds()
                if token_entry.used is None and timeDiff > 0:
                    # if token_entry.token == payload.get('token'):
                    user.password = payload['password'].encode('utf-8')
                    token_entry.used = datetime.utcnow()
                    db.session.commit()
                    return JsonResponse(MDHResponseBuilder(statusCode=HTTPStatus.OK.value))
                abort(HTTPStatus.FORBIDDEN.value, 'Code expired')
            abort(HTTPStatus.FORBIDDEN.value, 'Code does not match')
        abort(HTTPStatus.FORBIDDEN.value, 'User is not registered')
        
            
    except Exception as e:
        print("ERROR: ",e)
        abort(400,e)