import json
from json.decoder import JSONDecodeError
import logging

logger = logging.getLogger('celery_task')

from mdhwebserver.appservices import celery
from mdhwebserver.managers.task_manager import task_register
from mdhwebserver.application import app
from flask_mail import Message, Mail
from flask import render_template
from mdhwebserver.models.models import SmtpConfig
from mdhwebserver.appservices import database as db
from mdhwebserver.models.models import MailLog

@celery.task(bind=True, default_retry_delay=15)
def submit_task(self,subject, to, txt=None, html=None, **kwargs):
    send_email(subject, to, txt, html, **kwargs)

def send_email(subject, to, txt=None, html=None, **kwargs):
    error = None
    result = True
    e = None
    with app.app_context():
        e = None
        smtp_data=SmtpConfig.query.first();
        if not smtp_data:
            return
        if not smtp_data.smtpServer:
            return
        app.config.update(dict(
            MAIL_SERVER = smtp_data.smtpServer,
            MAIL_PORT = smtp_data.smtpPort,
            MAIL_USE_TLS = True if smtp_data.tls == True else False,
            MAIL_USE_SSL = False if smtp_data.tls == True else True,
            MAIL_USERNAME = smtp_data.smtpUsername,
            MAIL_PASSWORD = smtp_data.smtpPassword,
            MAIL_FROM = smtp_data.fromEmail,
            ))
        mail = Mail(app)
        mail.init_app(app)
        print("********* Email Task Started...")
        msg = Message(subject=subject,sender=app.config['MAIL_FROM'], recipients=to)
        msg.body = render_template(txt, **kwargs)
        msg.html = render_template(html, **kwargs)
        try:
            mail.send(msg)
        except Exception as e:
            print("Mail Exception: " + str(e))
            result = False
            error = str(e)
            e_result = e    # This is needed to save the exception
            #raise e
        if not to:
            return
        send_to = to[0] if to else None
        if len(to) > 1:
            send_to = send_to + " ..."
        MailLog.create(**{
            'sender': smtp_data.fromEmail,
            'to': send_to,
            'subject': subject,
            'server': smtp_data.smtpServer,
            'port': smtp_data.smtpPort,
            'result': result ,
            'error': error
      
        })
        db.session.commit()
        if result == False:
            raise e_result


