from flask import (
    Blueprint
)

import sqlalchemy as sa
import sqlalchemy.orm as orm

from mdhwebserver.models.models import SubassemblyInstance, PmFunction, TrendMetaData, Assembly, DeviceDataItem, Gauge,DeviceDataItemUnitDataPointAssociation
from mdhwebserver.builders.response_builder import SQLAlchemyResponseBuilder, DictResponseBuilder
from mdhwebserver.builders.responses import JsonResponse
from mdhwebserver.utils.virtual_gauges.virtual_gauges import update_virtual_gauges
from flask import request
from mdhwebserver.appservices import database as db

bp = Blueprint('gaugebuilder', __name__)

def create_virtual_pm_function(name,subassembly,trend_meta_datas,gauge_style="virtual",gauge_type='value',trend_existence=True,show_trend=None):
    vp = PmFunction.create(**{
        'name': name,
        'display_name': name,
        'subassembly_instance_id': subassembly.id,
        'gauge_style': gauge_style,
        'gauge_type': gauge_type,
        "trend_existence": trend_existence
    })
    tms=TrendMetaData().filter(TrendMetaData.id.in_(trend_meta_datas)).all()
    if show_trend is None:
        tmd_show_trend = tms[0]

    vp.show_trend = tmd_show_trend.name
    vp.units = tmd_show_trend.units
    vp.unit_system = 'fps' if tmd_show_trend.units == tmd_show_trend.fps_units else 'mks'
    vp.trend_meta_data = tms

    return vp

def update_virtual_pm_functions_kpis_old(subassembly,vp):
    for assemblies in subassembly.assemblies:
        for g in assemblies.gauges:
            if g.machine_view_type == "machine":
                update_virtual_gauges(g,set([vp]))

def edit_virtual_pm_function(subassembly,vp):
    for assemblies in subassembly.assemblies:
        for g in assemblies.gauges:
            if g.machine_view_type == "machine":
                ddi=DeviceDataItem.where(gauge_id=g.id, pm_func_id=vp.id).first()
                if ddi:
                    ddi.trend_meta_datas = []
                    ddi.device_unit_data_point_associations = []
                        
                update_virtual_gauges(g,set([vp]))

def update_virtual_pm_functions_kpis(assembly,vp):
    for g in assembly.gauges:
        if g.machine_view_type == "machine":
            update_virtual_gauges(g,set([vp]))


@bp.route('/virtual_pm_function',methods=['post'])
def post_virtual_pm_function():
    j = request.json
    print("J",j)
    trend_meta_datas = j.get("trend_meta_datas")
    print("j is: ",j)
    sid = j.get('subassembly_instance_id')
    show_trend = j.get('show_trend')
    subassembly = SubassemblyInstance.where(id=sid).options(
        orm.selectinload(SubassemblyInstance.assemblies).options(
            orm.selectinload(Assembly.gauges)
        )
    ).first()

    vp = create_virtual_pm_function(j.get('name'),subassembly,trend_meta_datas,
        gauge_style= "virtual",
        gauge_type= j.get('gauge_type','value'),
        trend_existence= j.get('trend_existence',True)              
        )

    # vp = PmFunction.create(**{
    #     'name': j.get('name'),
    #     'display_name': j.get('name'),
    #     'subassembly_instance_id': j.get('subassembly_instance_id'),
    #     'gauge_style': "virtual",
    #     'gauge_type': j.get('gauge_type','value'),
    #     "trend_existence": j.get('trend_existence',True)
    # })
    # tms=TrendMetaData().filter(TrendMetaData.id.in_(trend_meta_datas)).all()
    # if show_trend is None:
    #     tmd_show_trend = tms[0]

    # vp.show_trend = tmd_show_trend.name
    # vp.units = tmd_show_trend.units
    # vp.unit_system = 'fps' if tmd_show_trend.units == tmd_show_trend.fps_units else 'mks'
    # vp.trend_meta_data = tms

    # for assemblies in subassembly.assemblies:
    #     for g in assemblies.gauges:
    #         if g.machine_view_type == "machine":
    #             update_virtual_gauges(g,set([vp]))

    update_virtual_pm_functions_kpis_old(subassembly,vp)

    rb = SQLAlchemyResponseBuilder(vp).camel(True)
    db.session.commit()
    return JsonResponse(rb,content_type='application/json')

@bp.route('/virtual_pm_function',methods=['put'])
def put_virtual_pm_function():
    j = request.json
    trend_meta_datas = j.get("trend_meta_datas")
    vp = PmFunction.where(id=j.get('id')).first()
    tms=TrendMetaData().filter(TrendMetaData.id.in_(trend_meta_datas)).all()
    vp.trend_meta_data = tms

    edit_virtual_pm_function(vp.subassembly,vp)

    rb = SQLAlchemyResponseBuilder(vp).camel(True)
    db.session.commit()
    return JsonResponse(rb,content_type='application/json')

@bp.route('/virtual_pm_function2',methods=['post'])
def post_virtual_pm_function2():
    j = request.json
    print("J",j)
    trend_meta_datas = j.get("trend_meta_datas")
    print("j is: ",j)
    sid = j.get('subassembly_instance_id')
    show_trend = j.get('show_trend')
    trend_points = j.get('trend_points') 
    assembly_id = j.get('assembly_id')

    subassembly_names=set()
    pm_names=set()
    for tp in trend_points:
        subassembly_names.add(tp.get('subassembly_instance_name'))
        pm_names.add(tp.get('pm_function_name'))

    print("Subassembly names: ",subassembly_names)
    print("XXXXXXXXXXXXX pm names: ", pm_names)
    assembly=Assembly.where(id=assembly_id).options(
            orm.joinedload(Assembly.subassemblies_dict).options(
                orm.joinedload(SubassemblyInstance.pm_functions_dict.and_(PmFunction.name.in_(pm_names))).options(
                    orm.joinedload(PmFunction.trend_meta_mapped)
            )
        )
    ).first()
    # pm_names=set()
    # for tp in trend_points:
    #     pm_names.add(tp.get('pm_function_name'))

    # print("Names: ",pm_names)
    subassembly = SubassemblyInstance.where(id=sid).first()

    trend_meta_datas=set()
    for tp in trend_points:
        pm = assembly.subassemblies_dict[tp['subassembly_instance_name']].pm_functions_dict.get(tp.get('pm_function_name'))
        for tmd_name in tp.get('trend_names',[]):
            print("-------------- name: ",tmd_name)
            tmd = pm.trend_meta_mapped.get(tmd_name)
            if tmd:
                trend_meta_datas.add(tmd.id)

    print("--------------------- trend_meta_datas: ",trend_meta_datas)

    vp = create_virtual_pm_function(j.get('name'),subassembly,trend_meta_datas,
        gauge_style= "virtual",
        gauge_type= j.get('gauge_type','value'),
        trend_existence= j.get('trend_existence',True)              
        )
    update_virtual_pm_functions_kpis(assembly,vp)

    rb = SQLAlchemyResponseBuilder(vp).camel(True)

    # rb = SQLAlchemyResponseBuilder(subassembly).camel(True)
    # rb = SQLAlchemyResponseBuilder(assembly).camel(True)

    db.session.commit()
    return JsonResponse(rb,content_type='application/json')


def make_virtual_ddi(G,virtual_pm_function,gauge_map):
    ddi=DeviceDataItem.where(gauge_id=G.id, pm_func_id=virtual_pm_function.id).first()
    if ddi==None:
        print("No DDI")
        ddi=DeviceDataItem.create(**{
            'gauge_id': G.id,
            'pm_func_id': virtual_pm_function.id
        })
        ddi.pm_function_name = ddi.pm_function.name
        ddi.subassembly_instance_id = ddi.pm_function.subassembly_instance_id
        ddi.subassembly_instance_name = ddi.pm_function.subassembly_instance_name
        G.device_data_items.append(ddi)
    # pm = PmFunction.query.filter(PmFunction.id==virtual_pm_function.id)
    # ddi.kpis = get_virtual_gauge_kpis(pm.first(),G.id)
    # ddi.kpis = get_virtual_gauge_kpis2(virtual_pm_function,G.id,ddi.kpis)
    # print("XXXXXXXXXXXx ddi kpis: ", ddi.kpis)
    for gm in gauge_map:
        for tmid in gm['trend_meta_data_ids']:
            ddi.device_unit_data_point_associations.append(DeviceDataItemUnitDataPointAssociation(**{
                "gauge_id": gm['gauge_id'],
                "trend_meta_data_id": tmid
            }))
    return ddi


@bp.route('/virtual_gauge',methods=['post'])
def post_virtual_gauge_function():
    j = request.json
    print("J",j)
    trend_meta_datas = j.get("trend_meta_datas")
    print("j is: ",j)
    sid = j.get('subassembly_instance_id')
    show_trend = j.get('show_trend')
    gauge_id = j.get("gauge_id")
    gauge_map = j.get("gauge_map")
    print("Gauge_map: ", gauge_map)
    subassembly = SubassemblyInstance.where(id=sid).options(
        orm.selectinload(SubassemblyInstance.assemblies).options(
            orm.selectinload(Assembly.gauges)
        )
    ).first()
    G=Gauge.where(id=gauge_id).first()
    tmids = set()
    print("gauge_map: ",gauge_map)
    for gm in gauge_map:
        tmids |= set(gm['trend_meta_data_ids'])

    vp = PmFunction.create(**{
        'name': j.get('name'),
        'display_name': j.get('name'),
        'subassembly_instance_id': j.get('subassembly_instance_id'),
        'gauge_style': "abstract",
        'gauge_type': j.get('gauge_type','value'),
        "trend_existence": j.get('trend_existence',True)
    })
    tms=TrendMetaData().filter(TrendMetaData.id.in_(tmids)).all()
    if show_trend is None:
        tmd_show_trend = tms[0]

    vp.show_trend = tmd_show_trend.name
    vp.units = tmd_show_trend.units
    vp.unit_system = 'fps' if tmd_show_trend.units == tmd_show_trend.fps_units else 'mks'
    vp.trend_meta_data = tms

    ddi=make_virtual_ddi(G,vp,gauge_map)
    # for assemblies in subassembly.assemblies:
    #     for g in assemblies.gauges:
    #         if g.machine_view_type == "machine":
    #             update_virtual_gauges(g,set([vp]))

    rb = SQLAlchemyResponseBuilder(ddi).camel(True)
    db.session.commit()
    return JsonResponse(rb,content_type='application/json')

@bp.route('/virtual_gauges/<assembly_id>',methods=['get'])
def get_virtual_gauge_function(assembly_id):
    assembly=Assembly.where(id=assembly_id).options(
        orm.joinedload(Assembly.subassemblies).options(
            orm.joinedload(SubassemblyInstance.pm_functions.and_(PmFunction.gauge_style=='virtual')).options(
                orm.joinedload(PmFunction.trend_meta_data).options(orm.joinedload(TrendMetaData.pm_functions))
            )
        )
    ).first()
    d={
        "name": assembly.name,
        "subassemblies": []
    }
    for subassembly in assembly.subassemblies:
        if not subassembly.pm_functions:
            continue
        sub = {
            "name": subassembly.name,
            "pm_functions": []
        }
        for pm_function in subassembly.pm_functions:
            pmf = {
                "name": pm_function.name,
                "trend_meta_data":[]
            }
            for tm in pm_function.trend_meta_data:
                pmf['trend_meta_data'].append({
                    'name': tm.name,
                    'pm_function_name': tm.pm_function.name,
                    'subassembly_instance_name':tm.pm_function.subassembly_instance_name
                })
            sub['pm_functions'].append(pmf)
        d['subassemblies'].append(sub)
    # d = assembly._to_dict(
    #     camelKeys=False,
    #     only=["name","display_name",{"subassemblies": ["name",{"pm_functions":["name",{"trend_meta_data":["name"]}]}]}]
    # )
    # rb = SQLAlchemyResponseBuilder(assembly).camel(True)
    rb = DictResponseBuilder(d)
    return JsonResponse(rb,content_type='application/json')
