from ..gaugeeditor import bp
from mdhwebserver.appservices import database as db
from flask import request
from mdhwebserver.models.models import PmFunction
from mdhwebserver.builders import MDHResponseBuilder, JsonResponse, MDHErrorResponseBuilder, SQLAlchemyResponseBuilder
from sqlalchemy.orm import joinedload
from mdhwebserver.etl import create_sum_etl_gauges, create_avg_etl_gauges
from http import HTTPStatus
from mdhwebserver.wrappers.is_authorized import is_authorized

@bp.route('/etl_gauge', methods=['POST'])
@is_authorized()
def post_etl_gauge():
    data = request.json
    pm_function_id = data.get("pm_function_id")
    name = data.get("name")
    type = data.get("type")

    pmf = PmFunction.where(id=pm_function_id).options(joinedload(PmFunction.subassembly)).first()
    if pmf is None:
        return JsonResponse(MDHErrorResponseBuilder("PM Function not found"))
    if type == "sum":
        create_sum_etl_gauges(pmf,name)
    elif type == "avg":
        create_avg_etl_gauges(pmf,name)
    else:
        return JsonResponse(MDHErrorResponseBuilder("Invalid ETL Type"))
    db.session.commit()
    return JsonResponse(SQLAlchemyResponseBuilder([]))

@bp.route('/etl_gauge', methods=['DELETE'])
@is_authorized()
def delete_etl_gauge():

    pm_function_id = request.json.get("pm_function_id")
    if pm_function_id is None:
        return JsonResponse(MDHErrorResponseBuilder("Pm Function Id Missing"))

    pmf = PmFunction.where(etl_pm_function_id=pm_function_id).delete()
    db.session.commit()
    return JsonResponse(MDHResponseBuilder(statusCode=HTTPStatus.OK.value))