from mdhwebserver.models.models import (
    TrendMetaData,
    GaugeRange,
)
from mdhwebserver.modules.units.controllers.units import si_unit_convert
from mdhwebserver.appservices import database as db

# PmFunction Basic
def SetPmFunctionBasic(target, name, request, chosen_image, filename):
    print("chosen_image", chosen_image)
    if not name == target.display_name:
        target.display_name = name
    if 'image' in request.files:
        target.gauge_base_image = "web-images/" + filename
    if chosen_image:
        target.gauge_base_image = chosen_image

# PmFunction Advanced
def SetPmFunctionAdvanced(pmfunction, new_gauge_data):
    pmfunction.mro = new_gauge_data["mro"]
    pmfunction.trend_existence = new_gauge_data["trend_existence"]
    pmfunction.gauge_type = new_gauge_data["gauge_type"]
    pmfunction.units = new_gauge_data["units"]
    pmfunction.y_trend_min = new_gauge_data["y_trend_min"]
    pmfunction.y_trend_max = new_gauge_data["y_trend_max"]
    pmfunction.y_unit = new_gauge_data["y_unit"]
    pmfunction.x_unit = new_gauge_data["x_unit"]
    pmfunction.set_color = new_gauge_data["set_color"]
    pmfunction.show_trend = new_gauge_data["show_trend"]

# PmFunction Trend
def SetPmFunctionTrend(new_gauge_data):
    for item in new_gauge_data["trend_meta_data"]:
        trend = TrendMetaData.where(id=item["id"]).first()
        if trend is None:
            continue
        trend.display_name = item["display_name"]
        trend.color = item["color"]
        trend.save()

def SetPmFunctionAddGaugeRanges(PmFunc, gauge_range_data):
    response = []
    for gauge_range in gauge_range_data:
        try:
            name = gauge_range["name"]
            trend_data_name = gauge_range["trend_meta_data_name"]
            color = gauge_range["color"]
            try:
                range_start = float(gauge_range["start"])
            except Exception as e:
                print(e)
                range_start = None
            try:
                range_end = float(gauge_range["end"])
            except Exception as e:
                range_end = None

            convert_units = { }
            mks_start = fps_start = convert_units['start'] = range_start
            mks_end = fps_end = convert_units['end'] = range_end

            if PmFunc.trend_meta_mapped[trend_data_name].fps_units != PmFunc.trend_meta_mapped[trend_data_name].mks_units:
                if PmFunc.trend_meta_mapped[trend_data_name].units != PmFunc.trend_meta_mapped[trend_data_name].fps_units:
                    converted = si_unit_convert(PmFunc.trend_meta_mapped[trend_data_name].units,PmFunc.trend_meta_mapped[trend_data_name].fps_units,convert_units)
                    fps_start = converted.get("start")
                    fps_end = converted.get("end")
                else:
                    converted = si_unit_convert(PmFunc.trend_meta_mapped[trend_data_name].units,PmFunc.trend_meta_mapped[trend_data_name].mks_units,convert_units)
                    mks_start = converted.get("start")
                    mks_end = converted.get("end")
        except Exception as e:
            print(e)

        try:
            new_range = GaugeRange(**{
                'name': name,
                'trend_meta_data_name': trend_data_name,
                'trend_meta_data': PmFunc.trend_meta_mapped[trend_data_name],
                'pm_function': PmFunc,
                'color': color,
                'start': range_start,
                'end': range_end,
                'fps_start': fps_start,
                'fps_end': fps_end,
                'mks_start': mks_start,
                'mks_end': mks_end
            })
            db.session.add(new_range)
            db.session.commit()
            response.append({
                                "id": new_range.id,
                                "name": new_range.name,
                                "trend_meta_data_name": new_range.trend_meta_data_name,
                                "trend_meta_data_id": new_range.trend_meta_data_id,
                                "pm_function_id": new_range.pm_function_id,
                                "color": new_range.color,
                                "start": new_range.start,
                                "end": new_range.end,
                                "fps_start": new_range.fps_start,
                                "fps_end": new_range.fps_end,
                                "mks_start": new_range.mks_start,
                                "mks_end": new_range.mks_end,
                            },)
        except Exception as e:
            print(e)
    return response

def SetPmFunctionEditGaugeRanges(PmFunc, edit_gauge_range_data):
    for gauge_range in edit_gauge_range_data:
        # Get form data
        name = gauge_range["name"]
        trend_data_name = gauge_range["trend_meta_data_name"]
        color = gauge_range["color"]
        try:
            range_start = float(gauge_range["start"])
        except Exception as e:
            print(e)
            range_start = None
        try:
            range_end = float(gauge_range["end"])
        except Exception as e:
            range_end = None

        gauge_range = GaugeRange.where(id=gauge_range["id"]).first()
        if gauge_range is None:
            continue

        convert_units = { }
        mks_start = fps_start = convert_units['start'] = range_start
        mks_end = fps_end = convert_units['end'] = range_end

        if PmFunc.trend_meta_mapped[trend_data_name].fps_units != PmFunc.trend_meta_mapped[trend_data_name].mks_units:
            if PmFunc.trend_meta_mapped[trend_data_name].units != PmFunc.trend_meta_mapped[trend_data_name].fps_units:
                converted = si_unit_convert(PmFunc.trend_meta_mapped[trend_data_name].units,PmFunc.trend_meta_mapped[trend_data_name].fps_units,convert_units)
                fps_start = converted.get("start")
                fps_end = converted.get("end")
            else:
                converted = si_unit_convert(PmFunc.trend_meta_mapped[trend_data_name].units,PmFunc.trend_meta_mapped[trend_data_name].mks_units,convert_units)
                mks_start = converted.get("start")
                mks_end = converted.get("end)")

        try:
            gauge_range.name = name
            gauge_range.trend_meta_data_name = trend_data_name
            gauge_range.trend_meta_data = PmFunc.trend_meta_mapped[trend_data_name]
            gauge_range.pm_function = PmFunc
            gauge_range.color = color
            gauge_range.start = range_start
            gauge_range.end = range_end
            gauge_range.fps_start = fps_start
            gauge_range.fps_end = fps_end
            gauge_range.mks_start = mks_start
            gauge_range.mks_end = mks_end

            gauge_range.save()
        except Exception as e:
            print(e)
            return jsonify({"error": e}), 500