from ..module import bp
from flask import (
    request
)
from datetime import datetime, timezone
from mdhwebserver.builders.responses import JsonResponse
from mdhwebserver.builders.response_builder import SQLAlchemyResponseBuilder,ExpertGraphRespsoneBuilder
import time

from sqlalchemy.orm import (
        joinedload,
        with_expression,
        selectinload
)

import sqlalchemy as sa

from flask_jwt_extended import current_user
from mdhwebserver.wrappers.is_authorized import is_authorized


from mdhwebserver.models.models import (
        Gauge,
        GaugeTrend,
        PmFunction,
        SubassemblyInstance,
        DeviceDataItem,
        TrendMetaData,
        ExpertReport,
        ExpertDataItem,
        ExpertDataPoint,
        ExpertReportItem,
        DeviceUnitDataPoint,
        DeviceDataItemUnitDataPointAssociation
)


from mdhwebserver.models.SiUnitModels import UnitTranslate

@bp.route('', methods=['GET'])
def index():
    return 'testing'

@bp.route('unit_data', methods=['POST'])
@is_authorized()
def unit_data():

    startTime = request.json.get('start_time')
    endTime = request.json.get('end_time')
    camelKeys = request.json.get('camel',True)
    gauge_id = request.json.get('gauge_id')
    pm_function_id = request.json.get('pm_function_id')

    sTime = datetime.fromtimestamp(startTime, tz=timezone.utc)
    eTime = datetime.fromtimestamp(endTime, tz=timezone.utc)
    pm = PmFunction.query.filter(PmFunction.id==pm_function_id)
    pm = pm.options(
        selectinload(PmFunction.device_data_items.and_(DeviceDataItem.gauge_id == gauge_id)),
        joinedload(PmFunction.trend_meta_data)\
                .joinedload(TrendMetaData.unit_data_points\
                    .and_(DeviceUnitDataPoint.gauge_id == gauge_id)\
                    .and_(DeviceUnitDataPoint._timestamp.between(sTime,eTime))\
                ).options(
                with_expression(DeviceUnitDataPoint.mks, sa.func.unit_at_double(DeviceUnitDataPoint.value,sa.sql.text("trend_meta_data_1.mks_units"))), \
                with_expression(DeviceUnitDataPoint.fps, sa.func.unit_at_double(DeviceUnitDataPoint.value,sa.sql.text("trend_meta_data_1.fps_units"))), \
                # joinedload(DeviceUnitDataPoint.trend_meta_data) \
            ) \
                
        ) \
        .first()
    rb = SQLAlchemyResponseBuilder(pm)  \
            .only(['name',"id",{"device_data_items":["yellow_threshold","red_threshold"]},{"trend_meta_data":["color","display_name","name","id","pm_func_id","units","fps_units","mks_units",{"unit_data_points":["value","timestamp","mks","fps"]}]}]) \
            .camel(camelKeys)
    return JsonResponse(rb,content_type='application/json')


@bp.route('trend_unit_data', methods=['POST'])
@is_authorized()
def trend_unit_data():

    startTime = request.json.get('start_time')
    endTime = request.json.get('end_time')
    camelKeys = request.json.get('camel',True)
    gauge_id = request.json.get('gauge_id')
    trend_ids = request.json.get('trend_ids')


    sTime = datetime.fromtimestamp(startTime, tz=timezone.utc)
    eTime = datetime.fromtimestamp(endTime, tz=timezone.utc)

    tmd_points = TrendMetaData.query.filter(TrendMetaData.id.in_(trend_ids)).options(
        joinedload(TrendMetaData.unit_data_points\
            .and_(DeviceUnitDataPoint.gauge_id == gauge_id)\
            .and_(DeviceUnitDataPoint._timestamp.between(sTime,eTime))\
        ).options(
            with_expression(DeviceUnitDataPoint.mks, sa.func.unit_at_double(DeviceUnitDataPoint.value,sa.sql.text("trend_meta_data.mks_units"))), \
            with_expression(DeviceUnitDataPoint.fps, sa.func.unit_at_double(DeviceUnitDataPoint.value,sa.sql.text("trend_meta_data.fps_units"))), \
        )
    ).all()

    rb = SQLAlchemyResponseBuilder(tmd_points)  \
            .only(["color","display_name","name","id","pm_func_id","units","fps_units","mks_units",{"unit_data_points":["gauge_id","value","timestamp","mks","fps"]}]) \
            .camel(camelKeys)
    return JsonResponse(rb,content_type='application/json')


@bp.route('device_item_unit_data', methods=['POST'])
@is_authorized()
def device_item_unit_data():

    startTime = request.json.get('start_time')
    endTime = request.json.get('end_time')
    camelKeys = request.json.get('camel',True)
    gauge_id = request.json.get('gauge_id')
    trend_ids = request.json.get('device_data_item')


    sTime = datetime.fromtimestamp(startTime, tz=timezone.utc)
    eTime = datetime.fromtimestamp(endTime, tz=timezone.utc)


    # tmd_points = DeviceDataItem.query.filter(DeviceDataItem.id==trend_ids).options(
    #     selectinload(DeviceDataItem.device_unit_data.and_(DeviceUnitDataPoint._timestamp.between(sTime,eTime)))
    # )\
    #     .first()
    
    # tmd_points = DeviceDataItem.query.filter(DeviceDataItem.id==trend_ids) \
    #     .options(selectinload(DeviceDataItem.device_data_item_associations)) \
    #     .first()

    tmd_points = DeviceDataItem.query.filter(DeviceDataItem.id==trend_ids) \
        .options(selectinload(DeviceDataItem.device_unit_data_point_associations) \
            .options(
                joinedload(DeviceDataItemUnitDataPointAssociation.trend_meta_data).options(
                    joinedload(TrendMetaData.unit_data_points\
                    .and_(DeviceUnitDataPoint.gauge_id == DeviceDataItemUnitDataPointAssociation.gauge_id)\
                    .and_(DeviceUnitDataPoint._timestamp.between(sTime,eTime)
                )\
        ).options(
            with_expression(DeviceUnitDataPoint.mks, sa.func.unit_at_double(DeviceUnitDataPoint.value,sa.sql.text("trend_meta_data_1.mks_units"))), \
            with_expression(DeviceUnitDataPoint.fps, sa.func.unit_at_double(DeviceUnitDataPoint.value,sa.sql.text("trend_meta_data_1.fps_units"))), \
        )

            ))
        ) \
        .first()


    # tmd_points = DeviceDataItem.query.filter(DeviceDataItem.id==trend_ids) \
    #     .first()


    rb = SQLAlchemyResponseBuilder(tmd_points)  \
        .camel(camelKeys)

    return JsonResponse(rb,content_type='application/json')



@bp.route('report2/expert/<machine_id>/<reportname>', methods=['GET'])
@is_authorized()
def expert(machine_id=None, reportname=None):
    
    er = ExpertReport.query.filter(ExpertReport.name==reportname)
    startTime = request.args.get('startTime', 0, type=float)
    endTime = request.args.get('endTime', time.time(), type=float)
    sTime = datetime.fromtimestamp(startTime, tz=timezone.utc)
    eTime = datetime.fromtimestamp(endTime, tz=timezone.utc)

    er = er.options(
        joinedload(ExpertReport.items) \
        .selectinload(ExpertReportItem.data_points \
        .and_(ExpertDataPoint.machine_id == machine_id)\
        .and_(ExpertDataPoint._timestamp.between(sTime,eTime))
        )
    ).first()
    rb = SQLAlchemyResponseBuilder(er) \
        .only(['id','name',{'items':['name','color',{'data_points':['timestamp','value']}]}])
    return JsonResponse(rb)


from mdhwebserver.helpers.infixtopostfix import infix_to_postfix,postfix_calculate
 

@bp.route('report/expert/<machine_id>/<reportname>', methods=['GET'])
@is_authorized()
def expert2(machine_id=None, reportname=None):
    expression = "(pf1mean * v1mean * i1mean)/1000"
    #expression = "pf1mean"
    er = ExpertReport.query.filter(ExpertReport.name==reportname)
    startTime = request.args.get('startTime', 0, type=float)
    endTime = request.args.get('endTime', time.time(), type=float)
    sTime = datetime.fromtimestamp(startTime, tz=timezone.utc)
    eTime = datetime.fromtimestamp(endTime, tz=timezone.utc)

    #This doesn't work because the joinedload uses strings
    er = er.options(
        joinedload(ExpertReport.items) \
        .selectinload(ExpertReportItem.data_points \
        .and_(ExpertDataPoint.machine_id == machine_id)\
        .and_(ExpertDataPoint._timestamp.between(sTime,eTime))
        ),
        joinedload(ExpertReport.items) \
        .selectinload(ExpertReportItem.data_points_mapped \
        .and_(ExpertDataPoint.machine_id == machine_id)\
        .and_(ExpertDataPoint._timestamp.between(sTime,eTime))
        ),
        joinedload("items.parameters")
        
    ).first()
    #rb = SQLAlchemyResponseBuilder(er) 

    #rb = SQLAlchemyResponseBuilder(er) \
    #    .only(['id','name',{'items':['name','color',{'data_points':['timestamp','value']},'data_points_mapped']}])
    #return JsonResponse(rb)
    rb = ExpertGraphRespsoneBuilder(er,expression=expression)
    return JsonResponse(rb)
    #return "Done"


