from ..module import bp
from pathlib import Path

import logging
from mdhwebserver.appservices import database as db
from mdhwebserver import appconfigs

from http import HTTPStatus
from flask import (
    request
)
# from flask_login import login_required, current_user
from sqlalchemy.orm import joinedload, selectinload
from flask_jwt_extended import current_user
from mdhwebserver.builders.responses import JsonResponse
from mdhwebserver.builders.response_builder import SQLAlchemyResponseBuilder, MDHResponseBuilder, MDHErrorResponseBuilder
from mdhwebserver.builders import ValidationErrorResponseBuilder
from mdhwebserver.exceptions import ValidationException
from mdhwebserver.utils.listutils import dict_extract
from mdhwebserver import application

from mdhwebserver.models.models import (
    HeatmapItem
)

from mdhwebserver.wrappers.is_authorized import is_authorized

from sqlalchemy.exc import IntegrityError

logger = logging.getLogger('heatmap')

@bp.route('/heatmap', methods=['GET'])
@is_authorized()
def get_heatmap():
  user=current_user._get_current_object()
  id=request.args.get("id")
  name = request.args.get("name")

  if id is None and name is None:
    if user.heatmap_view_id is None:
      name = 'Default'
    else:
      id = user.heatmap_view_id

  if id is not None:
    hmv = get_heatmap_item_by_id(id)
  else:
    hmv = get_heatmap_item_by_name(name)
  rb = SQLAlchemyResponseBuilder(hmv) \
  .camel(False)
  return JsonResponse(rb)

def get_heatmap_item_by_name(name):
  hmv = HeatmapItem().where(name=name) \
    .options(
      selectinload(HeatmapItem.rchildren),
      selectinload(HeatmapItem.gauge)
    ) \
    .first()
  return hmv


def get_heatmap_item_by_id(id):
  hmv = HeatmapItem().where(id=id) \
    .options(
      selectinload(HeatmapItem.rchildren,recursion_depth=2).options(selectinload(HeatmapItem.gauge),selectinload(HeatmapItem.pm_function)),
      selectinload(HeatmapItem.gauge)
    ) \
    .first()
  return hmv


@bp.route('/heatmap4', methods=['GET'])
@is_authorized()
def get_heatmap4():
  user=current_user._get_current_object()
  id=request.args.get("id")
  name = request.args.get("name")

  if id is None and name is None:
    if user.heatmap_view_id is None:
      name = 'Default'
    else:
      id = user.heatmap_view_id

  if id is not None:
    hmv = get_heatmap_item_by_id(id)
  else:
    hmv = get_heatmap_item_by_name(name)
  rb = SQLAlchemyResponseBuilder(hmv) \
  .camel(False)
  return JsonResponse(rb)


@bp.route('/views', methods=['GET'])
@is_authorized()
def get_heatmap_views():
  return JsonResponse(SQLAlchemyResponseBuilder(HeatmapItem.where(view_type='view').all()).camel(False))

@bp.route('/views4', methods=['GET'])
@is_authorized()
def get_heatmap_views4():
  return JsonResponse(SQLAlchemyResponseBuilder(HeatmapItem.where(view_type='view').all()).camel(False))



from mdhwebserver.utils.validation.circular import rchildren_check_for_circular_references

@bp.route('/view', methods=['POST'])
@is_authorized()
def post_heatmap_view():
  id=request.json.get("id")
  name=request.json.get("name")
  new_name=request.json.get("new_name")
  items=request.json.get("items")
  if items is not None:
    items = [int(float(c)) for c in items]

  # print("----------- Items: ", items)
  if name is None and id is None:
    return JsonResponse(MDHErrorResponseBuilder(message="Name or ID is required"))
  
  if name is not None:
    hm_item = HeatmapItem.query.filter(HeatmapItem.name == name,HeatmapItem.view_type=='view').first()
  if id is not None:
    hm_item = HeatmapItem.query.filter(HeatmapItem.id == id,HeatmapItem.view_type=='view').first()
  if hm_item is not None and new_name is not None:
    hm_item.name = new_name

  if hm_item is None:
    hm_item = HeatmapItem.create(name=name,view_type='view')
  order_id = None

  if items is not None and len(items) > 0:
    # print("add items: ", items)
    hm_items = HeatmapItem.filter(HeatmapItem.id.in_(items)).all()
    check_result = rchildren_check_for_circular_references(hm_item, hm_items)
    if check_result:
      msg = f"{check_result.name} cannot be added to Heatmap View.  Circular references are not allowed."
      db.session.rollback()
      return JsonResponse(MDHErrorResponseBuilder(message=msg))
    hm_item.rchildren = hm_items
  
  rb = SQLAlchemyResponseBuilder(hm_item)
  hm_item.color_update()
  db.session.commit()
  return JsonResponse(rb,content_type='application/json')

@bp.route('/view/<id>', methods=['DELETE'])
@is_authorized()
def delete_heatmap_view(id):
  hm_item = HeatmapItem.query.filter(HeatmapItem.id == id).options(selectinload(HeatmapItem.rchildren)).first()
  print('--------- HM Item: ', hm_item)
  if hm_item:
    # hm_item.rchildren = []
    # hm_item.save()
    hm_item.delete()

  rb = SQLAlchemyResponseBuilder(hm_item)
  db.session.commit()
  return JsonResponse(rb,content_type='application/json')


