from ..module import bp
from pathlib import Path
import os
import json
import time
import gzip
import shutil
import shlex
import subprocess
import sqlalchemy as sa
import logging
from mdhwebserver.appservices import database as db
from mdhwebserver import appconfigs

from mdhwebserver.models.models import Gauge, Assembly, DeviceDataItem, SubassemblyInstance, HeatmapItem

from http import HTTPStatus
from flask import (
    request,
    redirect,
    abort,
    send_file
)
from flask_jwt_extended import current_user
from mdhwebserver.wrappers.is_authorized import is_authorized

from mdhwebserver.builders.responses import JsonResponse
from mdhwebserver.builders.response_builder import SQLAlchemyResponseBuilder, MDHResponseBuilder, MDHErrorResponseBuilder
from mdhwebserver.builders import ValidationErrorResponseBuilder
from mdhwebserver.exceptions import ValidationException


@bp.route('/supers',methods=['get'])
@is_authorized()
def get_supers():
    g = Gauge().where(machine_view_type='super').all()
    rb = SQLAlchemyResponseBuilder(g)
    db.session.commit()
    return JsonResponse(rb,content_type='application/json')


@bp.route('/supers/<gauge_id>',methods=['get'])
@is_authorized()
def get_super(gauge_id):
    g = Gauge().where(id=gauge_id).first()
    rb = SQLAlchemyResponseBuilder(g)
    db.session.commit()
    return JsonResponse(rb,content_type='application/json')


def create_heatmap_machine(machine, default_view = "All Machines"):
  if not machine.heatmap_item:
    heatmap_machine_item = HeatmapItem.create(name=machine.machine_name, view_type='super-machine', gauge_id=machine.id)
    all_machines  = HeatmapItem.query.filter(HeatmapItem.name == default_view).first()
    all_machines.rchildren.append(heatmap_machine_item)
  else:
    heatmap_machine_item = machine.heatmap_item
  heatmap_machine_item.sync_machine(machine)
  machine.color_update()


# @bp.route('/supers',methods=['post'])
# @is_authorized()
# def post_super():
#     machine_name=request.json.get("machine_name")
#     assembly_name=request.json.get("assembly")
#     print("Assembly_name: ", assembly_name)
#     mac=appconfigs.mdh.macAddress.replace(':','')
#     ts=int(time.time())
#     machine_id = f'machine-{mac}-{ts:x}'
#     if (assembly_name):
#         a = Assembly().where(name=assembly_name).first()
#     if a is None:
#         print("No Assembly found.")
#         a = Assembly.create(**{
#             'name': assembly_name,
#             'display_name': assembly_name
#         })
#     g = Gauge().create(**{
#         'machine_name':machine_name,
#         'machine_id': machine_id,
#         'assembly_id': a.id,
#         'onboarded': True,
#         'machine_view_type' : 'super'
#     })
#     create_heatmap_machine(g)
#     db.session.commit()
#     rb = SQLAlchemyResponseBuilder(g)
#     return JsonResponse(rb,content_type='application/json')

# @bp.route('/supers/add',methods=['post'])
# @is_authorized()
# def post_super_add():
#     gauge_id = request.json.get("gauge_id")
#     children = request.json.get("items")
#     children = [int(float(c)) for c in children]
#     print("gauge id: ", gauge_id)
#     print("items: ", children)
#     g=Gauge().where(id=gauge_id).first()
#     cc=Gauge().filter(Gauge.id.in_(children)).all()
#     g.children.extend(cc)

#     create_heatmap_machine(g)
#     db.session.commit()
#     rb = SQLAlchemyResponseBuilder(g)
#     return JsonResponse(rb,content_type='application/json')

# @bp.route('/supers/remove',methods=['post'])
# def post_super_remove():
#     gauge_id = request.json.get("gauge_id")
#     children = request.json.get("items")
#     children = [int(float(c)) for c in children]
#     print("gauge id: ", gauge_id)
#     print("items: ", children)
#     g=Gauge().where(id=gauge_id).first()
#     cc=Gauge().filter(Gauge.id.in_(children)).all()
#     for c in cc:
#         g.children.remove(c)

#     create_heatmap_machine(g)
#     db.session.commit()
#     rb = SQLAlchemyResponseBuilder(g)
#     return JsonResponse(rb,content_type='application/json')

from mdhwebserver.utils.validation.circular import check_for_circular_references

@bp.route('/supers',methods=['post'])
@is_authorized()
def add_edit_super_machine():
    # New machine
    mac=appconfigs.mdh.macAddress.replace(':','')
    ts=int(time.time())
    machine_id = f'machine-{mac}-{ts:x}'

    # Edit macine
    gauge_id = request.json.get("gauge_id")

    new_machine_name = request.json.get("new_machine_name")
    a = None
    g = None
    assembly_name=request.json.get("assembly")
    items = request.json.get("items")
    if items is not None:
        items = [int(float(c)) for c in items]

    if gauge_id is None and new_machine_name is None:
        return JsonResponse(MDHErrorResponseBuilder(message="Must Select a Machine or Enter a Machine Name"))

    if assembly_name is None:
        return JsonResponse(MDHErrorResponseBuilder(message="Please Enter or Select an Assembly"))
    
    if (gauge_id):
        g=Gauge().where(id=gauge_id).options(sa.orm.joinedload(Gauge.heatmap_item)).first()
    
    if (assembly_name):
        a = Assembly().where(display_name=assembly_name).first()
        if a is None:
            print("No Assembly found.")
            a = Assembly.create(**{
                'name': assembly_name,
                'display_name': assembly_name,
                'buildable': True
            })

    if g is None:
        g = Gauge().create(**{
            'machine_name': new_machine_name,
            'machine_id': machine_id,
            'assembly_id': a.id,
            'onboarded': True,
            'machine_view_type' : 'super'
        })
    else:
        if (a is not None):
            g.assembly_id = a.id
    
        
    
    if (new_machine_name):
        g.machine_name = new_machine_name
        if g.heatmap_item:
            g.heatmap_item.name = new_machine_name

    if items is not None and len(items) > 0:
        cc=Gauge().filter(Gauge.id.in_(items)).all()
        check_result = check_for_circular_references(g, cc)
        if check_result:
            msg = f"{check_result.machine_name} cannot be added to Machine.  Circular references are not allowed."
            db.session.rollback()
            return JsonResponse(MDHErrorResponseBuilder(message=msg))

        g.children = cc

    create_heatmap_machine(g)
    db.session.commit()
    rb = SQLAlchemyResponseBuilder(g).camel(False)
    return JsonResponse(rb,content_type='application/json')

# @bp.route('/supers/modify',methods=['post'])
# @is_authorized()
# def post_super_update():
#     gauge_id = request.json.get("gauge_id")
#     items_to_add = request.json.get("items_to_add")
#     items_to_remove = request.json.get("items_to_remove")
#     items_to_add = [int(float(c)) for c in items_to_add]
#     items_to_remove = [int(float(c)) for c in items_to_remove]

#     if items_to_add is not None and len(items_to_add) > 0:
#         print("---------------add items:----------------- ", items_to_add)
#         g=Gauge().where(id=gauge_id).first()
#         cc=Gauge().filter(Gauge.id.in_(items_to_add)).all()
#         g.children.extend(cc)
#     if items_to_remove is not None and len(items_to_remove) > 0:
#         print("----------------remove items:---------------- ", items_to_remove)
#         g=Gauge().where(id=gauge_id).first()
#         cc=Gauge().filter(Gauge.id.in_(items_to_remove)).all()
#         for c in cc:
#             g.children.remove(c)

#     create_heatmap_machine(g)
#     db.session.commit()
#     rb = SQLAlchemyResponseBuilder(g)
#     return JsonResponse(rb,content_type='application/json')

