from ..module import bp
from pathlib import Path
import os
import json
import time
import gzip
import shutil
import shlex
import subprocess
from sqlalchemy.orm import joinedload, selectinload
import logging
from mdhwebserver.appservices import database as db
from mdhwebserver import appconfigs

from mdhwebserver.models.models import Gauge, Assembly, DeviceDataItem, SubassemblyInstance, HeatmapItem

from http import HTTPStatus
from flask import (
    request,
    redirect,
    abort,
    send_file
)
from flask_jwt_extended import current_user
from mdhwebserver.wrappers.is_authorized import is_authorized

from mdhwebserver.builders.responses import JsonResponse
from mdhwebserver.builders.response_builder import SQLAlchemyResponseBuilder, MDHResponseBuilder, MDHErrorResponseBuilder
from mdhwebserver.builders import ValidationErrorResponseBuilder
from mdhwebserver.exceptions import ValidationException


@bp.route('/views',methods=['get'])
@is_authorized()
def get_views():
    g = Gauge().where(machine_view_type='view').all()
    rb = SQLAlchemyResponseBuilder(g)
    db.session.commit()
    return JsonResponse(rb,content_type='application/json')


@bp.route('/views/<view_id>',methods=['get'])
@is_authorized()
def get_view(view_id):
    g = Gauge().where(id=view_id).first()
    rb = SQLAlchemyResponseBuilder(g)
    db.session.commit()
    return JsonResponse(rb,content_type='application/json')

def create_heatmap_view_machine(machine, view_type='super-machine', default_view = "All Machines"):
  print("--------- Creating heatmap view machine.")
  if not machine.heatmap_item:
    heatmap_machine_item = HeatmapItem.create(name=machine.machine_name, view_type=view_type, gauge_id=machine.id)
    all_machines  = HeatmapItem.query.filter(HeatmapItem.name == default_view).first()
    all_machines.rchildren.append(heatmap_machine_item)
  else:
    heatmap_machine_item = machine.heatmap_item
  print("-------- Calling sync machine..")
  heatmap_machine_item.sync_machine(machine)
  machine.color_update()



# @bp.route('/views',methods=['post'])
# @is_authorized()
# def post_view():
#     machine_name=request.json.get("machine_name")
#     assembly_name=request.json.get("assembly")
#     print("Assembly_name: ", assembly_name)
#     mac=appconfigs.mdh.macAddress.replace(':','')
#     ts=int(time.time())
#     machine_id = f'machine-{mac}-{ts:x}'
#     a = None
#     if (assembly_name):
#         a = Assembly().where(name=assembly_name).first()
#     if a is None:
#         print("No Assembly found.")
#         a = Assembly.create(**{
#             'name': assembly_name,
#             'display_name': assembly_name
#         })
#     g = Gauge().create(**{
#         'machine_name':machine_name,
#         'machine_id': machine_id,
#         'assembly_id': a.id,
#         'onboarded': True,
#         'machine_view_type' : 'view'
#     })
#     create_heatmap_view_machine(g, view_type='machine')
#     db.session.commit()
#     rb = SQLAlchemyResponseBuilder(g)
#     return JsonResponse(rb,content_type='application/json')

@bp.route('/views',methods=['post'])
@is_authorized()
def add_edit_machine_view():
    # New machine
    mac=appconfigs.mdh.macAddress.replace(':','')
    ts=int(time.time())
    machine_id = f'machine-{mac}-{ts:x}'
    
    # Edit machine
    gauge_id = request.json.get("gauge_id")

    new_machine_name = request.json.get("new_machine_name")
    a = None
    g = None
    assembly_name=request.json.get("assembly")
    items = request.json.get("items")
    if items is not None:
        items = [int(float(c)) for c in items]

    if gauge_id is None and new_machine_name is None:
        return JsonResponse(MDHErrorResponseBuilder(message="Must Select a Machine or Enter a Machine Name"))

    if assembly_name is None:
        return JsonResponse(MDHErrorResponseBuilder(message="Please Enter or Select an assembly"))
    
    if (gauge_id):
        g=Gauge().where(id=gauge_id).options(joinedload(Gauge.heatmap_item)) \
        .options(joinedload(Gauge.assembly).options(selectinload(Assembly.subassemblies))) \
        .first()

    if (assembly_name):
        a = Assembly().where(display_name=assembly_name).first()
        if a is None:
            print("No Assembly found.")
            a = Assembly.create(**{
                'name': assembly_name,
                'display_name': assembly_name,
                'buildable': True
            })

    if (g is None):
        g = Gauge().create(**{
            'machine_name': new_machine_name,
            'machine_id': machine_id,
            'assembly': a,
            'onboarded': True,
            'machine_view_type' : 'view'
        })
    else:
        if (a is not None):
            # only for if the assembly name changes
            g.assembly_id = a.id

    if (new_machine_name):
        g.machine_name = new_machine_name
        if g.heatmap_item:
            g.heatmap_item.name = new_machine_name

    if items is not None and len(items) > 0:
        print("-------------------------------------------------- before subassembly loop")
        current_subassemblies = {x.id for x in g.assembly.subassemblies}
        cc=DeviceDataItem().filter(DeviceDataItem.id.in_(items)).all()
        g.device_data_items = cc

        subassembly_list = {x.subassembly_instance_id for x in g.device_data_items}
        new_subassemblies = subassembly_list.difference(current_subassemblies)

        if new_subassemblies:
            subassemblies = SubassemblyInstance().filter(SubassemblyInstance.id.in_(new_subassemblies)).all()
            g.assembly.subassemblies.extend(subassemblies)

    create_heatmap_view_machine(g, view_type='machine')            
    db.session.commit()
    rb = SQLAlchemyResponseBuilder(g).camel(False)
    return JsonResponse(rb,content_type='application/json')

# @bp.route('/views/add_gauges',methods=['post'])
# @is_authorized()
# def add_gauges():
#     gauge_id = request.json.get("gauge_id")
#     items_to_add = request.json.get("items_to_add")
#     items_to_add = [int(float(c)) for c in items_to_add]
#     print("gauge id: ", gauge_id)
#     print("items_to_add: ", items_to_add)
#     g=Gauge().where(id=gauge_id).first()
#     current_subassemblies = {x.id for x in g.assembly.subassemblies}
#     cc=DeviceDataItem().filter(DeviceDataItem.id.in_(items_to_add)).all()
#     # g.device_data_items = []
#     g.device_data_items.extend(cc)
#     subassembly_list = {x.subassembly_instance_id for x in g.device_data_items}
#     new_subassemblies = subassembly_list.difference(current_subassemblies)
#     print("Current_subassemblies: ", current_subassemblies)
#     print("subassembly_instance_list: ", subassembly_list)
#     print("new assemblies: ", new_subassemblies)

#     if new_subassemblies:
#         subassemblies = SubassemblyInstance().filter(SubassemblyInstance.id.in_(new_subassemblies)).all()
#         g.assembly.subassemblies.extend(subassemblies)
#     # dd = g.device_data_items  # Force to load the data items.
#     create_heatmap_view_machine(g, view_type='machine')
#     db.session.commit()
#     rb = SQLAlchemyResponseBuilder(g)
#     return JsonResponse(rb,content_type='application/json')

# @bp.route('/views/remove',methods=['post'])
# @is_authorized()
# def post_views_remove():
#     gauge_id = request.json.get("gauge_id")
#     device_data_items = request.json.get("items")
#     device_data_items = [int(float(c)) for c in device_data_items]
#     print("gauge id: ", gauge_id)
#     print("items: ", device_data_items)
#     g=Gauge().where(id=gauge_id).first()
#     cc=DeviceDataItem().filter(DeviceDataItem.id.in_(device_data_items)).all()
#     for c in cc:
#         g.device_data_items.remove(c)

#     create_heatmap_view_machine(g, view_type='machine')
#     db.session.commit()
#     rb = SQLAlchemyResponseBuilder(g)
#     return JsonResponse(rb,content_type='application/json')