from ..module import bp
from mdhwebserver.builders import SQLAlchemyResponseBuilder
from mdhwebserver.builders import JsonResponse
from mdhwebserver.utils.sqlalchemy import getModelByName
from sqlalchemy import text
from sqlalchemy.orm.mapper import Mapper

from flask import (
    request,
    abort,
    jsonify
)

from mdhwebserver.appservices import database as db
from sqlalchemy import inspect
from mergedeep import merge

from flask_jwt_extended import current_user
from mdhwebserver.wrappers.is_authorized import is_authorized

@bp.route('', methods=['GET'])
def index():
    return 'testing'

def add_with(model,with_list):
    if with_list:
        split_list = with_list.split(',')
        schema = make_schema(model, split_list)
        return model.with_(schema)
    return model

def get_relationship(model,relationship):
    a = getattr(model,relationship)
    return a

def make_dot_schema(model,dot_list):
    print("dot list: ",dot_list)
    if not dot_list:
        return {}
    relationship=dot_list[0]
    a = getattr(model,relationship)
    c = inspect(a)
    print("c: ",c)
    print("c argument: ", c.property.argument)
    print("type: ", type(c.property.argument))
    if isinstance(c.property.argument,str):
        nm = getModelByName(c.property.argument)
    elif isinstance(c.property.argument,Mapper):
        x = inspect(c.property.argument)
        nm = x.class_
    else:
        nm = c.property.argument
    return {a: make_dot_schema(nm,dot_list[1:])}

def make_schema(model,split_list):
    schema={}
    for item in split_list:
        print("item: ",item)
        dot_list=item.split('.')
        r=make_dot_schema(model,dot_list)
        print("r: ",r)
        # schema = {**schema,**r}
        schema = merge(schema,r)
    return schema

@bp.route('/objects/<obj>',methods=['GET'])
@is_authorized()
def objects(obj):
    print("--------------- Objectable object ------------------- ")
    srcmodel = model = getModelByName(obj)
    if model is None:
        abort(404,"No model found")
        #return jsonify({"error":"No Model found"})
#    model = add_where(model,request.args.get("where",None))
    model = add_with(model,request.args.get("with",None))
    camelKeys = bool(int(request.args.get('camel',True)))
    # add filter
    f = request.args.get("filter",None)
    # This is just for testing
    if (f):
        print("filter is ",f)
#        model=model.query.filter(text(f))
        model=model.filter(text(f))
    #kludgeany filter just for testing.
    anyfilter = request.args.get("any",None)
    if anyfilter:
        print("Have any filter for: ",any)
        attr=getattr(srcmodel,anyfilter)
        if attr:
            model=model.filter(attr.any())
    qresult = model.all()
    if qresult == None:
        return jsonify({"error":"No object found"})
#    rb = SQLAlchemyResponseBuilder(qresult).exclude(['data'])
    rb = SQLAlchemyResponseBuilder(qresult).camel(camelKeys)
    return JsonResponse(rb)


@bp.route('/object/<obj>/<_id>',methods=['GET'])
@is_authorized()
def single_object(obj,_id):
    model = getModelByName(obj)
    camelKeys = bool(int(request.args.get('camel',True)))
    if model is None:
        abort(404,"No Model found")
    model = add_with(model,request.args.get("with",None))
    qresult = model.find(_id)
    if qresult is None:
        abort(404,"No objects found")
    rb = SQLAlchemyResponseBuilder(qresult).camel(camelKeys)
    return JsonResponse(rb)


@bp.route('/qobject/<obj>',methods=['GET'])
@is_authorized()
def single_qobject(obj):
    model = getModelByName(obj)
    if model is None:
        return jsonify({"error":"No Model found"})
    model = add_with(model,request.args.get("with",None))
    #Add filter
    f = request.args.get("filter",None)
    # This is just for testing
    if (f):
        print("filter is ",f)
        model=model.filter(text(f))
    qresult = model.first()
    if qresult is None:
        return jsonify({"error":"No objects found"})
    rb = SQLAlchemyResponseBuilder(qresult).exclude(['data'])
    return JsonResponse(rb)


