import json
from json.decoder import JSONDecodeError
import logging

from mdhwebserver.exceptions.pumpsense_error import (
    MalformedPayloadError, 
    PumpsenseError
)
#from mdhwebserver.config.config import config_props
#from mdhwebserver.config import flask_app_config

#flask_app_config.create(config_props['server'])
#flask_app_config.app.app_context().push()

logger = logging.getLogger('celery_task')

from mdhwebserver.appservices import celery
from mdhwebserver.managers.task_manager import task_register
from ...pumpsense.processor import insert_gauge_data, insert_expert_data

@task_register("poweranalyzer.gauge")
@celery.task
def process_gauge_data(gauge_type, item, payload):
    logger.debug('Inside submit_task...')
    try:
        parsed_payload = json.loads(payload)
        #ingestion_service.ingest(parsed_payload)
        insert_gauge_data(parsed_payload,gauge_type)

    except JSONDecodeError as e:
            logger.error(f'Error while parsing payload: {e}', exc_info=True)
            raise MalformedPayloadError(
                        payload, 
                        'Payload is not in the expected format')

    except PumpsenseError as e:
        logger.error(f'exception occurred while saving gauge data: {e}', exc_info=True)
        raise e
    
    except:
        logger.error("exception occurred", exc_info=True)
        raise


@task_register("poweranalyzer.expert")
@celery.task
def process_expert_data(gauge_type, item, payload):
    try:
        parsed_payload = json.loads(payload)
        insert_expert_data(parsed_payload, gauge_type)
    
    except JSONDecodeError as e:
            logger.error(f'Error while parsing payload: {e}', exc_info=True)
            raise MalformedPayloadError(
                        payload, 
                        'Payload is not in the expected format')
    
    except:
        logger.error("exception occurred", exc_info=True)
        raise
    
