
from datetime import datetime, timezone
import simplejson as json
from mdhwebserver.appservices import database as db
from mdhwebserver.managers.service_manager import task_manager

from mdhwebserver.models.models import (
    ExpertDataGroup,
    ExpertDataItem,
    ExpertDataPoint,
    Gauge,
    PmFunction,
    DeviceDataItem,
    Assembly,
    SubassemblyInstance,
    Datahub,
    GaugeGroup,
    TrendMetaData,
    HeatmapItem,
    TrendMetaData,
    DeviceUnitDataPoint,
    GaugeRange,
    DeviceDataItemUnitDataPointAssociation
)

import time
from sqlalchemy.orm import joinedload, selectinload
from sqlalchemy.orm.attributes import flag_modified

from mdhwebserver.utils.heatmap import get_max_color, make_color_list

from mdhwebserver.models.SiUnitModels import UnitTranslate

from mdhwebserver.utils.listutils import (
    dict_map_extract,
    find_value
)
from mdhwebserver.utils.virtual_gauges.virtual_gauges import update_virtual_gauges_from_trends, update_device_data_items_kpis

# def getSubassemblyInstance(name,assembly_name,display_name = None):
#     subassemblies = SubassemblyInstance.where(name=name).first()
#     if not subassemblies:
#         print("Subassembly is None: ")
#         assembly = Assembly.firstOrCreate2({"name":assembly_name}, {"display_name": assembly_name})
#         if display_name is None:
#             display_name=name
#         subassemblies = SubassemblyInstance.create(**{
#             'name': name,
#             'display_name': display_name,
#             'gauge_type_id': 1
#         })
#         assembly.subassemblies.append(subassemblies)
#         assembly.save()
#     return subassemblies

def get_pm_function2(subassemblies, gauge):
    pm_func_name=gauge.get('output_name')
    for func in subassemblies.pm_functions:
        if func.name == pm_func_name:
            return func
    print("******** PM Func is None: ")
    if not gauge.get('create',False):
        return None
    sid=subassemblies.id # Stubbed until we handle this.
    units=gauge.get('units')
    #kludge for multi value for now.
    # since pm functions only support one data type then we will set it to the first in the dict.
    # If the units is none we will set it to value.
    if isinstance(units,dict):
        try:
            unit = next(iter(units.values()))
        except:
            unit="value"
    else:
        unit = units
    unit_translate=UnitTranslate.where(name=unit).first()
    unit_system = unit_translate.unit_system if unit_translate else 'mks'
    if 'bar_kpis' in gauge:
        bar_kpis = gauge.get('bar_kpis')
    elif 'stackedbar_kpis' in gauge:
        bar_kpis = gauge.get('stackedbar_kpis')
    else:
        bar_kpis = {}
    PmFunc = PmFunction.create(**{
        'name': pm_func_name,
        'display_name': pm_func_name,
        'subassembly_instance_id': sid,
        'trend_existence': gauge.get('trendExistence',False),
        'mro': gauge.get('mro',False),
        'gauge_type': gauge.get('gauge_type'),
        'gauge_base_image': 'default_base_image',
        'unit_system': unit_system,
        'units': unit,
        'x_unit': bar_kpis.get('x_unit'),
        'y_unit': bar_kpis.get('y_unit'),
        'y_trend_min': gauge.get('trends',{}).get('trendYMin'),
        'y_trend_max': gauge.get('trends',{}).get('trendYMax'),
        'set_color': gauge.get('set_color',False),
        'gauge_style': gauge.get('gauge_style','standard')


    },commit=False)
    return PmFunc

propagate_gauge_colors = {}
def getSubassemblyInstance(name,assembly_name,display_name = None):
    subassemblies = SubassemblyInstance.where(name=name).first()
    if not subassemblies:
        print("Subassembly is None: ")
        assembly = Assembly.firstOrCreate2({"name":assembly_name}, {"display_name": assembly_name})
        if display_name is None:
            display_name=name
        subassemblies = SubassemblyInstance.create(**{
            'name': name,
            'display_name': display_name
        })
        assembly.subassemblies.append(subassemblies)
        assembly.save()
    return subassemblies

# def get_pm_function2(subassemblies, gauge):
#     pm_func_name=gauge.get('output_name')
#     for func in subassemblies.pm_functions:
#         if func.name == pm_func_name:
#             return func
#     print("******** PM Func is None: ")
#     sid=subassemblies.id # Stubbed until we handle this.

#     PmFunc = PmFunction.create(**{
#         'name': pm_func_name,
#         'display_name': pm_func_name,
#         'subassembly_instance_id': sid,
#         'trend_existence': gauge.get('trendExistence',False),
#         'mro': gauge.get('mro',False),
#         'gauge_type': gauge.get('gauge_type'),
#         'gauge_base_image': 'default_base_image',
#         'unit_system': 'mks',
#         'units': gauge.get('units',None)
#     })
#     return PmFunc


def get_pm_function(key):
    s = key.split(':')
    pm_func_name = s[1].strip()
    subassembly_name = s[0].strip()
    subassemblies = SubassemblyInstance.where(name=subassembly_name).first()
    if not subassemblies:
        return None
    for func in subassemblies.pm_functions:
        if func.name == pm_func_name:
            return func
    return None
    # PmFunc = PmFunction.where(name=pm_func_name) \
    #    .first()


def get_gauge_group_function(PmFunc):
    pmFuncId = PmFunc.id
    pmFuncResp = PmFunction.where(id=pmFuncId).first()
    if not pmFuncResp:
        return None
    ggId = pmFuncResp.gauge_group_id
    gaugeGroup = GaugeGroup.where(id=ggId).first()
    if not gaugeGroup:
        return None
    return gaugeGroup

def make_bar_kpis(value):
    print("----------------- Making name/value kpis")
    names = value.get('name')
    values = value.get('value')
    kpis = {}

    if names and values and len(names) == len(values):
        print("Entering name/value kpi creatin..")
        for i in range(len(names)):
            kpis[names[i]] = values[i]
    return kpis


def fill_gauge(G, gauge_data, data_timestamp):
    # print(gauge_data)
    trends_used_set = set()
    for key, value in gauge_data.items():
        # s = key.split(':')
        # pm_func_name = s[1].strip()
        # PmFunc = PmFunction.where(name=pm_func_name) \
        #    .first()
        # print("PM Func: ",pm_func_name)
        # print(f"Subassembly for PmFunc is: {PmFunc.subassembly_instance_name} vs {s[0]}")
        # print("Subassembly id is: ",PmFunc.subassembly_instance_id)
        value['success'] = convert_true_false(value.get('success',False))
        PmFunc = get_pm_function(key)
        if PmFunc == None:
            print(f'********** Could not find pm func for key: {key}')
            continue
        GaugeGroup = get_gauge_group_function(PmFunc)
        if GaugeGroup is None:
            ggid = None
            ggname = None
        else:
            ggid = GaugeGroup.id
            ggname = GaugeGroup.display_name

        device_data = G.device_data_item(
            PmFunc.id, PmFunc.subassembly_instance_id)
        mart = value.get('mart', {})
        reason = value.get('reason')
        reason = reason[0:512] if reason else None
        max_gauge_color = None
        gauge_colors = value.get('color')
        if gauge_colors is not None:
            max_gauge_color = get_max_color(gauge_colors)
        if device_data is None:
            reason = value.get('reason')
            reason = reason[0:512] if reason else None
            device_data = DeviceDataItem(**{
                # 'data': value,
                # 'data': {},
                'gauge_id': G.id,
                'pm_func_id': PmFunc.id,
                'pm_function_name': PmFunc.name,
                'subassembly_instance_id': PmFunc.subassembly_instance_id,
                'subassembly_instance_name': PmFunc.subassembly.name,
                'gauge_group_id': ggid,
                'gauge_group_name': ggname,
                'colors': value.get('color'),
                'values': value.get('value'),
                'reason': reason,
                'activate_hourglass': value.get('activateHourGlass'),
                'feedback': value.get('feedback'),
                'user_input': value.get('userInput'),
                'hide_from_group': value.get('hideFromGroup'),
                'message': value.get('message'),
                'yellow_threshold': mart.get('yellow_threshold'),
                'red_threshold': mart.get('red_threshold'),
                'color': max_gauge_color
            })
            G.device_data_items.append(device_data)
        else:
            device_data.fill(**{
                # 'data': value,
                # 'data': {},
                'colors': value.get('color'),
                'values': value.get('value'),
                'reason': reason,
                'activate_hourglass': value.get('activateHourGlass'),
                'feedback': value.get('feedback'),
                'user_input': value.get('userInput'),
                'hide_from_group': value.get('hideFromGroup'),
                'message': value.get('message'),
                'yellow_threshold': mart.get('yellow_threshold'),
                'red_threshold': mart.get('red_threshold'),
                'color': max_gauge_color
            })
        # print("----------------- device_data: ",device_data)
        mart = value.get('mart', None)
        if mart == None:
            print("----------------- Making name/value kpis")
            names = value.get('name')
            values = value.get('value')
            if names and values and len(names) == len(values):
                print("Entering name/value kpi creatin..")
                kpis = {}
                for i in range(len(names)):
                    kpis[names[i]] = values[i]
                device_data.fill(**{'kpis':kpis})
            continue

        # print("---------------- Mart type: ",type(mart))
        if not isinstance(mart, dict):
            continue
#        trend_map = mapper(PmFunc.trend_meta,"name")
        trend_map = PmFunc.trend_meta_mapped
#        for key,value in trend_map.items():
#            print(f'mapper: {key} -- {value}')

        '''    
        if mart != None and PmFunc != None:
#            names = pluck(PmFunc.trend_meta,"name")
            names = PmFunc.trend_meta.pluck("name")
            #print("names: ",names)
            trends = dict_extract(mart,names);
            configs = dict_not_extract(mart,names);
            if 'unit' not in configs:
                configs['unit'] = ''
        else:
            trends = {}
            configs = {'unit':''}
        '''
        trends2 = {}
        configs2 = {'unit': ''}

        kpis = None
        if mart != None:
            kpis = {}
            for k, v in mart.items():
                # print(f'Mart -- key {k} -- {v}\n')
                trend_item = trend_map.get(k, None)
                if trend_item:
                    kpis[k] = v
                else:
                    configs2[k] = v

 #       print("Trends1: ",trends)
 #       print("configs1: ",configs)
            print("----------------------- kpis: ", kpis)
            if kpis and value['success']:
                device_data.fill(**{'kpis':kpis})
                units = PmFunc.units
                add_kpis(G,PmFunc,kpis,data_timestamp,units)
        # Let's add kluge for a bar graph.
        if PmFunc.gauge_type == 'bar' and value['success']:
            print("Have bar gauge for: ",PmFunc.name)
            print("kpis set as: ", kpis)
            names = value.get('name')
            print("names: ",names)
            values = value.get('value')
            print("values: ",values)
            bar_data = {names[i]: values[i] for i in range(len(names))}
            print("bar data: ",bar_data)
            device_data.fill(**{'kpis':bar_data})
        trends_used_set |= set(PmFunc.trend_meta_mapped.values())
        sync_device_data_items_trend_metas(device_data,PmFunc.trend_meta_data)
    print("-------------------- UPDATING VIRTUAL GAUGES ------------------------")
    print("XXXXXXXXXXXXXx Trends used set: ", trends_used_set)
    update_virtual_gauges_from_trends(G,trends_used_set)


def extract_gauge(payload):
    header = payload['header']
    assemblyName = header['assemblyName']
    # assembly = Assembly.query.filter(Assembly.name == assemblyName).first()
    assembly = Assembly.firstOrCreate2({'name': assemblyName}, {"display_name": assemblyName})
    print("Assembly name is: ",assemblyName)
    values = dict_map_extract(header, {
        'pumpNumber': 'pump_number',
        'machineId': 'machine_id',
        'machineName': 'machine_name',
        'companyName': 'company_name',
        'locationName': 'location_name',
    }, True)
#    values['data'] = payload['gauge']
    # values['created_on'] = datetime.fromtimestamp(header['timestamp'], tz=timezone.utc)
    print("header timestamp: ", header['timestamp'])
    values['updated_on'] = datetime.fromtimestamp(
        int(header['timestamp']), tz=timezone.utc)
    values['assembly_id'] = assembly.id
    return values


def make_machine_entry(values,gaugeTypeName,meta_data):
    data_timestamp = values['updated_on']
    G = Gauge.query.filter(Gauge.machine_id == values['machine_id']).first()
    if G and not G.datahub_id and meta_data:
        dh = Datahub.query.filter(
            Datahub.ip_address == meta_data['remote_addr']).first()
        if dh:
            values['datahub_id'] = dh.id
    if G is None:
        # G = Gauge.updateOrCreate(keys,**values)
        G = Gauge.create(**values)
    else:
        print("updated on: ", G.updated_on.timestamp())
        print("Values updated on. ", values['updated_on'].timestamp())
        if G.updated_on.timestamp() < values['updated_on'].timestamp():
            x=values.pop('machine_name')
            G.fill(**values)
        else:
            print("********** ERROR Out of sequence timestamp *************")
    return G

trend_colors = [
    'pink',
    'cyan',
    'blue',
    'purple',
    'green',
    'orange',
    'lightblue',
    'lightgreen'
    'black',
    'deeppink'
    'coral',
    'lavender',
    'magenta',
    'indigo',
    'darkgreen',
    'olive',
    'aqua',
    'aquamarine',
    'darkblue',
    'brown'
]

def get_unit_name(name,units):
    if isinstance(units,dict):
        return units.get(name)
    return units

def create_trend_item(PmFunc,kpi_name,units,color):
    unit = get_unit_name(kpi_name,units)
    if unit is None:
        return None
    unit_translation = UnitTranslate.where(name=unit).first()
    fps_units = unit
    mks_units = unit
    if unit_translation:
        if unit_translation.unit_system == 'fps':
            mks_units = unit_translation.translate_name
        else:
            fps_units = unit_translation.translate_name
    trend_item = TrendMetaData.create(**{
        'name': kpi_name,
        'display_name': kpi_name,
        'color': color,
        'units': unit,
        'mks_units': mks_units,
        'fps_units' : fps_units

            },commit=False)
    return trend_item

def add_kpis(G,PmFunc,kpis,data_timestamp, units):
    if kpis is None:
        return
    trend_map = PmFunc.trend_meta_mapped
    print("trend_map: ", trend_map)
    kpi_count = len(trend_map)
    print("---------------- KPI COUNT: ",kpi_count)
    for kpi_name,v in kpis.items():
        if kpi_name not in trend_map:
            trend_item = create_trend_item(PmFunc,kpi_name,units,trend_colors[kpi_count % len(trend_colors)])
            if trend_item is None:
                print(f"Unable to create trend_meta_data for {PmFunc.name} -- {kpi_name}")
                continue
            PmFunc.trend_meta_mapped[kpi_name] = trend_item
            PmFunc.trend_meta_data.append(trend_item)
            kpi_count += 1
        else:
            trend_item = trend_map[kpi_name]
        if v is None:
            continue
        unit = trend_item.units
        print("type is: ",type(v))
        # if not isinstance(v,str):
        #     v2=f'{v} {units}' if v is not None else None
        # else:
        #     v2 = v
        v2=f'{v} {unit}' if v is not None else None
        new_point = DeviceUnitDataPoint(**{
            'gauge_id': G.id,
            'timestamp': data_timestamp,
            # 'machine_id': G.machine_id,
            # 'trend_meta_data_id': trend_item.id,
            'trend_meta_data': trend_item,
            'value': v2
        })
        db.session.add(new_point)

def add_raw_kpis(G,PmFunc,kpis,units):
    if not kpis:
        return
    for timestamp,kpi in kpis.items():
        # Small kluge for converting timestamp to decimal.
        l1 = len(str(int(time.time())))
        l2 = len(str(timestamp))
        if l2 <= l1:
           fts = float(timestamp)
        else:
            fts = float(timestamp)/pow(10,l2-l1)
        #ts = datetime.fromtimestamp(float(timestamp)/1000, tz=timezone.utc)
        ts = datetime.fromtimestamp(fts, tz=timezone.utc)
        add_kpis(G,PmFunc,kpi,ts,units)

    # Just return the last one.
    return kpi
from mdhwebserver.modules.units.controllers.units import si_unit_convert

def make_threshold_color_list(device_data_item):
    colors=[]
    for trend_name,value in device_data_item.kpis.items():
        print("Checking trend: ", trend_name,value)
        color='green'
        for range in device_data_item.gauge_ranges:
            if range.trend_meta_data_name == trend_name:
                if range.start is None:
                    if value <= range.end:
                        color=range.color
                elif range.end is None:
                    if value > range.start:
                        color = range.color
                else:
                    if value > range.start and value <= range.end:
                        color=range.color
        colors.append(color)

    return colors

def convert_true_false(value):
    if isinstance(value,bool):
        return value
    if isinstance(value,str):
        if value == "true":
            return True
        return False

def sync_device_data_items_trend_metas(ddi,trend_metas):
    for tm in trend_metas:
        if tm not in ddi.trend_meta_datas:
            ddi.device_unit_data_point_associations.append(DeviceDataItemUnitDataPointAssociation(**{
                "gauge_id": ddi.gauge_id,
                "trend_meta_data_id": tm.id
            }))



def fill_gauge_format2(G,assembly_name,gauge_data,data_timestamp):
    print("******************** Fill Gauge Format 2 ***************************")
    trends_used_set = set()
    for cluster_name, value in gauge_data.items():
        print("Cluster name: ", cluster_name)
        print("Value: ", value)
        for pm_func_name, gauge in value.items():
            print("pm_func_name: ", pm_func_name)
            print("gauge: ", gauge)
            gauge['success'] = convert_true_false(gauge.get('success',False))
            # units=gauge.get('units')
            new_pm_function = False
            si=getSubassemblyInstance(cluster_name, assembly_name)
            PmFunc = get_pm_function2(si,gauge)
            if PmFunc == None:
                print(f'PmFunction not created: {pm_func_name}')
                continue

            # units=PmFunc.units
            units=gauge.get('units')
            print("PmFunc is: ",PmFunc)
            if PmFunc not in db.session.new:
                GaugeGroup = get_gauge_group_function(PmFunc)
            else:
                GaugeGroup = None
                new_pm_function=True
            if GaugeGroup is None:
                ggid = None
                ggname = None
            else:
                ggid = GaugeGroup.id
                ggname = GaugeGroup.display_name
            kpis = gauge.get('kpis')
            if PmFunc.gauge_style == 'raw' or PmFunc.gauge_style == "eventtrap":
                kpis = add_raw_kpis(G,PmFunc,gauge.get('raw_kpis'),units)
            if PmFunc.gauge_type == 'bar':
                if "bar_kpis" in gauge:
                    kpis = gauge.get('bar_kpis',{}).get('kpis')
            elif PmFunc.gauge_type == 'stackedbar':
                if "stackedbar_kpis" in gauge:
                    kpis = gauge.get('stackedbar_kpis',{})

            thresholds = gauge.get('thresholds',{})
            print("New: ",db.session.new)
            if PmFunc not in db.session.new:
                device_data = G.device_data_item(PmFunc.id, PmFunc.subassembly_instance_id)
            else:
                device_data = None
            reason = gauge.get('reason')
            reason = reason[0:512] if reason else None
            if device_data is None:
                device_data = DeviceDataItem(**{
                    # 'data': {},
                    'gauge_id': G.id,
                    # 'pm_func_id': PmFunc.id,
                    'pm_function': PmFunc,
                    'pm_function_name': PmFunc.name,
                    'subassembly_instance_id': PmFunc.subassembly_instance_id,
                    'subassembly_instance_name': si.name,
                    'gauge_group_id': ggid,
                    'gauge_group_name': ggname,
                    'colors': gauge.get('colors'),
                    'values': gauge.get('values'),
                    'reason': reason,
                    'activate_hourglass': gauge.get('activate_hour_glass'),
                    'feedback': gauge.get('feedback'),
                    'user_input': gauge.get('user_input'),
                    'hide_from_group': gauge.get('hide_from_group'),
                    'message': gauge.get('message'),
                    'yellow_threshold': thresholds.get('yellow_threshold'),
                    'red_threshold': thresholds.get('red_threshold'),
                    'kpis':kpis,
                    'color': None
                })
                G.device_data_items.append(device_data)
            else:
                device_data.fill(**{
                    'colors': gauge.get('colors'),
                    'values': gauge.get('values'),
                    'reason': reason,
                    'activate_hourglass': gauge.get('activate_hour_glass'),
                    'feedback': gauge.get('feedback'),
                    'user_input': gauge.get('user_input'),
                    'hide_from_group': gauge.get('hide_from_group'),
                    'message': gauge.get('message'),
                    'yellow_threshold': thresholds.get('yellow_threshold'),
                    'red_threshold': thresholds.get('red_threshold'),
                    'kpis':kpis,
                    # 'color': max_gauge_color
                })
            if gauge['success']:
                add_kpis(G,PmFunc,gauge.get('kpis'),data_timestamp,units)
            if 'ranges' in gauge and new_pm_function:
                gauge_range = gauge['ranges']
                for trend_name,range_item in gauge['ranges'].items():
                    if trend_name in PmFunc.trend_meta_mapped:
                        print("-- Range trend: ", trend_name)
                        counter=1
                        for r in range_item:
                            range_name=r.get('name') if 'name' in r else r['color']+str(counter)
                            convert_units = { }
                            mks_start = fps_start = convert_units['start'] = r.get('start')
                            mks_end = fps_end = convert_units['end']=r.get('end')
                            print("Convert: ",convert_units)
                            if PmFunc.trend_meta_mapped[trend_name].fps_units != PmFunc.trend_meta_mapped[trend_name].mks_units:
                                if PmFunc.trend_meta_mapped[trend_name].units != PmFunc.trend_meta_mapped[trend_name].fps_units:
                                    converted = si_unit_convert(PmFunc.trend_meta_mapped[trend_name].units,PmFunc.trend_meta_mapped[trend_name].fps_units,convert_units)
                                    fps_start = converted.get("start")
                                    fps_end = converted.get("end")
                                else:
                                    converted = si_unit_convert(PmFunc.trend_meta_mapped[trend_name].units,PmFunc.trend_meta_mapped[trend_name].mks_units,convert_units)
                                    mks_start = converted.get("start")
                                    mks_end = converted.get("end)")
                            new_range = GaugeRange(**{
                                'name': range_name,
                                'trend_meta_data_name': trend_name,
                                'trend_meta_data': PmFunc.trend_meta_mapped[trend_name],
                                'pm_function': PmFunc,
                                'color': r['color'],
                                'start': r.get('start'),
                                'end': r.get('end'),
                                'fps_start': fps_start,
                                'fps_end': fps_end,
                                'mks_start': mks_start,
                                'mks_end': mks_end

                            })
                            db.session.add(new_range)
                            # PmFunc.gauge_ranges.append(new_range)
                            counter += 1
            if PmFunc.gauge_ranges:
                if not device_data.gauge_ranges:
                    print("Adding ranges to device_data_item")
                    for range in PmFunc.gauge_ranges:
                        new_range = GaugeRange(**{
                            'name': range.name,
                            'trend_meta_data_name': range.trend_meta_data_name,
                            'trend_meta_data': range.trend_meta_data,
                            'color': range.color,
                            'start': range.start,
                            'end': range.end,
                            'fps_start': range.fps_start,
                            'fps_end': range.fps_end,
                            'mks_start': range.mks_start,
                            'mks_end': range.mks_end,
                            'device_data_item':device_data
                        })
                        db.session.add(new_range)
            if PmFunc.set_color:
                new_colors = make_threshold_color_list(device_data)
                device_data.colors = new_colors
            device_data.color = get_max_color(device_data.colors)
            trends_used_set |= set(PmFunc.trend_meta_mapped.values())
            sync_device_data_items_trend_metas(device_data,PmFunc.trend_meta_data)
    print("-------------------- UPDATING VIRTUAL GAUGES ------------------------")
    print("XXXXXXXXXXXXXx Trends used set: ", trends_used_set)
    update_virtual_gauges_from_trends(G,trends_used_set)

import traceback

def insert_gauge_data_format2(payload=None, gaugeTypeName=None, meta_data=None):
    print("Insert Gauge Data format 2")

    try:
        values = extract_gauge(payload)
        G = make_machine_entry(values,gaugeTypeName,meta_data)
        print("values: ", values)
        gauge_data = payload.get('gauges', None)
        assemblyName = payload['header']['assemblyName']

        print("gauge_data: ", gauge_data)
        data_timestamp = values['updated_on']
        if gauge_data:
            fill_gauge_format2(G,assemblyName,gauge_data,data_timestamp)

    except Exception as e:
        print("Format 2 exception: ", e)
        print(traceback.format_exc())
        db.session.rollback()
        raise e
    update_machine_color(G)
    G.save()
    db.session.commit()

def update_machine_color(machine):
    machine_color = machine.color
    machine.color_update()
    if machine_color != machine.color:
        propagate_machine_color(machine) # Only propagate if color changed.

    flag_modified(machine,"color")

def propagate_machine_color(machine):
    if not machine:
        return
    print(f"------------ Propagate machine color {machine.machine_name} -------------------")
    if not machine.parents:
        return
    
    for parent in machine.parents:
        machine_color = parent.color
        parent.color_update()
        if machine_color != parent.color:
            propagate_machine_color(parent)

def insert_gauge_data_format1(payload=None, gaugeTypeName=None, meta_data=None):
    print("Insert Gauge Data format 1")
    try:
        values = extract_gauge(payload)
    #    values['data'] = values['data']
        keys = {"machine_id": values['machine_id']}
        data_timestamp = values['updated_on']
        G = Gauge.query.filter(Gauge.machine_id == values['machine_id']).first()
        if G and not G.datahub_id and meta_data:
            dh = Datahub.query.filter(
                Datahub.ip_address == meta_data['remote_addr']).first()
            if dh:
                values['datahub_id'] = dh.id
        if G is None:
            # G = Gauge.updateOrCreate(keys,**values)
            G = Gauge.create(**values)
        else:
            print("updated on: ", G.updated_on.timestamp())
            if G.updated_on.timestamp() < values['updated_on'].timestamp():
                x=values.pop('machine_name')
                G.fill(**values)
            else:
                print("********** ERROR Out of sequence timestamp *************")
        gauge_data = payload.get('gauge', None)
        if gauge_data:
            fill_gauge(G,payload['gauge'],data_timestamp)
    except Exception as e:
            print("******** EXCEPTION on PAYLOAD LOGGING to /tmp/gauge_errors *********************")
            print(traceback.format_exc())
            #print(payload)
            ts=str(payload['header']['timestamp'])
            file_name=f"/tmp/gauge_errors/{ts}.json"
            with open(file_name,"w") as f:
                json.dump(payload,f,indent=4)
            db.session.rollback()
            raise e
    update_machine_color(G)
    G.save()
    db.session.commit()


def insert_gauge_data(payload=None, gaugeTypeName=None, meta_data=None):
    print("Insert Gauge Data")
    try:
        header = payload.get('header',{})
        print("Header: ", header)
        output_format = int(header.get('output_format',1))
        print("Output format: ", output_format)
        if output_format == 2:
            insert_gauge_data_format2(payload,gaugeTypeName,meta_data)
        else:
            insert_gauge_data_format1(payload,gaugeTypeName,meta_data)

    except Exception as e:
        print("Insert Gauge Data Exception: ", e)
        pass

def insert_expert_data(payload=None, gaugeTypeName=None):
    assemblyName = payload['subassemblyinstance']
    assembly = Assembly.query.filter(Assembly.name == assemblyName).first()
    values = {
        'machine_id': payload['machineId'],
        'machine_name': payload['machine'],
        'company_name': payload['company']
    }
    values['assembly_id'] = assembly.id
    # Stubbed values
    # Can't be null, but we don't want this either
    values['location_name'] = ""
    keys = {"machine_id": values['machine_id']}
    G = Gauge.updateOrCreate(keys, **values)
    edg = ExpertDataGroup.firstOrCreate(**{
        'name': payload['datatype'],
        'display_name': payload['datatype'],
        'assembly_id': assembly.id
    })

    for element in payload['data']:
        raw_timestamp = element['timestamp']
        epoch_value = datetime.fromtimestamp(
            raw_timestamp/1000000000, tz=timezone.utc)

        for key, param in element['items'].items():
            print(f"{key} = {param}")
            if key in edg.mapped_expert_data_items:
                print(f"Key in expert data items")
            else:
                print(f"Key not in expert data items")
                edg.mapped_expert_data_items[key] = ExpertDataItem(**{
                    'name': key,
                    'display_name': key
                })
            edg.mapped_expert_data_items[key].expert_data_points.append(ExpertDataPoint(**{
                '_timestamp': epoch_value,
                'value': param,
                'gauge_id': G.id,
                'machine_id': G.machine_id,
                'raw_timestamp': raw_timestamp
            }))

    G.save()
    db.session.commit()

color_ranks = {
    "green": 1,
    "yellow": 2,
    "red": 3
}
rank_colors = {
    "1": "green",
    "2": "yellow",
    "3": "red"
}


from mdhwebserver.utils.heatmap import get_max_color

def set_color(parent, ts):
    print("SET Color for: ", parent.name)
    if parent.last_color_update == ts:  # Already updated.
        return parent.color
    color_list=[]
    for child in parent.rchildren:
        if child.rchildren:
            print(f"has child with children: {child.name}")
            if child.last_color_update != ts:
                set_color(child,ts)
        color_list.append(child.color)
    print(f"Color list ({parent.name}): ",color_list)
    color = get_max_color(color_list)
    if color == parent.color:
        parent.color_count += 1
    else:
        parent.color_count = 0
    parent.color = color
    parent.last_color_update = ts
    print("Color is: ", parent.color)
    return parent.color

def process_machines_heatmap(machine):
    ts=int(time.time())
    parent_element_list = set()

    all_machines  = HeatmapItem.query.filter(HeatmapItem.name == 'All Machines').first()
    if all_machines is None:
        print("All machines does not exist in database: ")
        all_machines = HeatmapItem.create(name='All Machines', view_type="view")
        all_machines.save()

    m = machine.heatmap_item
    if m is None:
        m = HeatmapItem.create(name=machine.machine_name, view_type='machine')
        # all_machines.rchildren.append(m)
    
    if m not in all_machines.rchildren:
        all_machines.rchildren.append(m)
    m_id = machine.id
    m.gauge_id = m_id
    
    for ddi in machine.device_data_items:
        name=f"{ddi.subassembly_instance_name}: {ddi.pm_function_name}"
        pm_function_id = ddi.pm_func_id
        device_data_item_id = ddi.id
        if not ddi.pm_function.mro and ddi.pm_function.gauge_type != 'color':  #  All color gauges go to heatmap even if mro is disabled.  Don't like this but it will work for now.
            continue
        if not ddi.heatmap_item:
            ddi.heatmap_item = HeatmapItem.create(name=name,pm_function_id = pm_function_id,gauge_id = m_id,device_data_item_id=device_data_item_id, view_type='gauge')
        if ddi.heatmap_item not in m.rchildren:
            m.rchildren.append(ddi.heatmap_item)

        gm = ddi.heatmap_item
        parent_element_list.update(gm.rparents)
        gm.update_color(ddi.color)


    for item in parent_element_list:
        item.color_update(propagate=True, ts=ts)
        # machine_color = item.color
        # set_color(item,ts)
        # if item.color != machine_color:  # Color changed..
        #     propagate_heatmap_color(item)

    print("Parent element list: ", parent_element_list)


def insert_gauge_heatmap_data_format1(payload=None):
    machine = Gauge.query.filter(Gauge.machine_id == payload['header']['machineId']) \
        .options(
            selectinload(Gauge.heatmap_item),
            selectinload(Gauge.device_data_items).options(selectinload(DeviceDataItem.pm_function))
        ).first()
    if machine is None:
        print("machine does not exist in database: ", payload['header']['machineName'])
        return

    process_machines_heatmap(machine)
    db.session.commit()

def insert_gauge_heatmap_data(payload=None):
    insert_gauge_heatmap_data_format1(payload)

# def create_heatmap_propagate_task(heatmap_link_item=None):
#     print("create_heatmap_propagate_task - heatmap_link_item: ", heatmap_link_item)
#     if heatmap_link_item is None:
#         return

#     task_name = "heatmap.propagatecolor"
#     task = task_manager.get(task_name)
#     task.delay(heatmap_link_item)

def propagate_heatmap_color(hmi):
    if not hmi:
        return
    if not hmi.rparents:
        return
    
    for parent in hmi.rparents:
        # color_list = make_color_list(parent.rchildren)
        # print("--------- color list: ", color_list)
        # new_color = get_max_color(color_list)
        # if new_color != parent.color:
        #     parent.color = new_color
        parent.color_update()
        propagate_heatmap_color(parent)

