import json
from json.decoder import JSONDecodeError
import logging

from mdhwebserver.exceptions.pumpsense_error import (
    MalformedPayloadError, 
    PumpsenseError
)
#from mdhwebserver.config.config import config_props
#from mdhwebserver.config import flask_app_config

#flask_app_config.create(config_props['server'])
#flask_app_config.app.app_context().push()

logger = logging.getLogger('celery_task')

from mdhwebserver.appservices import celery, app
from mdhwebserver.managers.task_manager import task_register
from ..processor import insert_gauge_data, insert_gauge_heatmap_data

@task_register("pumpsense.gauge")
@celery.task(bind=True, default_retry_delay=15)
def submit_task(self,gauge_type,item,payload,meta_data=None):
    logger.debug('Inside submit_task..., retries: ' + str(self.request.retries))
    # global app
    try:
        with app.app_context():
            if isinstance(payload,str):
                parsed_payload = json.loads(payload)
            else:
                 parsed_payload = payload
            #ingestion_service.ingest(parsed_payload)
            insert_gauge_data(parsed_payload,gauge_type,meta_data)

    except JSONDecodeError as e:
            logger.error(f'Error while parsing payload: {e}', exc_info=True)
            raise MalformedPayloadError(
                        payload, 
                        'Payload is not in the expected format')

    except PumpsenseError as e:
        logger.error(f'exception occurred while saving gauge data: {e}', exc_info=True)
        raise e
    
    except Exception as exc:
        logger.error("exception occurred", exc_info=True)
        self.retry(exc=exc)
        raise

@task_register("pumpsense.gaugeheatmap")
@celery.task(bind=True, default_retry_delay=15)
def submit_heatmap_task(self,gauge_type,item,payload,meta_data=None):
    logger.debug('Inside submit_task..., retries: ' + str(self.request.retries))
    try:
        with app.app_context():
            if isinstance(payload,str):
                parsed_payload = json.loads(payload)
            else:
                 parsed_payload = payload
            #ingestion_service.ingest(parsed_payload)
            insert_gauge_heatmap_data(parsed_payload)

    except JSONDecodeError as e:
            logger.error(f'Error while parsing payload: {e}', exc_info=True)
            raise MalformedPayloadError(
                        payload, 
                        'Payload is not in the expected format')

    except PumpsenseError as e:
        logger.error(f'exception occurred while saving gauge data: {e}', exc_info=True)
        raise e
    
    except Exception as exc:
        logger.error("exception occurred", exc_info=True)
        self.retry(exc=exc)
        raise

#@task_register("heatmap.propagatecolor")
#@celery.task(bind=True, default_retry_delay=15)
#def propagate_color_task(self,item):
#    try:
#        with app.app_context():
#            propagate_gauge_color(item)
#    except Exception as e:
#         logger.error(f'Error while propagating color: {e}', exc_info=True)
         
        
