from ..module import bp
from appframework.builders.responses import JsonResponse
from appframework.builders.response_builder import MDHErrorResponseBuilder, DictResponseBuilder, MDHResponseBuilder 
from appframework.builders import ValidationErrorResponseBuilder
from appframework import appconfigs
from pynovatec.helpers.system import reboot_device
from flask import request,abort
import os
from werkzeug.utils import secure_filename
import subprocess
from getmac import get_mac_address as gma
import time
from cerberus import Validator
from mdhwebserver.managers.service_manager import task_manager

ota_file_path = "/tmp"
update_script= "/usr/src/run/scripts/ota_update.sh"

@bp.route('/updateSystemTime',methods=['POST'])
def updateSystemTime():
    thetime=request.data.decode('UTF-8')
    print(f'Time: {thetime}')
    newtime=int((int(thetime)/1000))	# divide by 1000 since that is what is being sent.
    if (type(thetime) != str) or (len(thetime) == 0):
        return JsonResponse(MDHErrorResponseBuilder("Invalid data"))
    ret = os.system("date -s @" + str(newtime) + " ; hwclock -w")
    if ret != 0:  # this isn't working date still returns 0.
        return JsonResponse(MDHErrorResponseBuilder("Failed to set time"))
    return JsonResponse({"success":True, "message":"Time Sync Up Successful"})


@bp.route('/datahubUpdate', methods=['POST'])
def datahubUpdate():
    try:
        incoming_mac = request.form.get("mac")
        if incoming_mac.lower() == gma():
            response = upload_file(request, ota_file_path)
            print("Response: ",response)
            if response["success"]:
                update_file =  ota_file_path + "/" + response['filename']
                if execute_update_script(update_script, update_file):
                        response = {"success": True, "message": "Update process started successfully!"}
                else:
                        response = MDHErrorResponseBuilder("Failed to start the update!")
            else:
                response = MDHErrorResponseBuilder(response["message"])
        else:
            response = MDHErrorResponseBuilder("MAC ID did not match!")
        return JsonResponse(response)
    except:
        raise


def upload_file(req, upload_path):
    try:
        response = {"success": False, "message": ""}
        if 'file' not in req.files:
            response["message"] = "No file found in the request"
        else:
            file = req.files['file']
            if not file:
                response["message"] = "Invalid file in request"
            elif file.filename == '':
                response["message"] = "No file sent in the request"
            elif not allowed_file(file.filename):
                response["message"] = "File has invalid extension"
            else:
                filename = secure_filename(file.filename)
                file.save(os.path.join(upload_path, filename))
                response['filename'] = file.filename
                response["success"] = True
        return response
    except:
        raise
    
def execute_update_script(update_script, ota_filepath):
    print("Update file: ",ota_filepath)
    subprocess.Popen(["/sbin/sysupgrade", ota_filepath])
    return True

def allowed_file(filename):
    try:
        allowed_exts = {'zip','gz'}
        return '.' in filename and filename.rsplit('.', 1)[1].lower() in allowed_exts
    except:
        raise


def get_interface_mac(interface: str):
    try:
        mac = gma(interface=interface).strip()
        if len(mac.strip()) != 17:
            raise Exception("Invalid MAC or interface {0}: {1}".format(interface, mac))
        return mac
    except:
        raise


@bp.route('/macaddress',methods=["GET"])
def get_device_macaddress():
    try:
        mac = get_interface_mac(appconfigs.system.interface)
    except:
        abort(400,f"No valid mac address found")
        raise
    return JsonResponse(DictResponseBuilder({"mac":mac}))


def validate_mac(incoming_mac):
    if incoming_mac.lower() != get_interface_mac(appconfigs.system.interface):
        raise Exception("Invalid mac address")

def validate_file(files):
        if 'file' not in files:
            raise Exception("No file found in the request")
        file = files['file']
        if not file:
            raise Exception("Invalid file in request")
        elif file.filename == '':
            raise Exception("No file sent in the request")
        elif not allowed_file(file.filename):
            raise Exception("File has invalid extension")

def validate_upgrade_file_name(filename):
    name_prefix=filename.split("-")[0]
    id=appconfigs.system.id
    if id != name_prefix:
        raise Exception(f'Wrong firmware for {appconfigs.system.id}')


@bp.route('/upgrade',methods=["POST"])
def datahub_update():
    try:
        force = request.form.get("force",False)
        # validate_mac(request.form.get("mac"))
        dh_list = request.form.get('dh_list')
        validate_file(request.files)
        file = request.files['file']
        filename = secure_filename(file.filename)
        if not force:
            validate_upgrade_file_name(os.path.basename(filename))
        saved_file=os.path.join(appconfigs.system.ota_file_path, filename)
        file.save(saved_file)
        print(dh_list)
        # execute_update_script(appconfigs.system.update_script,saved_file)
        
    except Exception as e:
        abort(400,str(e))
    print("Returning response: ")
    return JsonResponse(MDHResponseBuilder(message="Update process started successfully!"))


@bp.route('/reboot',methods=["GET"])
def reboot():
    reboot_device()
    return JsonResponse(MDHResponseBuilder(message="Rebooting..."))

@bp.route('/time',methods=['POST'])
def postTime():
    v = Validator({
        'epoch': {'required':True, 'type': ['integer','float']}
    })
    json = request.get_json()
    if not v.validate(json):
        return JsonResponse(ValidationErrorResponseBuilder(v))

    epoch = json.get("epoch")
    thetime=int(epoch)
    ret = os.system("date -s @" + str(thetime) + " ; hwclock -w")
    return JsonResponse({"success":True, "message":"Time Sync Up Successful"})


@bp.route('/time',methods=['GET'])
def getTime():
    return JsonResponse(DictResponseBuilder({"epoch":time.time()}))

