from flask import (
    Response,
    Blueprint,
    request,
    jsonify
)


from sqlalchemy.orm import (
        joinedload,
        subqueryload,
        load_only,
        selectinload
)

from http import HTTPStatus
import jsonpickle

from mdhwebserver.builders.responses import JsonResponse
from mdhwebserver.builders.response_builder import SQLAlchemyResponseBuilder, MDHErrorResponseBuilder, MDHResponseBuilder

from ..testing import bp
import json
from datetime import datetime, timezone

from mdhwebserver.models.models import (
        Gauge,
        GaugeTrend,
        PmFunction,
        SubassemblyInstance,
        DeviceDataItem,
        TrendMetaData,
        Assembly,
        ExpertDataGroup,
        ExpertDataItem,
        ExpertDataPoint,
        Datahub
)

from sqlalchemy import inspect

from mdhwebserver.appservices import database as db
#from mdhwebserver.application import app

#from mdhwebserver import appconfigs


@bp.route('',methods=['GET'])
def index():
    return "Testing -- "


@bp.route('/store',methods=['POST'])
def store():
    data=request.data.decode('UTF-8')
    print(data)
    payload = json.loads(data)
    values=dict()

    header = payload['header']
    values['pump_number'] = header.get('pumpNumber')
    values['machine_id'] = header['machineId']
    values['machine_name'] = header['machineName']
    values['company_name'] = header['companyName']
    values['location_name'] = header['locationName']
    values['created_on'] = datetime.fromtimestamp(header['timestamp'], tz=timezone.utc)
    values['updated_on'] = datetime.fromtimestamp(header['timestamp'], tz=timezone.utc)
    values['data'] = payload['gauge']
    print("values: ",values)

    G=Gauge(**values)
    db.session.add(G)
    db.session.commit()

    return "Storing..."


from mdhwebserver.modules.pumpsense.processor import insert_gauge_data, insert_expert_data, insert_gauge_heatmap_data
from mdhwebserver.mro.check_for_mro import check_for_mro

@bp.route('/updateOrCreate2',methods=['POST'])
def updateOrCreate2():
    # data=request.data.decode('UTF-8')
#    print(data)
    # payload = json.loads(data)
    payload = request.json
    insert_gauge_data(payload)
    insert_gauge_heatmap_data(payload)
    # MRO Testing.."
    check_for_mro(payload)
    return "Done"
#    return jsonify(G.toJson())



@bp.route('/firstOrCreate',methods=['POST'])
def firstOrCreate():
    data=request.data.decode('UTF-8')
#    print(data)
    payload = json.loads(data)
    values = extract_gauge(payload)
    G = Gauge.firstOrCreate(**values)
    db.session.commit()
    print("First or create G is: ",G)
    return "Done\n"
        

#from sqlalchemy.orm import Mapper
#from sqlalchemy.ext.declarative import mapper

#.options(joinedload(Gauge.trends.and_(GaugeTrend.pm_func_id==pm_func).and_(GaugeTrend.created_on > '2021-06-20'))\

@bp.route('/pmResults/<gauge>/<pm_func>',methods=['GET'])
def pmResults(gauge=None,pm_func=None):
#.options(joinedload(Gauge.trends.and_(GaugeTrend.pm_func_id==pm_func).and_(GaugeTrend.created_on > '2021-06-19')))\
    #jl=joinedload(Gauge.trends.and_(GaugeTrend.pm_func_id==pm_func).and_(GaugeTrend.created_on > '2021-06-19'))
    gauge_load=Gauge.trends.and_(GaugeTrend.pm_func_id==pm_func)
    gauge_load=gauge_load.and_(GaugeTrend.created_on.between('2021-06-03','2021-06-11'))
   # gauge_load=gauge_load.and_(GaugeTrend.created_on > '2021-06-19')
    #jl=joinedload(gauge_load).options(load_only("data"))
    jl=joinedload(gauge_load)
    jl2=joinedload(Gauge.device_data)
#    G = Gauge.where(id=gauge) \
#    m = mapper(Gauge,gauges,include_properties=[Gauge.id,Gauge.machine_id])
#        .options(jl2,load_only("id","machine_id"))\
#    G = Gauge.where(id=gauge)\
#        .filter(Gauge.company_name=='HellermannTyton')\
#        ._where_(Gauge.company_name=='HellermannTyton')\
    G = Gauge.query\
        ._with(gauge_load)\
        ._with("device_data")\
        .find(gauge)
#        .with_only_columns([Gauge.id,Gauge.machine_id])\
#        .with_entities(Gauge.id,Gauge.machine_id,Gauge.trends)\
#    Gauge.options(jonedload(Gauge.configs))
# Load after results example..
#    db.subqueryload(G.configs)
#    inspect(G).add_property(__mapper_args__ = {'include_properties': ['id','machine_id' ] })

#    return jsonify(G.to_dict(nested=True,exclude=['data']))
    return jsonify(G._to_dict(exclude=['data'],
        only=['company_name',{'device_data':['created_on']},{'trends':['created_on','data']}]))

@bp.route('/pmFunctions',methods=['GET'])
def pmFunctions():
    pm = PmFunction.query._with("trend_meta").all()
    result = [
        o._to_dict() for o in pm
    ]
    return jsonify(result)

@bp.route('/subassemblies',methods=['GET'])
def subassemblies():
    schema = {
        SubassemblyInstance.pm_functions: {
            PmFunction.trend_meta: {}
        }
    }
    pm = SubassemblyInstance.with_(schema).all()
    # pm = SubassemblyInstance.with_joined(SubassemblyInstance.pm_functions,SubassemblyInstance.pm_functions.trend_meta).all()
    result = [
        o._to_dict() for o in pm
    ]
    return jsonify(result)


@bp.route('/gauges',methods=['GET'])
def gauges():
    pm = Gauge.all()
    result = [
        o._to_dict(exclude=['data']) for o in pm
    ]
    return jsonify(result)


def getModelByName(name):
    for mapper in db.Model.registry.mappers:
        print(f'Comparing: {name}, {mapper.class_.__name__}')
        if name == mapper.class_.__name__:
            return mapper.class_
    return None

def add_with(model,with_list):
    if with_list:
        return model.with_joined(with_list)
    return model

def add_where(model,where):
    if where is None:
        return model
    print("----------------- Where is: ",where)
#    return model.where(where)
#    return model.where(id=3)
    return model

@bp.route('/objects/<obj>',methods=['GET'])
def objects(obj):
    model = getModelByName(obj)
    if model is None:
        return jsonify({"error":"No Model found"})
#    model = add_where(model,request.args.get("where",None))
    model = add_with(model,request.args.get("with",None))
    qresult = model.all()
#    for item in qresult:
#        print("Timestamp: ",item.timestamp)
    if qresult:
        result = [ o._to_dict() for o in qresult]
    else:
        result = {"error":"No object found"}
    return jsonify(result)

@bp.route('/object/<obj>/<_id>',methods=['GET'])
def single_object(obj,_id):
    print("------------- request.args",request.args)
    model = getModelByName(obj)
    if model is None:
        return jsonify({"error":"No Model found"})
    model = add_with(model,request.args.get("with",None))
    qresult = model.find(_id)
    if qresult is None:
        return jsonify({"error":"No objects found"})
    return jsonify(qresult._to_dict(camelKeys=True))

@bp.route('/graph/<_id>/<pid>',methods=['GET'])
def graph_object(_id,pid):
    tm = TrendMetaData.where(**{'pm_func_id':pid})._with('data_points').all()
    rb = SQLAlchemyResponseBuilder(tm).only(['color','display_name','name',{"data_points":["timestamp","value"]}])
#    rb = SQLAlchemyResponseBuilder(tm)
    return JsonResponse(rb,content_type='application/json')

#    result = [ o._to_dict(includeHybrid=True,camelKeys=True,only=['color','display_name','name',{"data_points":["timestamp","value"]}]) for o in tm]
#    result = [ o._to_dict(includeHybrid=True,camelKeys=True) for o in tm]
#    return jsonify(result)
#    return Response(result)


@bp.route('/responsetest',methods=['GET'])
def responsetest():
    print("Response test: ")
    r={"response":{"test":"testing"}}
    resp = jsonpickle.encode(r)
    G = Gauge.first()
    gr = G._to_dict()
    rb = SQLAlchemyResponseBuilder(G)
    return JsonResponse(rb,content_type='application/json')

@bp.route('/anytest',methods=['GET'])
def anytest():

    G=PmFunction.query.options(joinedload(PmFunction.trend_meta_data)).filter(~PmFunction.trend_meta_data.any()).all()
    rb = SQLAlchemyResponseBuilder(G)
    return JsonResponse(rb)


from mdhwebserver.exceptions.pumpsense_error import PumpsenseError
@bp.errorhandler(PumpsenseError)
def handle_error(e):
    rb = MDHErrorResponseBuilder("blue print handling error")
    return JsonResponse(rb)
    
    return "blue print handling error"

@bp.route('/exception',methods=['GET'])
def exception():
    raise PumpsenseError("Error occured")
    return "After raise"
    
from flask import abort


@bp.route('/abort',methods=['GET'])
def do_abort():
    abort(404,"My 404 error message")
    return "After raise"

# from mdhwebserver.modules.pumpsense.processor import getGaugeType

def processExpertData(payload):
    data = payload['data']
    assemblyName = data['subassemblyinstance']
    assembly = Assembly.query.filter(Assembly.name==assemblyName).first()
    values = {
        'machine_id':data['machineId'],
        'machine_name':data['machine'],
        'company_name':data['company']
    }
    # gt = getGaugeType('poweranalyzer')
    # values['gauge_type_id'] = gt.id
    values['assembly_id'] = assembly.id
    # Stubbed values
    values['location_name'] = ""    # Can't be null, but we don't want this either
    keys = { "machine_id": values['machine_id'] }
    G = Gauge.updateOrCreate(keys,**values)
    edg = ExpertDataGroup.firstOrCreate(**{
        'name':data['datatype'],
        'display_name':data['datatype'],
        'assembly_id':assembly.id
    });

    for key,param in data['params'].items():
        print(f"{key} = {param}")
        if key in edg.mapped_expert_data_items:
            print(f"Key in expert data items")
        else:
            print(f"Key not in expert data items")
            edg.mapped_expert_data_items[key] = ExpertDataItem(**{
                'name':key,
                'display_name':key
            })
        edg.mapped_expert_data_items[key].expert_data_points.append(ExpertDataPoint(**{
            '_timestamp':datetime.fromtimestamp(int(payload['timestamp'])/1000000000, tz=timezone.utc),
            'value':param,
            'gauge_id':G.id,
            'machine_id':G.machine_id,
            'raw_timestamp':payload['timestamp']
        }))
    G.save()
    db.session.commit()

    return G
    
@bp.route('/expertDataPost',methods=['POST'])
def expertDataPost():
    data=request.data.decode('UTF-8')
#    print(data)
    payload = json.loads(data)
    for p in payload:
        processExpertData(p)
    return jsonify(payload)


@bp.route('/expertDataPost2',methods=['POST'])
def expertDataPost2():
    data=json.loads(request.data.decode('UTF-8'))
    processExpertData2(data)
    return jsonify(data)


def processExpertData2(payload):
    insert_expert_data(payload, 'poweranalyzer')


from flask import session

@bp.route('/session',methods=['GET'])
def get_session():
	print("Session: ", session)
	return "Done"

@bp.route('/subassembly/<_id>',methods=['GET'])
def subassembly_test(_id):
    print("id is: ",_id)
    s = SubassemblyInstance.query.filter(SubassemblyInstance.id==_id).first()
    rb = SQLAlchemyResponseBuilder(s)
    return JsonResponse(rb,content_type='application/json')
    return "id"


@bp.route('/pmfunction/<subassembly_instance_name>/<pm_function_name>',methods=['GET'])
def pmfunction_test(subassembly_instance_name,pm_function_name):
    _id=1
    print("id is: ",_id)
    print("pm_function_name is ",pm_function_name)
    #s = PmFunction.query.filter(PmFunction.id==_id).all()
    #s = PmFunction.query.filter(PmFunction.subassembly_instance_name==subassembly_instance_name).all()
    s = PmFunction.query.filter(PmFunction.subassembly_instance_name==subassembly_instance_name).filter(PmFunction.name == pm_function_name).all()
    rb = SQLAlchemyResponseBuilder(s)
    return JsonResponse(rb,content_type='application/json')
    return "id"


@bp.route('/with_test',methods=['GET'])
def with_test():
    split_list=["machines","machines.configs","machines.device_data_items"]
    d=Datahub().with_joined(*split_list).all()
    rb = SQLAlchemyResponseBuilder(d)
    return JsonResponse(rb,content_type='application/json')

@bp.route('/with_test2',methods=['GET'])
def with_test2():
    split_list=["machines","machines.configs","machines.device_data_items"]
    schema={
        Datahub.machines:{
            Gauge.configs: {}
        }
    }
    d=Datahub().with_(schema).all()
    rb = SQLAlchemyResponseBuilder(d)
    return JsonResponse(rb,content_type='application/json')

def get_relationship(model,relationship):
    a = getattr(model,relationship)
    return a

from mergedeep import merge

def make_dot_schema(model,dot_list):
    print("dot list: ",dot_list)
    if not dot_list:
        return {}
    relationship=dot_list[0]
    a = getattr(model,relationship)
    c = inspect(a)
    print("c: ",c)
    print("c argument: ", c.property.argument)
    print("type: ", type(c.property.argument))
    if isinstance(c.property.argument,str):
        nm = getModelByName(c.property.argument)
    else:
        nm = c.property.argument
    return {a: make_dot_schema(nm,dot_list[1:])}

def make_schema(model,split_list):
    schema={}
    for item in split_list:
        print("item: ",item)
        dot_list=item.split('.')
        r=make_dot_schema(model,dot_list)
        print("r: ",r)
        # schema = {**schema,**r}
        schema = merge(schema,r)
    return schema


@bp.route('/with_test3',methods=['GET'])
def with_test3():
    split_list=["machines","machines.configs","machines.device_data_items","machines.mro_logs"]
    # split_list=["machines","machines.configs"]

    schema={
        Datahub.machines:{
            Gauge.configs: {}
        }
    }
    print("schema: ", schema)
    s2=make_schema(Datahub,split_list)
    print("s2: ",s2)
    d=Datahub().with_(s2).all()
    rb = SQLAlchemyResponseBuilder(d)
    return JsonResponse(rb,content_type='application/json')

# from mdhwebserver.models.models import UnitTest
from mdhwebserver.models.models import DeviceUnitDataPoint

from sqlalchemy.sql import operators, text
from sqlalchemy.orm import column_property, with_expression

CONVERT=operators.custom_op('@')


# @bp.route('/unit_test/<name>',methods=['GET'])
# def unit_test(name):
#     print("Unit test",name)
#     value = request.args.get('value')
#     print("Value is: ", value)
#     ut = UnitTest.create(**{
#         'name': name,
#         'value': value
#     })
#     db.session.commit()
#     rb = SQLAlchemyResponseBuilder(ut)
#     return JsonResponse(rb,content_type='application/json')

# @bp.route('/show_unit_test',methods=['GET'])
# def show_unit_test():
#     # uts = UnitTest().query.options(load_only(UnitTest.value.op("@",'degF'))).all()
#     uts = UnitTest().query.options(with_expression(UnitTest.converted, UnitTest.value.op("@")("degF"))).all()
#     rb = SQLAlchemyResponseBuilder(uts)
#     return JsonResponse(rb,content_type='application/json')

@bp.route('/show_device_unit_test',methods=['GET'])
def show_device_unit_test():
    pmFunctionName="ValueTestCounter"
    mid="machine-gauge-test1"
    pm = PmFunction.query.filter(PmFunction.name==pmFunctionName)
    pm = pm.options(
        joinedload(PmFunction.trend_meta_data)
        .selectinload(TrendMetaData.unit_data_points
            .and_ (DeviceUnitDataPoint.machine_id == mid)            
        ).with_expression(DeviceUnitDataPoint.converted, DeviceUnitDataPoint.value.op("@")("%"))
                    
    ).first()
    # tm=TrendMetaData.query.options(
    #     selectinload(TrendMetaData.unit_data_points).with_expression(DeviceUnitDataPoint.converted, DeviceUnitDataPoint.value.op("@")("%"))
    # ).all()
    # uts = DeviceUnitDataPoint().query.options(with_expression(DeviceUnitDataPoint.converted, DeviceUnitDataPoint.value.op("@")("%"))).all()
    rb = SQLAlchemyResponseBuilder(pm)
    return JsonResponse(rb,content_type='application/json')


def si_unit_convert(from_units,to_units,unit_list):
    result_list=[]
    if type(unit_list) is list:
        vlist=[f"'{float(x)} {from_units}'@'{to_units}'" for x in unit_list]
        csv=",".join(vlist)
        print("list: ", vlist)
    elif type(unit_list) is dict:
        vlist=[f"'{float(x)} {from_units}'@'{to_units}' as {key}" for key,x in unit_list.items()]
        csv=",".join(vlist)

    print("csv: ", csv)
    result = db.session.execute(text(f"select {csv}"))
    print("result: ", result)
    record=result.fetchone()
    print("Record: ", record)

    if record:
        result_list = [float(x.split(" ")[0]) for x in record]
    if type(unit_list) is dict:
        dict_list=dict(zip(unit_list.keys(),result_list))
        print("dict list: ", dict_list)
        return dict_list
    return result_list

@bp.route('/convert',methods=['post'])
def convert():
    try:
        data =request.data.decode('UTF-8')
        payload = json.loads(data)
        print("DATA: ", payload)
        from_units = payload['from']
        to_units = payload['to']
        # list=[f"'{float(x)} {from_units}'@'{to_units}'" for x in payload['values']]
        # csv=",".join(list)
        # print("list: ", list)
        # print("csv: ", csv)
        # result = db.session.execute(text(f"select {csv}"))
        # print("result: ", result)
        # record=result.fetchone()
        # if record:
        #     result_list = [float(x.split(" ")[0]) for x in record]

        result_list = si_unit_convert(from_units,to_units,payload['values'])
        result = {
            'from': from_units,
            'to': to_units,
            'values' : result_list
        }

        return JsonResponse(MDHResponseBuilder(data=result))
    except Exception as e:
        return JsonResponse(MDHErrorResponseBuilder(str(e)))

from mdhwebserver.models.SiUnitModels import SiUnitUnit

@bp.route('/units',methods=['get'])
def si_units():
    units=SiUnitUnit().all()
    rb = SQLAlchemyResponseBuilder(units)
    return JsonResponse(rb,content_type='application/json')

from sqlalchemy import func

@bp.route('/dimensions',methods=['get'])
def si_dimensions():
    # r= db.session.query(func.upper('degC'))
    r=SiUnitUnit().where(unit=func.dimension('degF')).all()
    rb = SQLAlchemyResponseBuilder(r)
    return JsonResponse(rb,content_type='application/json')

from mdhwebserver.models.models import HeatmapItem
# from mdhwebserver.models.models import HeatMapItemAssociation
from mdhwebserver.models.models import heatmap_child_parent_items
from sqlalchemy import Table, select
from mdhwebserver.modules.pumpsense.processor import get_max_color
import time
    
def set_color(parent, ts):
    print("SET Color for: ", parent.name)
    if parent.last_color_update == ts:  # Already updated.
        return parent.color
    color_list=[]
    for child in parent.rchildren:
        if child.rchildren:
            print(f"has child with children: {child.name}")
            if child.last_color_update != ts:
                set_color(child,ts)
        color_list.append(child.color)
    print(f"Color list ({parent.name}): ",color_list)
    color = get_max_color(color_list)
    if color == parent.color:
        parent.color_count += 1
    else:
        parent.color_count = 0
    parent.color = color
    parent.last_color_update = ts
    print("Color is: ", parent.color)
    return parent.color


def heatmap_propagate():
    subquery = db.session.query(heatmap_child_parent_items.c.parent_id).subquery() #.distinct()
    # subquery = select(HeatMapItemAssociation.parent_id).subquery() #.distinct()
    parent_list = HeatmapItem.query.filter(HeatmapItem.id.in_(subquery))
    ts=int(time.time())
    for parent in parent_list:
        set_color(parent,ts)
    db.session.commit()
    return parent_list

@bp.route('/color_propagate',methods=['get'])
def color_propagate():
    parent_list = heatmap_propagate()
    rb = SQLAlchemyResponseBuilder(parent_list)
    return JsonResponse(rb,content_type='application/json')
    # return "Done"

@bp.route('/append_test',methods=['get'])
def append_test():
    parent=request.args.get("parent")
    # child=request.args.get("child")
    children = request.args.getlist("children")
    children = [int(float(c)) for c in children]
    print("----- Children: ", children)
    g=Gauge().where(id=parent).first()
    # child_ids = g.child_ids
    # print("------- Child ids: ", child_ids)
    # print("-------- Child is: ", child)
    # c=Gauge().where(id=child).first()
    # c=Gauge(id=child).first()

    print("Before append")
    # g.children.append(c)
    g.children=[]
    # g.children.extend([c])
    # g.child_ids.extend([child])
    # children = [10,11,100]
    # cc=Gauge().filter(Gauge.id.in_(children)).all()
    # for x in cc:
    #     print("--- Name: ", x.machine_name)
    cc=Gauge().filter(Gauge.id.in_(children)).all()
    # cc=Gauge().filter(Gauge.id.in_(children)).with_entities(Gauge.id)

    # cids = [r[0] for r in cc]
    # print("-- cids: ", cids)
    # g.child_ids.extend(cids)
    g.children.extend(cc)
    print("After append")
    rb = SQLAlchemyResponseBuilder(g)
    print("Before commit")
    db.session.commit()
    print("After commit")

    return JsonResponse(rb,content_type='application/json')

@bp.route('/view_test',methods=['get'])
def view_test():
    parent=request.args.get("gauge")
    device_item_list = request.args.getlist("items")
    device_item_list = [int(float(c)) for c in device_item_list]
    g=Gauge().where(id=parent).first()
    cc=DeviceDataItem().filter(DeviceDataItem.id.in_(device_item_list)).all()
    g.device_data_items = []
    g.device_data_items.extend(cc)
    rb = SQLAlchemyResponseBuilder(g)
    db.session.commit()
    return JsonResponse(rb,content_type='application/json')

@bp.route('/json_test',methods=['post'])
def json_test():
    j = request.json
    print(j)
    print(request.get_json())
    print("Type: ", type(request.data.decode('UTF-8')))
    return "Done"

from datetime import datetime, timedelta
from mdhwebserver.modules.pumpsense.processor import add_kpis
from mdhwebserver.etl import hourly_etls

@bp.route('/etl_test',methods=['get'])
def etl_test():
    print("HOURLY ETL: ")
    hourly_etls()
    # end_time = datetime.now().replace(microsecond=0, second=0, minute=0)
    # start_time = end_time - timedelta(hours=1)
    # PmFunc = PmFunction.where(name="Active Energy Usage Hourly").first()

    # trend_meta_id=781
    # # q=f"select gauge_id,sum(value) as etl_value from device_unit_data_points \
    # #     where trend_meta_data_id={trend_meta_id} and timestamp between '{start_time}' and '{end_time}' group by gauge_id "
    
    # pm_function_id=537 
    # q=f"select dp.gauge_id as gauge_id,pmf.id as pm_function_id, sum(value) as etl_value from pm_functions pmf \
    #     join trend_meta_data tmd on pmf.id = tmd.pm_func_id \
    #     join device_unit_data_points dp on dp.trend_meta_data_id = tmd.id \
    #     where pmf.id = {pm_function_id} and dp.timestamp between '{start_time}' and '{end_time}' group by gauge_id, pmf.id "

    # print("Query: ", q)

    # # return 'done'
    # result = db.session.execute(text(q))
    # record = result.fetchone()
    # while (record):
    #     print("--------------- RECORD: ", record)
    #     G = Gauge.where(id=record[0]).first()
    #     # PmFunc = PmFunction.where(id=int(record[1])).first()
    #     kpis = {'hourly': record[2]}
    #     device_data = G.device_data_item(PmFunc.id, PmFunc.subassembly_instance_id)
    #     # GaugeGroup = get_gauge_group_function(PmFunc)
    #     GaugeGroup = None
    #     if GaugeGroup is None:
    #         ggid = None
    #         ggname = None
    #     else:
    #         ggid = GaugeGroup.id
    #         ggname = GaugeGroup.name


    #     if device_data is None:
    #         device_data = DeviceDataItem(**{
    #             'data': {},
    #             'gauge_id': G.id,
    #             # 'pm_func_id': PmFunc.id,
    #             'pm_function': PmFunc,
    #             'pm_function_name': PmFunc.name,
    #             'subassembly_instance_id': PmFunc.subassembly_instance_id,
    #             'subassembly_instance_name': PmFunc.subassembly_instance_name,
    #             'gauge_group_id': ggid,
    #             'gauge_group_name': ggname,
    #             'colors': ['green'],
    #             'values': None,
    #             'reason': None,
    #             'activate_hourglass': False,
    #             'feedback': False,
    #             'user_input': False,
    #             'hide_from_group': False,
    #             'message': None,
    #             'yellow_threshold': None,
    #             'red_threshold': None,
    #             'kpis':kpis,
    #             'color': None
    #         })
    #         G.device_data_items.append(device_data)
    #     else:
    #         device_data.fill(**{
    #             'colors': ['green'],
    #             'values': None,
    #             'reason': None,
    #             'activate_hourglass': False,
    #             'feedback': None,
    #             'user_input': False,
    #             'hide_from_group': False,
    #             'message': None,
    #             'yellow_threshold': None,
    #             'red_threshold': None,
    #             'kpis':kpis,
    #             # 'color': max_gauge_color
    #         })
    #     add_kpis(G,PmFunc,kpis,end_time,PmFunc.units)

    #     record = result.fetchone()

    # db.session.commit()
    return "DONE"



@bp.route('/virtual_pm_function',methods=['post'])
def post_virtual_pm_function():
    j = request.json
    print("J",j)
    trend_meta_datas = j.get("trend_meta_datas")
    print("j is: ",j)
    sid = j.get('subassembly_instance_id')
    show_trend = j.get('show_trend')
    subassembly = SubassemblyInstance.where(id=sid).options(
        selectinload(SubassemblyInstance.assemblies).options(
            selectinload(Assembly.gauges)
        )
    ).first()

    vp = PmFunction.create(**{
        'name': j.get('name'),
        'display_name': j.get('name'),
        'subassembly_instance_id': j.get('subassembly_instance_id'),
        'gauge_style': "virtual",
        'gauge_type': j.get('gauge_type','value'),
        "trend_existence": j.get('trend_existence',True)
    })
    tms=TrendMetaData().filter(TrendMetaData.id.in_(trend_meta_datas)).all()
    if show_trend is None:
        tmd_show_trend = tms[0]

    vp.show_trend = tmd_show_trend.name
    vp.units = tmd_show_trend.units
    vp.unit_system = 'fps' if tmd_show_trend.units == tmd_show_trend.fps_units else 'mks'
    vp.trend_meta_data = tms

    for assemblies in subassembly.assemblies:
        for g in assemblies.gauges:
            if g.machine_view_type == "machine":
                update_virtual_gauges(g,set([vp]))

    rb = SQLAlchemyResponseBuilder(vp).camel(True)
    db.session.commit()
    return JsonResponse(rb,content_type='application/json')
    return "Done"

import sqlalchemy as sa


from mdhwebserver.utils.virtual_gauges.virtual_gauges import get_virtual_pm_functions, update_virtual_gauges,update_virtual_gauges_from_trends

def update_virtual_gauge(gauge_id):
    G = Gauge.where(id=gauge_id).options(
        selectinload(Gauge.assembly).options(
            selectinload(Assembly.subassemblies).options(
                selectinload(SubassemblyInstance.pm_functions).options(
                    selectinload(PmFunction.trend_meta_data_direct)
                )
            )
        )
    ).first()
    print(G.toJson())
    tmids = set()
    tms = set()
    for subassembly in G.assembly.subassemblies:
        for pmf in subassembly.pm_functions:
            tms |= set(pmf.trend_meta_data_direct)
            for tm in pmf.trend_meta_data_direct:
                tmids.add(tm.id)
    virtual_functions = update_virtual_gauges_from_trends(G,tms)
    return virtual_functions

@bp.route('virtual_kpis/<gauge_id>/<pmid>', methods=['GET'])
def virtual_kpis(gauge_id,pmid):
    vf = update_virtual_gauge(gauge_id)
    # ddi=DeviceDataItem.where(gauge_id=gauge_id, pm_func_id=pmid).first()
    # if ddi==None:
    #     print("No DDI")
    #     ddi=DeviceDataItem.create(**{
    #         'gauge_id': gauge_id,
    #         'pm_func_id': pmid
    #     })
    #     ddi.pm_function_name = ddi.pm_function.name
    #     ddi.subassembly_instance_id = ddi.pm_function.subassembly_instance_id
    #     ddi.subassembly_instance_name = ddi.pm_function.subassembly_instance_name
    # pm = PmFunction.query.filter(PmFunction.id==pmid)
    # ddi.kpis = get_virtual_gauge_kpis(pm.first(),gauge_id)
    # # pmr=pm = pm.options(
    # #     # selectinload(PmFunction.device_data_items.and_(DeviceDataItem.gauge_id == gauge_id)),
    # #     joinedload(PmFunction.trend_meta_data)\
    # #             .joinedload(TrendMetaData.unit_data_points\
    # #                 .and_(DeviceUnitDataPoint.gauge_id == gid)
    # #             )
                
    # #     ) \
    # #     .first()
    # # pmr=TrendMetaData.query.filter(TrendMetaData.pm_func_id==pmid).options(
    # #             joinedload(TrendMetaData.unit_data_points\
    # #                 .and_(DeviceUnitDataPoint.gauge_id == gauge_id)
    # #             )
    # #     ) \
    # #     .first()

    db.session.commit()
    # # rb = SQLAlchemyResponseBuilder(ddi)
    rb = SQLAlchemyResponseBuilder(vf).camel(True).only([
        "name","display_name","color",{"pm_functions":["name","id","gauge_style"]}
    ])

    return JsonResponse(rb,content_type='application/json')
  

@bp.route('virtual_unit_data', methods=['POST'])
def virtual_unit_data():

    print("------------------ UNIT DATA __________________")
    startTime = request.json.get('startTime')
    endTime = request.json.get('endTime')
    pmFunctionName = request.json.get('pmFunctionName')
    subassembly_name= request.json.get('subAssemblyInstanceName')
    camelKeys = request.json.get('camel',True)
    units = request.json.get('units')
    unit_system = request.json.get('unit_system')
    gauge_id = request.json.get('gauge_id')

    sTime = datetime.fromtimestamp(startTime, tz=timezone.utc)
    eTime = datetime.fromtimestamp(endTime, tz=timezone.utc)
    subassembly = SubassemblyInstance.query.filter(SubassemblyInstance.name == subassembly_name) \
        .options(joinedload(SubassemblyInstance.pm_functions.and_(PmFunction.name==pmFunctionName))) \
        .first()
    print("subassembly: ", subassembly.toJson())
    # if units == None:
    #     pm1 = PmFunction.query.filter(PmFunction.name==pmFunctionName).filter(PmFunction.subassembly_instance_id == subassembly.id).first()
    #     units=pm1.units
    #     if unit_system is not None:
    #         if pm1.unit_system and pm1.unit_system != unit_system:
    #             print("------------Getting unit translation")
    #             unit_translate=UnitTranslate.where(unit_system=pm1.unit_system, name=pm1.units).first()
    #             if unit_translate:
    #                 print("----------Have unit translation.")
    #                 units=unit_translate.translate_name
    print("---------------- units = ",units)
    pm = PmFunction.query.filter(PmFunction.name==pmFunctionName).filter(PmFunction.subassembly_instance_id == subassembly.id)
    # pm = pm.options(
    #     joinedload(PmFunction.device_data_items.and_(DeviceDataItem.gauge_id == gauge_id)),
    #     joinedload(PmFunction.trend_meta_data)\
    #             .selectinload(TrendMetaData.unit_data_points\
    #                 .and_(DeviceUnitDataPoint.gauge_id == gauge_id)\
    #                 .and_(DeviceUnitDataPoint._timestamp.between(sTime,eTime))\
    #             ).with_expression(DeviceUnitDataPoint.converted, DeviceUnitDataPoint.value.op("@@")(units)) \
    #     ) \
    #     .first()
    pm = pm.options(
        # selectinload(PmFunction.device_data_items.and_(DeviceDataItem.gauge_id == gauge_id)),
        joinedload(PmFunction.trend_meta_data)\
                .joinedload(TrendMetaData.unit_data_points\
                    .and_(DeviceUnitDataPoint.gauge_id == gauge_id)\
                    .and_(DeviceUnitDataPoint._timestamp.between(sTime,eTime))\
                ).options(
                with_expression(DeviceUnitDataPoint.mks, sa.func.unit_at_double(DeviceUnitDataPoint.value,sa.sql.text("trend_meta_data_1.mks_units"))), \
                with_expression(DeviceUnitDataPoint.fps, sa.func.unit_at_double(DeviceUnitDataPoint.value,sa.sql.text("trend_meta_data_1.fps_units"))), \
                # joinedload(DeviceUnitDataPoint.trend_meta_data) \
            ) \
                
        ) \
        .first()


    rb = SQLAlchemyResponseBuilder(pm)  \
            .only(['name',"id",{"device_data_items":["yellow_threshold","red_threshold"]},{"trend_meta_data":["color","display_name","name","id","pm_func_id","units","fps_units","mks_units",{"unit_data_points":["value","timestamp","mks","fps"]}]}]) \
            .camel(camelKeys)
    return JsonResponse(rb,content_type='application/json')
