from ..module import bp
from pathlib import Path
import os
import json
import time
import gzip
import shutil
import shlex
import subprocess
from datetime import datetime
import logging
from mdhwebserver.appservices import database as db
from mdhwebserver import appconfigs

from http import HTTPStatus
from flask import (
    request,
    redirect,
    abort,
    send_file
)
from flask_jwt_extended import current_user
from mdhwebserver.wrappers.is_authorized import is_authorized

from mdhwebserver.builders.responses import JsonResponse
from mdhwebserver.builders.response_builder import SQLAlchemyResponseBuilder, MDHResponseBuilder, MDHErrorResponseBuilder
from mdhwebserver.builders import ValidationErrorResponseBuilder
from mdhwebserver.exceptions import ValidationException
from mdhwebserver.utils.listutils import dict_extract
from mdhwebserver import application

from sqlalchemy.exc import IntegrityError
from sqlalchemy.sql import text

logger = logging.getLogger('timescale')

hypertable_sql = """SELECT j.hypertable_name,
            config
        FROM timescaledb_information.jobs j
        WHERE j.proc_name = 'policy_retention';"""
    

@bp.route('/policies', methods=['GET'])
@is_authorized()
def get_timescale_retention_policies():
    policies = db.session.execute(text(hypertable_sql)).fetchall()
    # rb = SQLAlchemyResponseBuilder(policies)
    return JsonResponse(policies,content_type='application/json')


@bp.route('/policy', methods=['PUT'])
@is_authorized()
def update_timescale_retention_policy():
    name = request.json.get('name')
    retention_period = request.json.get('retention_period')

    if not name or not retention_period:
        return JsonResponse(MDHErrorResponseBuilder(message="Policy name and retention period are required"))

    can_delete_policy = False
    policies = db.session.execute(text(hypertable_sql)).fetchall()
    for policy in policies:
        if policy.hypertable_name == name:
            can_delete_policy = True
            break

    if can_delete_policy:
        sql = "SELECT remove_retention_policy(:name);"
        db.session.execute(text(sql), {'name': name})
        db.session.commit()
        # add 5 second sleep to allow the policy to be deleted before adding a new one
        time.sleep(5)
    
    can_add_policy = True
    policies = db.session.execute(text(hypertable_sql)).fetchall()
    for policy in policies:
        if policy.hypertable_name == name:
            can_add_policy = False
            return JsonResponse(MDHErrorResponseBuilder(message="Policy with the same name already exists"))
            break
    
    if can_add_policy:
        sql = "SELECT add_retention_policy(:name, INTERVAL :period);"
        db.session.execute(text(sql), {'name': name, 'period': retention_period})
        db.session.commit()
    return JsonResponse(MDHResponseBuilder(statusCode=HTTPStatus.OK.value))