from ..module import bp
from pathlib import Path
import os
import json
import time
import gzip
import shutil
import shlex
import subprocess

import logging
from mdhwebserver.appservices import database as db
from mdhwebserver import appconfigs

from http import HTTPStatus
from flask import (
    request,
    redirect,
    abort,
    send_file
)
from flask_jwt_extended import current_user
from mdhwebserver.wrappers.is_authorized import is_authorized

from mdhwebserver.builders.responses import JsonResponse
from mdhwebserver.builders.response_builder import SQLAlchemyResponseBuilder, MDHResponseBuilder, MDHErrorResponseBuilder, DictResponseBuilder
from mdhwebserver.builders import ValidationErrorResponseBuilder
from mdhwebserver.exceptions import ValidationException
from mdhwebserver.models.SiUnitModels import SiUnitUnit
from sqlalchemy.sql import text
from sqlalchemy.orm import column_property, with_expression
from mdhwebserver.models.models import DeviceDataItem, PmFunction
import sqlalchemy as sa

def si_unit_convert(from_units,to_units,unit_list):
    result_list=[]
    if type(unit_list) is list:
        vlist=[f"'{float(x)} {from_units}'@'{to_units}'" for x in unit_list]
        csv=",".join(vlist)
    elif type(unit_list) is dict:
        vlist=[]
        for key,x in unit_list.items():
            if x is not None:
                vlist.append(f"'{float(x)} {from_units}'@'{to_units}' as \"{key}\"")
        #vlist=[f"'{float(x)} {from_units}'@'{to_units}' as \"{key}\"" for key,x in unit_list.items()]
        csv=",".join(vlist)
    else:
        csv=None
    if csv:
        result = db.session.execute(text(f"select {csv}"))
        record=result.fetchone()
    else:
        record = None
    if record:
        result_list = [float(x.split(" ")[0]) for x in record]
    if type(unit_list) is dict:
        idx=0;
        dict_list={}
        for k,v in unit_list.items():
            if v is not None:
                dict_list[k] = result_list[idx]
                idx += 1 
        return dict_list
    return result_list

@bp.route('/convert',methods=['post'])
def convert():
    try:
        data =request.data.decode('UTF-8')
        payload = json.loads(data)
        from_units = payload['from']
        to_units = payload['to']
        result_list = si_unit_convert(from_units,to_units,payload['values'])
        result = {
            'from': from_units,
            'to': to_units,
            'values' : result_list
        }

        return JsonResponse(MDHResponseBuilder(data=result))
    except Exception as e:
        return JsonResponse(MDHErrorResponseBuilder(str(e)))

from mdhwebserver.models.SiUnitModels import SiUnitUnit

@bp.route('/units',methods=['get'])
def si_units():
    units=SiUnitUnit().all()
    rb = SQLAlchemyResponseBuilder(units)
    return JsonResponse(rb,content_type='application/json')

from sqlalchemy import func

@bp.route('/dimensions',methods=['get'])
def si_dimensions():
    # r= db.session.query(func.upper('degC'))
    r=SiUnitUnit().where(unit=func.dimension('degF')).all()
    rb = SQLAlchemyResponseBuilder(r)
    return JsonResponse(rb,content_type='application/json')

def string_convert(s,from_units,to_units):
    split = s.split(' ')
    vlist=[]
    # print(f"String convert from {from_units} to {to_units}")
    for element in split:
        try:
            tmp=float(element)  # Will thow an exception if it can't be converted to a float.
            # print("element is numeric: ", element)
            qelement = f"round(unit_at_double('{float(element)} {from_units}','{to_units}')::numeric,1)"
            vlist.append(qelement)
        except:
            vlist.append(f"'{element}'")

    csv=",".join(vlist)
    result = db.session.execute(text(f"select {csv}"))
    record=result.fetchone()
    if record:
        rlist = [str(x) for x in record]
        ns = " ".join(rlist)
        s = ns
    return s

# This is just a quick fix for stacked bar unit conversion.  Really we need to just make the unit convreted smarter to figure out
# What the data object looks like.

def kpi_unit_convvert(gauge_type, from_units,to_units,unit_list):
    if gauge_type != 'stackedbar':
        return si_unit_convert(from_units,to_units,unit_list)
    if "kpis" not in unit_list:
        return si_unit_convert(from_units,to_units,unit_list)
    kpis={
        'x_unit': unit_list.get('x_unit'),
        'y_unit': unit_list.get('y_unit'),
        'kpis': {}
    }
    for key,kpi_list in unit_list['kpis'].items():
        kpis['kpis'][key] = si_unit_convert(from_units,to_units,kpi_list)
        
    return kpis

@bp.route('/kpiconvert/<id>/<unit_system>',methods=['get'])
def kpiconvert(id,unit_system):
    # r= db.session.query(func.upper('degC'))
    data_item = DeviceDataItem.where(id=id) \
        .options(sa.orm.joinedload(DeviceDataItem.pm_function).options(
            sa.orm.selectinload(PmFunction.unit_translation), \
            sa.orm.selectinload(PmFunction.x_unit_translation), \
            sa.orm.selectinload(PmFunction.y_unit_translation)
            ) \
        )\
        .first()
    x_unit = data_item.pm_function.x_unit
    y_unit = data_item.pm_function.y_unit
    units = data_item.pm_function.units
    kpis=data_item.kpis
    # if data_item.pm_function.unit_translation:
    #     print("Units type doesn't match need to convert.")
    #     # new_kpis = si_unit_convert(data_item.pm_function.unit_translation.name, data_item.pm_function.unit_translation.name.translate_name,kpis)
    #     # kpis = new_kpis
    if data_item.pm_function.x_unit_translation:
        if data_item.pm_function.x_unit_translation.unit_system != unit_system:
            new_kpis={}

            for key,value in kpis.items():
                v = key
                ns = string_convert(v,data_item.pm_function.x_unit_translation.name,data_item.pm_function.x_unit_translation.translate_name)
                new_kpis[ns] = value
            kpis = new_kpis
            x_unit = data_item.pm_function.x_unit_translation.translate_name
    if data_item.pm_function.y_unit_translation:
        if data_item.pm_function.y_unit_translation.unit_system != unit_system:
            new_kpis = kpi_unit_convvert(data_item.pm_function.gauge_type,data_item.pm_function.y_unit_translation.name, data_item.pm_function.y_unit_translation.translate_name,kpis)
            kpis = new_kpis
            y_unit = data_item.pm_function.y_unit_translation.translate_name
    # elif data_item.pm_function.unit_translation:  # Don't want to do both y_unit convert and unit convert, y has precidence.
    #     if data_item.pm_function.unit_translation.unit_system != unit_system:
    #         print("Units type doesn't match need to convert.")
    #         new_kpis = si_unit_convert(data_item.pm_function.unit_translation.name, data_item.pm_function.unit_translation.translate_name,kpis)
    #         kpis = new_kpis
    #         units = data_item.pm_function.unit_translation.translate_name
    d = {
        'x_unit': x_unit,
        'y_unit': y_unit,
        'units': units,
        'kpis': kpis,
        'colors': data_item.colors
    }
    return JsonResponse(DictResponseBuilder(d),content_type='application/json')
