
print("Importing perodic_tasks")
from celery.schedules import crontab
from .appservices import database as db
from mdhwebserver.application import app
from mdhwebserver.etl import hourly_etls, daily_etls, weekly_etls, monthly_etls, yearly_etls
from .appservices import celery

# @celery.on_after_configure.connect
@celery.on_after_finalize.connect

def setup_periodic_tasks(sender, **kwargs):
    print("XXXXXXXXXXXXXXXXXXXXXXXXxx Setting up perodic tasks.")
    # Calls test('hello') every 10 seconds.
    # sender.add_periodic_task(10.0, test.s('hello'), name='add every 10')

    # Calls test('hello') every 30 seconds.
    # It uses the same signature of previous task, an explicit name is
    # defined to avoid this task replacing the previous one defined.
    # sender.add_periodic_task(30.0, test.s('hello'), name='add every 30')

    # Calls test('world') every 30 seconds
    sender.add_periodic_task(60.0, gray_out_heatmap_items.s('world'), expires=10, name='Gray Out Heatmap')
    sender.add_periodic_task(60.0, hourly_etl_task.s('world2'), expires=10, name='Hourly ETL')
    sender.add_periodic_task(60.0, daily_etl_task.s('world2'), expires=10, name='Daily ETL')
    sender.add_periodic_task(60.0, weekly_etl_task.s('world2'), expires=10, name='Weekly ETL')
    sender.add_periodic_task(60.0, monthly_etl_task.s('world2'), expires=10, name='Monthly ETL')
    sender.add_periodic_task(60.0, yearly_etl_task.s('world2'), expires=10, name='Yearly ETL')



    sender.add_periodic_task(60.0, hourly_etl_task.s('world2'), expires=10, name='Hourly ETL')

    sender.add_periodic_task(crontab(minute="*/6"), 
        daily_etl_task.s('world2'), expires=10, name='Daily ETL')
    
    sender.add_periodic_task(crontab(minute="5",hour="*/1"), 
        weekly_etl_task.s('world2'), expires=10, name='Weekly ETL')
    
    sender.add_periodic_task(crontab(minute="15",hour="*/1"), 
        monthly_etl_task.s('world2'), expires=10, name='Monthly ETL')
    
    # sender.add_periodic_task(crontab(minute="15",hour="20",day_of_week=6), 
    sender.add_periodic_task(crontab(minute="25",day_of_week="*/1"), 
        yearly_etl_task.s('world2'), expires=10, name='Yearly ETL')

    sender.add_periodic_task(crontab(minute="45"), 
        clean_kombu_messages.s(), expires=10, name='Kombu Clean up')

    # # Executes every Monday morning at 7:30 a.m.
    # sender.add_periodic_task(
    #     crontab(hour=7, minute=30, day_of_week=1),
    #     test.s('Happy Mondays!'),
    # )

@celery.task
def test(arg):
    print(arg)

from mdhwebserver.models.models import HeatmapItem
from sqlalchemy import update, text

@celery.task
def gray_out_heatmap_items(arg):
    with app.app_context():
        # hmi = HeatmapItem.filter("HeatmapItem.updated_on < now() - INTERVAL '11 mimutes'").all()
        stmt = (update(HeatmapItem)
            .where(text("updated_on < now() at time zone 'utc' - INTERVAL '11 minutes'"))
            .values(color='grey')
        )
        # print(hmi)
        db.session.execute(stmt)
        db.session.commit()


@celery.task
def hourly_etl_task(arg):
    with app.app_context():
        hourly_etls()

@celery.task
def daily_etl_task(arg):
    with app.app_context():
        daily_etls()

@celery.task
def weekly_etl_task(arg):
    with app.app_context():
        weekly_etls()

@celery.task
def monthly_etl_task(arg):
    with app.app_context():
        monthly_etls()

@celery.task
def yearly_etl_task(arg):
    with app.app_context():
        yearly_etls()

@celery.task
def clean_kombu_messages():
    RETENTION_DAYS=2
    with app.app_context():
        query_string=f"delete from kombu_message where visible='f' and timestamp < NOW() - INTERVAL '{RETENTION_DAYS} DAYS'"
        db.session.execute(text(query_string))
        db.session.commit()