
from .models.models import (
        PmFunction,
        SubassemblyInstance,
        TrendMetaData,
        GaugeType,
        Assembly,
        GaugeGroup,
        GaugeRange,
        UnitTranslate
)

from .appservices import database as db
from mdhwebserver.utils.listutils import dict_extract
import simplejson as json

from mdhwebserver.etl import create_sum_etl_gauges, get_etl_type, get_etl_append_string, create_avg_etl_gauges

gaugeTypeFile="./seeding/gauge_types.json"

def add_units(u_system):
    for us in u_system:
        if len(us) == 4:
            new_unit = UnitTranslate.firstOrCreate(**us)
            db.session.add(new_unit)

        else:
            new_unit = UnitTranslate.firstOrCreate(**{
                'unit_system': 'fps',
                'name': us.get('fps_name'),
                'translate_unit_system': 'mks',
                'translate_name': us.get('mks_name')
            })
            db.session.add(new_unit)
            new_unit = UnitTranslate.firstOrCreate(**{
                'unit_system': 'mks',
                'name': us.get('mks_name'),
                'translate_unit_system': 'fps',
                'translate_name': us.get('fps_name')
            })
            db.session.add(new_unit)
    
def make_etl_gauges(subassembly, source_pm_func, etl_gauge_list):

    print(f"------------ MAKE ETL GAUGES ({source_pm_func.name}) ------------------")
    etl_name_sum_set=set()
    etl_name_avg_set=set()
    for etl_gauge in etl_gauge_list:
        etl_type = get_etl_type(etl_gauge.get('gauge_style'))
        print("ETL TYPE: ", etl_type)
        append_string = get_etl_append_string(etl_gauge.get('gauge_style'))
        print("ETL APPPEND STRING: ",append_string)
        # name = etl_gauge['name'].rsplit(' ',1)[0]
        name = etl_gauge['name'].replace(append_string,"").rstrip()
        print(f"NAME: [{name}]")
        if etl_type == 'sum':
            etl_name_sum_set.add(name)
        elif etl_type == 'avg':
            etl_name_avg_set.add(name)
    print("ETL SUM SET: ",etl_name_sum_set)
    print("ETL AVG SET: ",etl_name_avg_set)

    for etl_name in etl_name_sum_set:
        create_sum_etl_gauges(source_pm_func,etl_name)
    for etl_name in etl_name_avg_set:
        create_avg_etl_gauges(source_pm_func,etl_name)

def pm_function_add_mro_issues(pm_fun,mro_issues):
    if not mro_issues:
        return
    for issue in mro_issues:
        new_mro_issue = MroIssue.firstOrCreate(**{
            'pm_function': pm_fun,
            'color': issue.get('color'),
            'description': issue.get('description'),
            'preventive_action' : issue.get('preventive_action')
        })


def make_pm_function(subassembly,pm_function):
    print(f"MAKE PM FUNCTION: {pm_function['name']}")
    pm_function.pop('id')
    pm_function.pop('created_on')
    pm_function.pop('subassembly_instance_id')
    pm_function.pop('updated_on')
    pm_function.pop('gauge_group_id')
    pm_function.pop('etl_pm_function_id',None)
    etl_gauge_list = pm_function.pop('etl_gauges',None)
    print("ETL GAUGES: ", etl_gauge_list)
    if 'unit_translation_id' in pm_function:
        pm_function.pop('unit_translation_id')
    pm_function.pop('subassembly_instance_name')
    if 'config' in pm_function:
        pm_function.pop('config')
    trend_meta_data = pm_function.pop('trend_meta_data') if 'trend_meta_data' in pm_function else None
    if 'gauge_group' in pm_function:  # This needs to get restructured..
        gauge = pm_function.pop('gauge_group')
        gg = seed_gauge_group_func(gauge)
        pm_function['gauge_group_id'] = gg.id
    mro_issues = pm_function.pop('mro_issues',None)
    print("Creating pm function.")
    gauge_ranges = pm_function.pop('gauge_ranges') if 'gauge_ranges' in pm_function else None
    pmf = PmFunction.create(**pm_function, subassembly=subassembly)
    print("Created pm function")
    tm_map = {}
    if trend_meta_data:
        print(" -------------------- Adding trend meta data to pmf")
        for trend_meta in trend_meta_data:
            tm = TrendMetaData.create(**{
                'name': trend_meta.get('name'),
                'display_name': trend_meta.get('display_name'),
                'color': trend_meta.get('color'),
                'units': trend_meta.get('units'),
                'mks_units': trend_meta.get('mks_units'),
                'fps_units': trend_meta.get('fps_units'),

            })
            pmf.trend_meta_data.append(tm)
            pmf.trend_meta_data_direct.append(tm)
            tm_map[tm.name] = tm
    if gauge_ranges:
        for gauge_range in gauge_ranges:
            gauge_range.pop("id")
            tm_item = tm_map.get(gauge_range['trend_meta_data_name'])
            if tm_item:
                gr = GaugeRange(**gauge_range, trend_meta_data = tm_item)
                pmf.gauge_ranges.append(gr)
    subassembly.pm_functions_dict[pmf.name] = pmf
    if etl_gauge_list:
        make_etl_gauges(subassembly,pmf,etl_gauge_list)
    pm_function_add_mro_issues(pmf,mro_issues)
    
    return pmf

import os
from mdhwebserver.mro.utilities import get_pm_function
from mdhwebserver.models.models import MroIssue

def seed_mro_issues():
    module_dir = os.path.join(os.path.dirname(__file__),"seedfiles/mro_issues")
    print("module_dir: ",module_dir)

    data = {}
    for module in os.listdir(module_dir):
        modpath=module_dir + "/" + module
        if os.path.isdir(modpath):
            continue
        print("Loading mro seed file: ",modpath)
        with open(modpath) as f:
            mro_issues = json.load(f)
        print("MRO SEED: ",json.dumps(mro_issues))

        issue_list = {}
        for issue in mro_issues:
            pm_name = issue['pmFunction']['name']
            if pm_name not in issue_list:
                issue_list[pm_name] = []
            issue_list[pm_name].append(issue)
            pm_functions = PmFunction.where(name=pm_name).all()
            if pm_functions is None:
                print(f"Pm Function not found: {issue['pmFunction']['subAssembly']['name']}:{issue['pmFunction']['name']}")
                continue
            for pm_fun in pm_functions:
                new_mro_issue = MroIssue.firstOrCreate(**{
                    'pm_function_id': pm_fun.id,
                    'color': issue.get('color'),
                    'description': issue.get('description'),
                    'preventive_action' : issue.get('preventive_action')
                })
        db.session.commit()
        # if extension == 'py':
        #     seed_module(modname)
        # elif extension == 'json':
        #     seed_json(modpath)
        # else:
        #     continue
        print(json.dumps(issue_list,indent=4))

def seed_from_json(data):
    assembly_data = data['data']
    # print("Assembly: ", assembly_data)
    ass = dict_extract(assembly_data,['name','display_name','image'])
    # print("ass: ", ass)
    assembly = Assembly.firstOrCreate2({'name': ass['name']},ass)
    for subassembly_data in assembly_data.get('subassemblies',[]):
        # print("Subassembly: ", subassembly_data)
        sub = dict_extract(subassembly_data,['name','display_name'])
        subassembly = SubassemblyInstance.where(name=sub.get('name')).first()
        if subassembly is None:
            subassembly = SubassemblyInstance.create(**sub)
            print("Appending assembly")
        if subassembly.name not in assembly.subassemblies_dict:
            assembly.subassemblies_dict[subassembly.name] = subassembly
        print("-------- Processing pm functions")
        for pm_function in subassembly_data.get('pm_functions',[]):
            print(f"pm_function: ({pm_function['name']})", pm_function)
            if pm_function['name'] in subassembly.pm_functions_dict:
                print("Gauge already created: ", pm_function['name'])
                if 'etl_gauges' in pm_function and pm_function['etl_gauges']:
                    if not subassembly.pm_functions_dict[pm_function['name']].etl_gauges:
                        make_etl_gauges(subassembly,subassembly.pm_functions_dict[pm_function['name']],pm_function['etl_gauges'])
                continue
            # This is sort of kludgie but for testing so we don't create the etl gauge.
            if 'etl_pm_function_id' in pm_function and pm_function['etl_pm_function_id'] is not None:
                print("Kludge continue...")
                continue
            # Lets not create virtual or abstract gauges during seeding at this time.  Need to rethink how to handle this.
            if pm_function['gauge_style'] == 'virtual':
                continue
            if pm_function['gauge_style'] == 'abstract':
                continue

            pmf = make_pm_function(subassembly,pm_function)
            # pm_function.pop('id')
            # pm_function.pop('created_on')
            # pm_function.pop('subassembly_instance_id')
            # pm_function.pop('updated_on')
            # pm_function.pop('gauge_group_id')
            # etl_gauges = pm_function.pop('etl_gauges',None)
            # if 'unit_translation_id' in pm_function:
            #     pm_function.pop('unit_translation_id')
            # pm_function.pop('subassembly_instance_name')
            # if 'config' in pm_function:
            #     pm_function.pop('config')
            # trend_meta_data = pm_function.pop('trend_meta_data') if 'trend_meta_data' in pm_function else None
            # if 'gauge_group' in pm_function:  # This needs to get restructured..
            #     gauge = pm_function.pop('gauge_group')
            #     gg = seed_gauge_group_func(gauge)
            #     pm_function['gauge_group_id'] = gg.id
            # print("Creating pm function.")
            # gauge_ranges = pm_function.pop('gauge_ranges') if 'gauge_ranges' in pm_function else None
            # pmf = PmFunction.create(**pm_function)
            # print("Created pm function")
            # tm_map = {}
            # if trend_meta_data:
            #     print(" -------------------- Adding trend meta data to pmf")
            #     for trend_meta in trend_meta_data:
            #         tm = TrendMetaData.create(**{
            #             'name': trend_meta.get('name'),
            #             'display_name': trend_meta.get('display_name'),
            #             'color': trend_meta.get('color'),
            #             'units': trend_meta.get('units'),
            #             'mks_units': trend_meta.get('mks_units'),
            #             'fps_units': trend_meta.get('fps_units'),

            #         })
            #         pmf.trend_meta_data.append(tm)
            #         tm_map[tm.name] = tm
            # if gauge_ranges:
            #     for gauge_range in gauge_ranges:
            #         gauge_range.pop("id")
            #         tm_item = tm_map.get(gauge_range['trend_meta_data_name'])
            #         if tm_item:
            #             gr = GaugeRange(**gauge_range, trend_meta_data = tm_item)
            #             pmf.gauge_ranges.append(gr)
            # subassembly.pm_functions_dict[pmf.name] = pmf
    db.session.commit()

def seed_gauge_types(gaugeTypes):
    for gaugeType in gaugeTypes:
        GaugeType.firstOrCreate(**gaugeType)
    db.session.commit()

def gauge_type_seeder():
    print("GaugeTypeSeeder opening: ",gaugeTypeFile)
    with open(gaugeTypeFile) as file_json:
        gaugeTypes = json.loads(file_json.read())
    seed_gauge_types(gaugeTypes)

def subassembly_seeder(subassemblies,assemblies):
    for subassembly in subassemblies:
        if 'id' in subassembly:
            subassembly.pop('id')
        if '_id' in subassembly:
            subassembly.pop('_id')
        pmfuncs = subassembly.pop('pm_functions')
        pm_func = seed_pm_func(pmfuncs)
        # subassembly['pm_functions'] = pm_func
        print("subassembly----: ",subassembly)
        # s = SubassemblyInstance.create(**subassembly)
        s = SubassemblyInstance.where(name = subassembly['name']).first()
        if s is None:
            s = SubassemblyInstance.create(**subassembly)
        # s = SubassemblyInstance.firstOrCreate(**subassembly)
        # s.pm_functions = pm_func
        s.pm_functions.extend(pm_func)
    db.session.commit()
    
    print("Making assemblies")
    for ass in assemblies:
        print("Making assembly: ",ass)
        print("name is: ",ass['name'])
        assembly = Assembly.where(name = ass['name']).first()
        if assembly is None:
            a = Assembly.create(**{
                'name':ass['name'],
                'display_name': ass.get('dispaly_name',ass['name']),
                'image': ass.get('image')
            })
            for subass in ass.get('subassemblies',{}):
                sub = SubassemblyInstance.query.filter(SubassemblyInstance.name==subass).first()
                if sub:
                    print("Sub is: ",sub._to_dict());
                else:
                    print("Couldn't find sub: ",subass)
                a.subassemblies.append(sub)
            db.session.commit()
    print("---------- End assemblies ----------------- ")


def seed_gauge_group_func(gg_values):
    gauge_group = GaugeGroup.firstOrCreate(**gg_values)
    return gauge_group

def seed_pm_func(pm_funcs):
    print("Seeding pm_func")

    pms = []
    for pm_func in pm_funcs:
        print(pm_func)
        if 'id' in pm_func:
            pm_func.pop('id')
        if '_id' in pm_func:
            pm_func.pop('_id')
        pm_func_config = pm_func.get('config',{})
        if 'config' in pm_func:
            pm_func.pop('config')
        print("******************** instance id check **************************")
        if 'subassembly_instance_id' in pm_func:
            pm_func.pop('subassembly_instance_id')
        if 'subassembly_instance_name' in pm_func:
            pm_func.pop('subassembly_instance_name')
        
        trend_metas = pm_func.pop('trend_meta',None)
        if trend_metas is None:
            trend_metas = pm_func.pop('trendMetaData',None)
        print("Pm Function: ",pm_func)
        if trend_metas:
            trends=[]
            for trend_meta in trend_metas:
                if 'id' in trend_meta:
                    trend_meta.pop('id')
                if 'pm_func_id' in trend_meta:
                    trend_meta.pop('pm_func_id')
                print("Trend Meta: ",trend_meta)
                trends.append(TrendMetaData(**trend_meta))
                #p.trend_meta.append(TrendMetaData(**trend_meta))
            pm_func['trend_meta'] = trends
        if 'gauge_group' in pm_func:
            gauge = pm_func.pop('gauge_group')
            gg = seed_gauge_group_func(gauge)
            pm_func['gauge_group_id'] = gg.id
        if pm_func_config != {}:
            if 'yTrendUnit' in pm_func_config:
                pm_func['units'] = pm_func_config.get('yTrendUnit')
                pm_func['trend_existence'] = True
            if 'xUNIT' in pm_func_config:
                pm_func['x_unit'] = pm_func_config['xUNIT'].get('display_name')
            if 'yUNIT' in pm_func_config:
                pm_func['y_unit'] = pm_func_config['yUNIT'].get('display_name')

            pm_func['gauge_base_image'] = pm_func_config.get('gaugeCode')
            pm_func['gauge_type'] = pm_func_config.get('type')
        print(pm_func)
        p = PmFunction.create(**pm_func)
        pms.append(p)
    return pms
