from mdhwebserver.models.models import DeviceUnitDataPoint, DeviceDataItem,TrendMetaData, PmFunction,DeviceDataItemUnitDataPointAssociation
import sqlalchemy as sa
import sqlalchemy.orm as orm


def update_device_data_items_kpis(ddi):
    new_kpis = ddi.kpis.copy() if ddi.kpis is not None else {}
    for ap in ddi.device_unit_data_point_associations:
        unit_data=DeviceUnitDataPoint.where(gauge_id=ap.gauge_id,trend_meta_data_id=ap.trend_meta_data_id) \
            .options(
                orm.with_expression(DeviceUnitDataPoint.mks, sa.func.unit_at_double(DeviceUnitDataPoint.value,sa.sql.text(f"'{ap.trend_meta_data.units}'")))
            ).order_by(sa.desc(DeviceUnitDataPoint._timestamp)) \
            .first()
        print("--- UnitData: ",unit_data.toJson())
        new_kpis[ap.trend_meta_data.name] = unit_data.mks
    ddi.kpis = new_kpis

def get_virtual_gauge_kpis2(vg,gid,ddi):
    # tmids=[]
    # for tm in vg.trend_meta_data:
    #     tmids.append(tm.id)
    # tms=TrendMetaData.query.filter(TrendMetaData.id.in_(tmids))
    # print("--------------- tmids: ",tmids)
    # print("------- Gauge id: ", gid)
    new_kpis = ddi.kpis.copy() if ddi.kpis is not None else {}
    for tm in vg.trend_meta_data:
        if tm not in ddi.trend_meta_datas:
            ddi.device_unit_data_point_associations.append(DeviceDataItemUnitDataPointAssociation(**{
                'gauge_id': gid,
                'trend_meta_data_id': tm.id
            }))
        unit_data=DeviceUnitDataPoint.where(gauge_id=gid,trend_meta_data_id=tm.id) \
            .options(
                orm.with_expression(DeviceUnitDataPoint.mks, sa.func.unit_at_double(DeviceUnitDataPoint.value,sa.sql.text(f"'{tm.units}'")))
            ).order_by(sa.desc(DeviceUnitDataPoint._timestamp)) \
            .first()
        # print("--- UnitData: ",unit_data.toJson())
        if unit_data:
            new_kpis[tm.name] = unit_data.mks
        else:
            new_kpis[tm.name] = None
        new_kpis[tm.name] = unit_data.mks
    print("------ KPIS: ",new_kpis)
    return new_kpis


def make_virtual_gauge_kpis(G,virtual_pm_function):
    ddi=DeviceDataItem.where(gauge_id=G.id, pm_func_id=virtual_pm_function.id).first()
    if ddi==None:
        print("No DDI")
        ddi=DeviceDataItem.create(**{
            'gauge_id': G.id,
            'pm_func_id': virtual_pm_function.id
        })
        ddi.pm_function_name = ddi.pm_function.name
        ddi.subassembly_instance_id = ddi.pm_function.subassembly_instance_id
        ddi.subassembly_instance_name = ddi.pm_function.subassembly_instance_name
        G.device_data_items.append(ddi)
    # pm = PmFunction.query.filter(PmFunction.id==virtual_pm_function.id)
    # ddi.kpis = get_virtual_gauge_kpis(pm.first(),G.id)
    ddi.kpis = get_virtual_gauge_kpis2(virtual_pm_function,G.id,ddi)
    print("XXXXXXXXXXXx ddi kpis: ", ddi.kpis)
    return ddi

def update_virtual_gauges(G,virtual_pm_functions_set):
    for vpmf in virtual_pm_functions_set:
        make_virtual_gauge_kpis(G,vpmf)


def get_virtual_pm_functions(trends):
    tmids = set()
    for tm in trends:
        tmids.add(tm.id)

    virtual_functions = TrendMetaData.filter(TrendMetaData.id.in_(tmids)).options(
        orm.joinedload(TrendMetaData.pm_functions.and_(PmFunction.gauge_style == "virtual"))
    ).all()

    return virtual_functions

def update_trend_kpis(tms):
    for tm in tms:
        for ddi in tm.device_data_items:
            if ddi.pm_function.gauge_style == 'abstract':
                print("Update abstract gauge: ", ddi.pm_function.name)
                update_device_data_items_kpis(ddi)


def update_virtual_gauges_from_trends(G,tms):
    print("tms is : ", tms)
    virtual_functions = get_virtual_pm_functions(tms)
    print("XXXXXXXXXX Virtual_functons: ",virtual_functions)
    virtual_pm_functions_set=set()
    for vf in virtual_functions:
        if vf.pm_functions:
            # virtual_pm_functions_set.add(set(vf.pm_functions))
            virtual_pm_functions_set |= set(vf.pm_functions)
    print("Virtual pm functions: ", virtual_pm_functions_set)
    update_virtual_gauges(G,virtual_pm_functions_set)
    update_trend_kpis(tms)
    return virtual_functions
