"""empty message

Revision ID: b3896a553a31
Revises: d7e4baef6fa6
Create Date: 2023-10-17 13:32:11.439137

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql
from mdhwebserver.models.models import TrendMetaData,PmFunction

# revision identifiers, used by Alembic.
revision = 'b3896a553a31'
down_revision = 'd7e4baef6fa6'
branch_labels = None
depends_on = None


missing_trends = {
    'Vibration (Amplitude)': {
        'name': 'trend',
        'display_name': 'trend',
        'color': 'blue'
    },
    'Sensor Installation': {
        'name': 'trend',
        'display_name': 'trend',
        'color': 'blue'
    },
    'Humidity ( 10 minutes )': {
        'name': 'trend',
        'display_name': 'trend',
        'color': 'blue'
    }
}

def fix_missing_trend_meta_data():
    global missing_trends

    for pm_func_name, trend in missing_trends.items():
        pms = PmFunction.where(name=pm_func_name)
        for pm in pms:
            print("pm name: ",pm.name)
            pm.trend_meta_data.append(TrendMetaData(**{
                'name': 'trend',
                'display_name': 'trend',
                'color': 'blue'
            }))
            pm.save()

def remove_missing_trend_meta_data():
    for pm_func_name, trend in missing_trends.items():
        pms = PmFunction.where(name=pm_func_name)
        for pm in pms:
            for tmd in pm.trend_meta_data:
                tmd.delete()
            pm.save()



def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('graph_configs',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('pm_function_id', sa.Integer(), nullable=True),
    sa.Column('unit_system', sa.String(length=4), nullable=True),
    sa.Column('y_max', sa.Float(), nullable=True),
    sa.Column('y_min', sa.Float(), nullable=True),
    sa.Column('y_unit', sa.String(), nullable=True),
    sa.Column('x_unit', sa.String(), nullable=True),
    sa.Column('yellow_threshold', sa.Float(), nullable=True),
    sa.Column('yellow_threshold_text', sa.String(), nullable=True),
    sa.Column('red_threshold', sa.Float(), nullable=True),
    sa.Column('red_threshold_text', sa.String(), nullable=True),
    sa.Column('created_on', sa.TIMESTAMP(), nullable=False),
    sa.ForeignKeyConstraint(['pm_function_id'], ['pm_functions.id'], ),
    sa.PrimaryKeyConstraint('id')
    )
    with op.batch_alter_table('device_data_items', schema=None) as batch_op:
        batch_op.add_column(sa.Column('colors', sa.ARRAY(sa.String(length=20)), nullable=True))
        batch_op.add_column(sa.Column('values', sa.ARRAY(sa.Float()), nullable=True))
        batch_op.add_column(sa.Column('reason', sa.String(length=256), nullable=True))
        batch_op.add_column(sa.Column('activate_hourglass', sa.Boolean(), nullable=True))
        batch_op.add_column(sa.Column('feedback', sa.Boolean(), nullable=True))
        batch_op.add_column(sa.Column('user_input', sa.Boolean(), nullable=True))
        batch_op.add_column(sa.Column('hide_from_group', sa.Boolean(), nullable=True))
        batch_op.add_column(sa.Column('message', sa.String(length=256), nullable=True))
        batch_op.add_column(sa.Column('yellow_threshold', sa.Float(), nullable=True))
        batch_op.add_column(sa.Column('red_threshold', sa.Float(), nullable=True))
        batch_op.add_column(sa.Column('kpis', postgresql.JSONB(astext_type=sa.Text()), nullable=True))

    with op.batch_alter_table('pm_functions', schema=None) as batch_op:
        batch_op.add_column(sa.Column('trend_existence', sa.Boolean(), nullable=True))
        batch_op.add_column(sa.Column('mro', sa.Boolean(), nullable=True))
        batch_op.add_column(sa.Column('gauge_type', sa.String(length=20), nullable=True))
        batch_op.add_column(sa.Column('gauge_base_image', sa.String(length=256), nullable=True))
        batch_op.add_column(sa.Column('unit_system', sa.String(length=4), nullable=True))
        batch_op.add_column(sa.Column('units', sa.String(length=20), nullable=True))

    with op.batch_alter_table('users', schema=None) as batch_op:
        batch_op.alter_column('is_admin',
               existing_type=sa.BOOLEAN(),
               nullable=False,
               existing_server_default=sa.text('false'))

    # ### end Alembic commands ###

    op.execute("update device_data_items set colors=replace(replace(data->>'color','[','{'),']','}')::text[]")
    op.execute("update pm_functions \
        set gauge_type=config->>'type'"
    )
    op.execute("update pm_functions as pmf \
             set mro=subquery.mro::boolean, trend_existence=subquery.trend_existence::boolean, units=subquery.unit  \
             from (SELECT \
             pm_func_id,data->>'mro'::text as mro, data->>'trendExistence'::text as trend_existence, data->>'unit'::text as unit \
             from device_data_items as ddi) AS subquery where subquery.pm_func_id=pmf.id" \
    )

    op.execute("update pm_functions set units=config->>'yTrendUnit', gauge_base_image=config->>'gaugeCode'")
    op.execute("update pm_functions set unit_system='fps' where units='fahrenheit'")
    op.execute("insert into graph_configs(pm_function_id,unit_system,y_unit,x_unit,created_on) select id as pm_function_id, unit_system, units as y_unit, units as x_unit, now() as created_on from pm_functions")
    op.execute("update graph_configs as gc \
      set y_max=t2.y_max::float, y_min = t2.y_min::float, \
      yellow_threshold = t2.yellow_threshold::float, \
      yellow_threshold_text = t2.yellow_threshold_text, \
      red_threshold = t2.red_threshold::float, \
      red_threshold_text = t2.red_threshold_text \
      from (select \
      pm_func_id,data->'mart'->>'trendYMax' as y_max, data->'mart'->>'trendYMin' as y_min, \
      data->'mart'->>'yellow_threshold' as yellow_threshold, \
      data->'mart'->>'yellow_threshold_text' as yellow_threshold_text, \
      data->'mart'->>'red_threshold' as red_threshold, \
      data->'mart'->>'red_threshold_text' as red_threshold_text \
      from device_data_items \
      where id in (select id from (select pm_func_id,min(id) as id from device_data_items group by pm_func_id) as t1)) as t2 \
      where gc.pm_function_id = t2.pm_func_id"
    )

    #fix_missing_trend_meta_data()



def downgrade():

    #remove_missing_trend_meta_data()

    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table('users', schema=None) as batch_op:
        batch_op.alter_column('is_admin',
               existing_type=sa.BOOLEAN(),
               nullable=True,
               existing_server_default=sa.text('false'))

    with op.batch_alter_table('pm_functions', schema=None) as batch_op:
        batch_op.drop_column('units')
        batch_op.drop_column('unit_system')
        batch_op.drop_column('gauge_base_image')
        batch_op.drop_column('gauge_type')
        batch_op.drop_column('mro')
        batch_op.drop_column('trend_existence')

    with op.batch_alter_table('device_data_items', schema=None) as batch_op:
        batch_op.drop_column('kpis')
        batch_op.drop_column('red_threshold')
        batch_op.drop_column('yellow_threshold')
        batch_op.drop_column('message')
        batch_op.drop_column('hide_from_group')
        batch_op.drop_column('user_input')
        batch_op.drop_column('feedback')
        batch_op.drop_column('activate_hourglass')
        batch_op.drop_column('reason')
        batch_op.drop_column('values')
        batch_op.drop_column('colors')

    op.drop_table('graph_configs')
    # ### end Alembic commands ###
