"""add gauge_group

Revision ID: d7e4baef6fa6
Revises: 5479ea45eb46
Create Date: 2023-07-24 14:54:54.680982

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.sql import text
import datetime


# revision identifiers, used by Alembic.
revision = 'd7e4baef6fa6'
down_revision = '5479ea45eb46'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('gauge_groups',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('created_on', sa.TIMESTAMP(), nullable=True, default=datetime.datetime.utcnow),
    sa.Column('name', sa.String(length=128), nullable=False),
    sa.Column('display_name', sa.String(length=128), nullable=False),
    sa.PrimaryKeyConstraint('id')
    )
    with op.batch_alter_table('device_data_items', schema=None) as batch_op:
        batch_op.add_column(sa.Column('gauge_group_id', sa.Integer(), nullable=True))
        batch_op.add_column(sa.Column('gauge_group_name', sa.String(length=128), nullable=True))
        batch_op.create_foreign_key(None, 'gauge_groups', ['gauge_group_id'], ['id'])

    with op.batch_alter_table('pm_functions', schema=None) as batch_op:
        batch_op.add_column(sa.Column('gauge_group_id', sa.Integer(), nullable=True))
        batch_op.create_foreign_key(None, 'gauge_groups', ['gauge_group_id'], ['id'])

    groups = [
        {'id': 1, 'name': 'PreventativeMaintenance',
        'display_name': 'Electrical Panel1: Preventative Maintenance'},
        {'id': 2, 'name': 'Utility', 'display_name': 'Electrical Panel1: Utility'},
        {'id': 3, 'name': 'PredictiveMaintenance', 'display_name': 'Electrical Panel1: Predictive Maintenance'},
        {'id': 4, 'name': 'PowerMeter', 'display_name': 'Electrical Panel1: Power Meter'},
        {'id': 5, 'name': 'PowerMeter', 'display_name': 'Electrical Panel2: Power Meter'},
        {'id': 6, 'name': 'PreventativeMaintenance', 'display_name': 'Electrical Panel2: Preventative Maintenance'},
        {'id': 7, 'name': 'Utility', 'display_name': 'Electrical Panel2: Utility'},
        {'id': 8, 'name': 'PredictiveMaintenance', 'display_name': 'Electrical Panel2: Predictive Maintenance'},
        {'id': 9, 'name': 'PredictiveMaintenance', 'display_name': 'Electrical Panel3: Predictive Maintenance'},
        {'id': 10, 'name': 'Utility', 'display_name': 'Electrical Panel3: Utility'},
        {'id': 11, 'name': 'PreventiveMaintenance', 'display_name': 'Electrical Panel3: Preventative Maintenance'},
        {'id': 12, 'name': 'PowerMeter', 'display_name': 'Electrical Panel3: Power Meter'},
        {'id': 13, 'name': 'PredictiveMaintenance', 'display_name': 'Electrical Panel4: Predictive Maintenance'},
        {'id': 14, 'name': 'Utility', 'display_name': 'Electrical Panel4: Utility'},
        {'id': 15, 'name': 'PreventativeMaintenance', 'display_name': 'Electrical Panel4: Preventative Maintenance'},
        {'id': 16, 'name': 'PowerMeter', 'display_name': 'Electrical Panel4: Power Meter'}
    ]

    conn = op.get_bind()
    for group in groups:
        conn.execute(text("INSERT INTO gauge_groups (id, name, display_name) VALUES (:id, :name, :display_name)"), group)

    group_mappings = {}
    group_mappings['energySense1'] = {
        'VRMS': 4,
        'IRMS': 4,
        'Active Power': 4,
        'Power Factor': 4,
        'Voltage Harmonic Distortion': 4,
        'Current Harmonic Distortion': 4,
        'Sag': 1,
        'Swell': 1,
        'Usage': 2,
        'Current Imbalance': 1,
        'Voltage Imbalance': 1,
        'Over Voltage': 1,
        'Under Voltage': 1,
        'Frequency Variation': 1,
        'Ground Condition': 1
    }
    group_mappings['energySense2'] = {
        'VRMS': 5,
        'IRMS': 5,
        'Active Power': 5,
        'Power Factor': 5,
        'Voltage Harmonic Distortion': 5,
        'Sag': 6,
        'Current Harmonic Distortion': 5,
        'Swell': 6,
        'Usage': 7,
        'Current Imbalance': 6,
        'Voltage Imbalance': 6,
        'Over Voltage': 6,
        'Under Voltage': 6,
        'Frequency Variation': 6,
        'Ground Condition': 6
    }
    group_mappings['energySense3'] = {
        'VRMS': 12,
        'IRMS': 12,
        'Active Power': 12,
        'Power Factor': 12,
        'Voltage Harmonic Distortion': 12,
        'Current Harmonic Distortion': 12,
        'Sag': 11,
        'Swell': 11,
        'Usage': 10,
        'Current Imbalance': 11,
        'Voltage Imbalance': 11,
        'Over Voltage': 11,
        'Frequency Variation': 11,
        'Ground Condition': 11
    }
    group_mappings['energySense4'] = {
        'VRMS': 16,
        'IRMS': 16,
        'Active Power': 16,
        'Power Factor': 16,
        'Voltage Harmonic Distortion': 16,
        'Current Harmonic Distortion': 16,
        'Sag': 15,
        'Swell': 15,
        'Usage': 14,
        'Current Imbalance': 15,
        'Voltage Imbalance': 15,
        'Over Voltage': 15,
        'Under Voltage': 15,
        'Frequency Variation': 15,
        'Ground Condition': 15
    }
    group_mappings['heaterSense1'] = {
        'Over Voltage': 1,
        'Frequency Variation': 1,
        'Machine Utilization': 2,
        'Swell': 1,
        'Current Harmonic Distortion': 4,
        'Voltage Harmonic Distortion': 4,
        'Ground Condition': 1,
        'Current Imbalance': 1,
        'Voltage Imbalance': 1,
        'Sag': 1,
        'Under Voltage': 1,
        'Heater Failure': 3,
        'VRMS': 4,
        'IRMS': 4,
        'Active Power': 4,
        'Power Factor': 4
    }
    group_mappings['heaterSense2'] = {
        'Frequency Variation': 6,
        'Swell': 6,
        'Voltage Harmonic Distortion': 5,
        'Over Voltage': 6,
        'Under Voltage': 6,
        'Sag': 6,
        'Heater Failure': 8,
        'Machine Utilization': 7,
        'Ground Condition': 6,
        'Current Imbalance': 6,
        'Current Harmonic Distortion': 5,
        'Voltage Imbalance': 6,
        'IRMS': 5,
        'VRMS': 5,
        'Power Factor': 5
    }
    group_mappings['heaterSense3'] = {
        'Swell': 11,
        'Frequency Variation': 11,
        'Voltage Harmonic Distortion': 12,
        'Over Voltage': 11,
        'Under Voltage': 11,
        'Sag': 11,
        'Heater Failure': 9,
        'Machine Utilization': 10,
        'Ground Condition': 11,
        'Current Imbalance': 11,
        'Current Harmonic Distortion': 9,
        'Voltage Imbalance': 11,
        'IRMS': 12,
        'VRMS': 12,
        'Power Factor': 12,
        'Active Power': 12
    }
    group_mappings['heaterSense4'] = {
        'Ground Condition': 15,
        'Machine Utilization': 14,
        'Frequency Variation': 15,
        'Swell': 15,
        'Voltage Harmonic Distortion': 16,
        'Over Voltage': 15,
        'Under Voltage': 15,
        'Sag': 15,
        'Heater Failure': 13,
        'Current Imbalance': 15,
        'Current Harmonic Distortion': 16,
        'Voltage Imbalance': 15,
        'IRMS': 16,
        'VRMS': 16,
        'Power Factor': 16,
        'Active Power': 16
    }
    group_mappings['motorSense1'] = {
        'Sag': 1,
        'Swell': 1,
        'Current Imbalance': 1,
        'Voltage Imbalance': 1,
        'Machine Utilization': 2,
        'Utilization Under Load': 2,
        'Motor Bearing Failure': 3,
        'Stator Winding Failure': 3,
        'VRMS': 4,
        'IRMS': 4,
        'Power Factor': 4,
        'Active Power': 4,
        'Voltage Harmonic Distortion': 4,
        'Current Harmonic Distortion': 4,
        'Over Voltage': 1,
        'Under Voltage': 1,
        'Frequency Variation': 1,
        'Ground Condition': 1
    }
    group_mappings['motorSense2'] = {
        'Motor Bearing Failure': 8,
        'Stator Winding Failure': 8,
        'Utilization Under Load': 7,
        'Voltage Imbalance': 6,
        'Current Harmonic Distortion': 5,
        'Frequency Variation': 6,
        'Current Imbalance': 6,
        'Sag': 6,
        'Ground Condition': 6,
        'Swell': 6,
        'Over Voltage': 6,
        'Machine Utilization': 7,
        'Voltage Harmonic Distortion': 5,
        'Under Voltage': 6,
        'VRMS': 5,
        'IRMS': 5,
        'Power Factor': 5,
        'Active Power': 5
    }
    group_mappings['motorSense3'] = {
        'Motor Bearing Failure': 9,
        'Stator Winding Failure': 9,
        'Utilization Under Load': 10,
        'Voltage Imbalance': 11,
        'Current Harmonic Distortion': 12,
        'Frequency Variation': 11,
        'Current Imbalance': 11,
        'Sag': 11,
        'Ground Condition': 11,
        'Swell': 11,
        'Over Voltage': 11,
        'Machine Utilization': 10,
        'Voltage Harmonic Distortion': 12,
        'Under Voltage': 11,
        'VRMS': 12,
        'IRMS': 12,
        'Power Factor': 12,
        'Active Power': 12
    }
    group_mappings['motorSense4'] = {
        'Motor Bearing Failure': 13,
        'Stator Winding Failure': 13,
        'Utilization Under Load': 14,
        'Voltage Imbalance': 15,
        'Current Harmonic Distortion': 16,
        'Frequency Variation': 15,
        'Current Imbalance': 15,
        'Sag': 15,
        'Ground Condition': 15,
        'Swell': 15,
        'Over Voltage': 15,
        'Machine Utilization': 14,
        'Voltage Harmonic Distortion': 16,
        'Under Voltage': 15,
        'VRMS': 16,
        'IRMS': 16,
        'Power Factor': 16,
        'Active Power': 16
    }

    #iterate over group_mappings, do a query to get the id from subassembly_instances where subassembly_instances.name equals the group_mappings keys.
    
    for group_mapping in group_mappings:
        res = conn.execute(text("SELECT id FROM subassembly_instances WHERE name = :name"), {'name':group_mapping})
        record = res.fetchone()
        # print(record)
        if len(record) > 0:
            sub_id = record[0]
            for pm_function in group_mappings[group_mapping]:
                conn.execute(text("UPDATE pm_functions set gauge_group_id = :gauge_group_id WHERE name = :name and subassembly_instance_id = :sub_id"), {'name': pm_function, 'gauge_group_id': group_mappings[group_mapping][pm_function], 'sub_id': sub_id})

                conn.execute(text("update device_data_items set gauge_group_id = t1.gauge_group_id, gauge_group_name = t1.gauge_group_name from (select pm.id as pm_func_id, gg.id as gauge_group_id, gg.name as gauge_group_name from pm_functions pm join gauge_groups gg on pm.gauge_group_id = gg.id) t1 where device_data_items.pm_func_id = t1.pm_func_id;"))

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table('pm_functions', schema=None) as batch_op:
        batch_op.drop_constraint('pm_functions_gauge_group_id_fkey', type_='foreignkey')
        # batch_op.drop_constraint(None, type_='foreignkey')
        batch_op.drop_column('gauge_group_id')

    with op.batch_alter_table('device_data_items', schema=None) as batch_op:
        batch_op.drop_constraint('device_data_items_gauge_group_id_fkey', type_='foreignkey')
        batch_op.drop_column('gauge_group_name')
        batch_op.drop_column('gauge_group_id')

    op.drop_table('gauge_groups')
    # ### end Alembic commands ###
